//  UserSession.m
//  PatientViewer
//
//  Created by Grant Oladipo on 3/1/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxx

#import "Alerter.h"
#import "Connection.h"
#import "LabTest.h"
#import "Patient.h"
#import "UserSession.h"
#import "Medication.h"
#import "LabResult.h"

@implementation UserSession

@synthesize sessionExpireNotifcation;



-(id)noPropertyValue {
	return nil;
}

+(void)reset {
	
	UserSession *currentUserSession = [self currentUserSession];

	[currentUserSession removeAllProperties];
	[currentUserSession removeAllLinks];
	
	[currentUserSession setPropertyValue:[NSString stringWithFormat:@"%f", UserSessionNoTimeOutSetValue] 
								  forKey:UserSessionClientTimeoutKey];
}

-(void)wipeData {

	[self removeAllProperties];
	[self removeAllLinks];
	
	[self setPropertyValue:[NSString stringWithFormat:@"%f", UserSessionHasTimedOutValue]
					forKey:UserSessionClientTimeoutKey];
}

+(NSString *)clientTimeout {
	return [[self currentUserSession] propertyValueForKey:UserSessionClientTimeoutKey];
}

+(BOOL)hasTimedOut:(NSDate *)currentTime {
	
	BOOL result = NO;

	CGFloat timeOut = [[self clientTimeout] floatValue];
	CGFloat timeElapsed = [currentTime timeIntervalSinceDate:[[self currentUserSession] propertyValueForKey:UserSessionTimeStampKey]]/SixtySeconds;
	
	if (timeOut == UserSessionHasTimedOutValue || 
		(timeOut != UserSessionNoTimeOutSetValue && timeElapsed > timeOut) ) {
		
		result = YES;
	}
	
	return result;
}

+(UILocalNotification *)createLocalNotification {
	
	UILocalNotification *notification = [[UILocalNotification alloc] init];
	notification.timeZone = [NSTimeZone defaultTimeZone];
	notification.alertBody = @"Your Patient Viewer session will expire in 5 minutes.";
	notification.alertAction = @"Renew";
	notification.soundName = UILocalNotificationDefaultSoundName;
	
	return [notification autorelease];
}

+(void)setTimeStamp:(NSDate *)currentTime {
	
	UserSession *session = [self currentUserSession];
	
	if (session.sessionExpireNotifcation != nil) {
		[[UIApplication sharedApplication] cancelLocalNotification:session.sessionExpireNotifcation];
	}
	
	[session setPropertyValue:currentTime forKey:UserSessionTimeStampKey];
	
	CGFloat timeOut = [[self clientTimeout] floatValue];
	
	if (timeOut != UserSessionNoTimeOutSetValue) {
	
		session.sessionExpireNotifcation = [self createLocalNotification];	
        

		session.sessionExpireNotifcation.fireDate = [currentTime dateByAddingTimeInterval:SixtySeconds*(timeOut-5)];
        
		[[UIApplication sharedApplication] scheduleLocalNotification:session.sessionExpireNotifcation];
	}
}

+(BOOL)enforceTimeout {
	
	NSDate *currentTime = [NSDate date];

	BOOL result = [self hasTimedOut:currentTime];
    
	if (result) {
		
		[[NSNotificationCenter defaultCenter] postNotificationName:ConnectionShouldWipeData object:nil userInfo:nil];
		
		[Alerter enqueueAlertViewWithTitle:@"User Session Timed Out" message:@"Please login again"];
		
	} else {
		[UserSession setTimeStamp:currentTime];
	}
	
	return result;
}


#pragma mark - Notification
-(void)processShouldWipeData:(NSNotification *)notification {
	
    [self wipeData];
}

-(void)registerForNotifications {
	
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter addObserver:self selector:@selector(processShouldWipeData:) name:ConnectionShouldWipeData object:nil];
}

-(void)unregisterFromNotifications {
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}


#pragma mark - Initialization
-(id)init {
	
	self = [super init];
	
	if (self) {
		[self registerForNotifications];
		[self setPropertyValue:[NSString stringWithFormat:@"%f", UserSessionNoTimeOutSetValue] 
									  forKey:UserSessionClientTimeoutKey];
	}
	
	return self;
}

+(id)currentUserSession {
	
	static UserSession *currentUserSession = nil;
	
	if (!currentUserSession) {
		currentUserSession = [[UserSession alloc] init];
	}
	
	return currentUserSession;
}

-(NSString *)generateUUIDString {

	CFUUIDRef uuid = CFUUIDCreate(kCFAllocatorDefault);
	
	NSString *uuidString = (NSString *)CFUUIDCreateString(kCFAllocatorDefault, uuid);
	
	[uuidString autorelease];

	CFRelease(uuid);
	
	return uuidString;
}

-(id)refreshWithAttributeDictionary:(NSDictionary *)attributeDict {

	[super refreshWithAttributeDictionary:attributeDict];
	
	[self setPropertyValue:[self generateUUIDString] forKey:UserSessionSessionIdKey];
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[self unregisterFromNotifications];
	
	[sessionExpireNotifcation release], sessionExpireNotifcation = nil;
	
	[super dealloc];
}


#pragma mark - Patient
+(void)removeSelectedPatient {
	[self setSelectedPatient:nil];
}

+(Patient *)selectedPatient {
	return [[self currentUserSession] propertyValueForKey:UserSessionSelectedPatientKey];
}

+(void)setSelectedPatient:(Patient *)aPatient {
		
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter postNotificationName:UserSessionWillChangePatient object:self userInfo:nil];

	[[self currentUserSession] setPropertyValue:aPatient forKey:UserSessionSelectedPatientKey];
	
	[defaultCenter postNotificationName:UserSessionDidChangePatient object:nil userInfo:nil];
}


#pragma mark - Lab Test
+(void)removeSelectedLabTest {
	[self setSelectedLabTest:nil];
}

+(LabTest *)selectedLabTest{
	return [[self currentUserSession] propertyValueForKey:UserSessionSelectedLabTestKey];
}

+(void)setSelectedLabTest:(LabTest *)aLabTest {
	
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter postNotificationName:UserSessionWillChangeLabTest object:self userInfo:nil];
	
	[[self currentUserSession] setPropertyValue:aLabTest forKey:UserSessionSelectedLabTestKey];
	
	[defaultCenter postNotificationName:UserSessionDidChangeLabTest object:nil userInfo:nil];
}


+(void)showMicroResults {
    NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter postNotificationName:UserSessionShowMicroResults object:self userInfo:nil];
	
}

+(void)showChemTest {
    NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter postNotificationName:UserSessionShowChemTest object:self userInfo:nil];    
}


#pragma mark - Lab Result
+(void)removeSelectedLabResult {
	[self setSelectedLabResult:nil];
}

+(LabResult *)selectedLabResult{
	return [[self currentUserSession] propertyValueForKey:UserSessionSelectedLabResultKey];
}

+(void)setSelectedLabResult:(LabResult*)aLabResult {
    NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter postNotificationName:UserSessionWillChangeLabResult object:self userInfo:nil];
	
	[[self currentUserSession] setPropertyValue:aLabResult forKey:UserSessionSelectedLabResultKey];
	
	[defaultCenter postNotificationName:UserSessionDidChangeLabResult object:nil userInfo:nil];
    
}


#pragma mark - Medication
+(NSString *)sessionId {
	return [[self currentUserSession] propertyValueForKey:UserSessionSessionIdKey];
}

+(void)removeSelectedMedication {
	[self setSelectedMedication:nil];
}

+(id)selectedMedication {
	return [[self currentUserSession] propertyValueForKey:UserSessionSelectedMedicationKey];
}

+(void)setSelectedMedication:(Medication *)aMedication {
	
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter postNotificationName:UserSessionWillChangeMedication object:self userInfo:nil];
	
	[[self currentUserSession] setPropertyValue:aMedication forKey:UserSessionSelectedMedicationKey];
	
	[defaultCenter postNotificationName:UserSessionDidChangeMedication object:nil userInfo:nil];

}

+(void)updateSelectedMedicationWithPropertyValue:(id)value forKey:(NSString*)key {
    [[self selectedMedication] setPropertyValue:value forKey:key];
}

+(void)removeCurrentMedications {
	[self setCurrentMedications:nil];
}

+(id)currentMedications {
	return [[self currentUserSession] propertyValueForKey:UserSessionCurrentMedicationsKey];
}

+(void)setCurrentMedications:(NSArray *)theCurrentMedications {
	
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter postNotificationName:UserSessionWillChangeCurrentMedications object:self userInfo:nil];
	
	[[self currentUserSession] setPropertyValue:theCurrentMedications forKey:UserSessionCurrentMedicationsKey];
		
	[defaultCenter postNotificationName:UserSessionDidChangeCurrentMedications object:nil userInfo:nil];
}


#pragma mark - MHPUser
+(MHPUser *)mhpUser {
	return [[self currentUserSession] propertyValueForKey:UserSessionMHPUserKey];
}

+(void)setMHPUser:(MHPUser *)anMHPUser {
	[[self currentUserSession] setPropertyValue:anMHPUser forKey:UserSessionMHPUserKey];
}


#pragma mark - Credential
+(NSURLCredential *)credential {
	return [[self currentUserSession] propertyValueForKey:UserSessionCredentialKey];
}

+(void)setCredential:(NSURLCredential *)aCredential {
	[[self currentUserSession] setPropertyValue:aCredential	forKey:UserSessionCredentialKey];
}

@end