//  Alerter.m
//  PatientViewer
//
//  Created by Grant Oladipo on 5/25/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxx

#import "Alerter.h"
#import "Queue.h"

@implementation Alerter

Queue *alertQueue = nil;
BOOL showingAlert = NO;
UIAlertView *lastAddedAlertView = nil;


#pragma mark - Initialization
-(id)init {
	
	self = [super init];
	
	if (self) {
		alertQueue = [[Queue alloc] init];
		showingAlert = NO;
	}
	
	return self;
}

+(id)sharedAlerter {
	
	static Alerter *sharedAlerter = nil;
	
	if (sharedAlerter == nil) {
		sharedAlerter = [[Alerter alloc] init];
	}
	
	return sharedAlerter;
}

-(void)showNextAlert {
	
    if ([alertQueue isEmpty]) {
        [lastAddedAlertView release], lastAddedAlertView = nil;
    } else {
     
        UIAlertView *alert = [alertQueue pop];
        
        [alert show];
    }
}

+(void)enqueueAlertViewWithTitle:(NSString *)title message:(NSString *)message cancelButtonTitle:(NSString *)cancelButtonTitle {
	
	Alerter *sharedAlerter = [self sharedAlerter];
	
    cancelButtonTitle = (cancelButtonTitle ? cancelButtonTitle : @"Ok");
    
    UIAlertView *alertView = [[[UIAlertView alloc] initWithTitle:title 
                                                        message:message 
                                                       delegate:sharedAlerter 
                                              cancelButtonTitle:cancelButtonTitle 
                                              otherButtonTitles:nil] autorelease];
	
    if ([alertView isEqualToAlertView:lastAddedAlertView] == NO) {
        
        [alertQueue push:alertView];
        
        [lastAddedAlertView release];
        lastAddedAlertView = [alertView retain];
        
        if (showingAlert == NO) {
            [sharedAlerter showNextAlert];
        }
    }
}

+(void)enqueueAlertViewWithTitle:(NSString *)title message:(NSString *)message {
	[self enqueueAlertViewWithTitle:title message:message cancelButtonTitle:nil];
}

+(void)enqueueAlertViewForNoInternetConnection {
    
    NSString *title = @"Unable to connect to the iHealthAdapter Server";
    NSString *message = @"Please check your network settings.";
    
    [self enqueueAlertViewWithTitle:title message:message];
}

+(void)enqueueAlertViewForStatusCode:(NSInteger)statusCode {
    
    NSString *title;

    NSString *message = [[NSString alloc] init];
	
    if (statusCode < 500) {
        title = @"Unable to connect to the iHealthAdapter Server";
		message	= @"Please ensure you are connected to the VA Wifi or to the VPN. ";
    } else {
        title = @"Error occured on MHP Server!";
    }
	
	message = [NSString stringWithFormat:@"%@%@", message, [NSString stringWithFormat:@"HTTP Status %d - %@.", statusCode, [[NSHTTPURLResponse localizedStringForStatusCode:statusCode] capitalizedString]]];
    
    [self enqueueAlertViewWithTitle:title message:message];
}


#pragma mark - Alert View Delegate
-(void)didPresentAlertView:(UIAlertView *)alertView {
	showingAlert = YES;
}

-(void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {

	showingAlert = NO;
    
	[self showNextAlert];
}

@end