//
//  AllergyParser.m
//  PatientViewer
//
//  Created by Grant Oladipo on 5/4/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.

#import "Allergy.h"
#import "AllergyParser.h"
#import "Link.h"
#import "ManagerUtility.h"
#import "PatientMainContainer.h"

@implementation AllergyParser


#pragma mark - Custom Actions
-(void)parseData:(NSData *)data {
	
	[xmlParser release];
	xmlParser = [[NSXMLParser alloc] initWithData:data];
	[xmlParser setDelegate:self];
	[xmlParser setShouldProcessNamespaces:YES];
	
	if (![xmlParser parse]) {
		NSLog(@"***Parsing UNSUCCESSFUL!");
		NSLog(@"Tried to parse:\n%@", [ManagerUtility prettyPrintXML:[[[NSString alloc] initWithData:data encoding:NSStringEncodingConversionAllowLossy] autorelease]]);
	}
}


#pragma mark - Initialization
-(id)initWithDelegate:(id)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = [theDelegate retain];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[delegate release], delegate = nil;
	[xmlParser release], xmlParser = nil;
	
	[super dealloc];
}


#pragma mark - NSXMLParserDelegate Methods
-(void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	
	if ([elementName isEqualToString:@"link"]) {
		[parentElement addLink:myLink];
	} else if ([elementName isEqualToString:@"allergy"]){
		
		parentElement = delegate;
		[delegate addAllergy:allergy];
	} 
}

-(void)parserDidEndDocument:(NSXMLParser *)parser {
	[delegate parserFinishedCreatingResource:self];
}

-(void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {	
	
	if ([elementName isEqualToString:@"link"]) {
		
		[myLink release];
		myLink = [[Link alloc] initWithAttributeDictionary:attributeDict];
		
	} else if ([elementName isEqualToString:@"allergy"]) {
		
		[allergy release];
		allergy = [[Allergy alloc] initWithAttributeDictionary:attributeDict];
		parentElement = allergy;
		
	} else if ([elementName isEqualToString:@"allergies"]) {
		
		parentElement = delegate;
		[delegate setTotalNumberOfAllergies:[[attributeDict objectForKey:@"size"] integerValue]];
	}
}

@end