#import "Alert_Delegate_AppDelegate.h"
#import "Alerter.h"
#import "AppDelegate.h"
#import "ApplicationProperty.h"
#import "Connection.h"
#import "ConnectionFactory.h"
#import "DataLoaderIndicator.h"
#import "MBProgressHUD.h"
#import "UserSession.h"

@implementation AppDelegate

@synthesize window;



+(NSString*)getSourceScheme {   
    
    NSString *scheme;
    
    NSDictionary *appPlist = [[NSBundle mainBundle] infoDictionary];
    NSArray *urlTypes = [appPlist objectForKey:@"CFBundleURLTypes"];
    NSArray *urlSchemes = [[urlTypes objectAtIndex:0] objectForKey:@"CFBundleURLSchemes"];
    
    scheme =  [urlSchemes objectAtIndex:0];
    
    return  scheme;
}

+(void)launchFeedback {
	
	UIApplication *app = [UIApplication sharedApplication];
	
    NSString *sourceScheme = [self getSourceScheme];
    
	NSString *urlString = [NSString stringWithFormat:@"mhp3://?postURL=%@&sourceScheme=%@", [ApplicationProperty feedbackUrlString], sourceScheme];
    
	NSURL *url = [NSURL URLWithString:urlString];
	
	if ([app canOpenURL:url]) {
		[app openURL:url];
	}
}



-(void)showProgressNotifier {
	MBProgressHUD *progressNotifier = [MBProgressHUD showHUDAddedTo:self.window animated:YES];
	progressNotifier.labelText = @"Connecting to server";
}

-(void)hideProgressNotifier {	
	[MBProgressHUD hideHUDForView:self.window animated:YES];
}



- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {    
	
	[[UIApplication sharedApplication] cancelAllLocalNotifications];
	
	[DataLoaderIndicator createIndicator];
	
	[self.window makeKeyAndVisible];
    
	return YES;
}

-(void)applicationDidBecomeActive:(UIApplication *)application {
	
	[ApplicationProperty applicationPropertyReset:YES];
	
	[UserSession enforceTimeout];
	
	[self showProgressNotifier];
	
	[[ConnectionFactory connection] performSelector:@selector(performHealthCheck) withObject:nil afterDelay:1];
}

-(BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation {
	return YES;
}

-(void)application:(UIApplication *)application didReceiveLocalNotification:(UILocalNotification *)notification {
	
	if (application.applicationState == UIApplicationStateActive) {
		
		Alert_Delegate_AppDelegate *delegate = [[Alert_Delegate_AppDelegate alloc] init];
		
		NSString *appName = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleName"];
		
		UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:appName 
															message:notification.alertBody 
														   delegate:delegate 
												  cancelButtonTitle:@"Close" 
												  otherButtonTitles:notification.alertAction, nil];
		[alertView show];
		[alertView release];
	}
}


#pragma mark - Initialization
-(id)init {
	
	self = [super init];
	
	if (self) {
		
		window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
		
		NSDictionary *dictionary = [NSDictionary dictionaryWithObjectsAndKeys:@"1.1", @"version_number_preference", @"https://10.41.252.41:443/MobileHealthPlatformWeb/rest/", @"root_url_preference", @"", @"custom_root_url_preference", @"ASIConnection", @"connection_library_preference", nil];
		
        [[NSUserDefaults standardUserDefaults] registerDefaults:dictionary];
	}
	
	return self;
}




#pragma mark - Memory management
- (void)dealloc {
		
	[window release], window = nil;
	
	[super dealloc];
}

@end