#import "ApplicationProperty.h"

@implementation ApplicationProperty 


#pragma mark - Initialization
-(id)init {
	
	self = [super init];
	
	if (self) {
		pathOfFile = [[[NSBundle bundleForClass:[self class]] pathForResource:@"configuration" ofType:@"plist"] retain];
		configurationProperties = [[NSMutableDictionary alloc] initWithContentsOfFile:pathOfFile];
	}
	
	return self;
}

+(id)applicationPropertyReset:(BOOL)reset {
	
	static ApplicationProperty *defaultApplicationProperty = nil;
	
	if (reset) {
		[defaultApplicationProperty release], defaultApplicationProperty = nil;
	}
	
	if (defaultApplicationProperty == nil) {
		defaultApplicationProperty = [[ApplicationProperty alloc] init];
	}
	
	return defaultApplicationProperty;
}

+(id)applicationProperty {
	
	return [self applicationPropertyReset:NO];
}

+(NSString *)applicationName {

	static NSString *applicationName = nil;
	
	if (applicationName == nil) {
		applicationName = (NSString *)[[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleName"] retain];
	}
	
	return applicationName;
}

+(NSString *)applicationBuildVersion {

	static NSString *applicationBuildVerison = nil;
	
	if (applicationBuildVerison == nil) {
		applicationBuildVerison = (NSString *)[[[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"] retain];
	}
	
	return applicationBuildVerison;
}

-(NSString *)rootUrlString {

	if (rootUrlString) {
		return rootUrlString;
	}

	rootUrlString = [[configurationProperties objectForKey:@"RootURL"] retain];
	
	if (rootUrlString == nil) {
		
		NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
		
		NSString *customUrlString = [defaults objectForKey:@"custom_root_url_preference"];
		
		if (customUrlString && [customUrlString isEqualToString:@""] == NO) {
			rootUrlString = [customUrlString retain];
		} else {
			rootUrlString = [[defaults objectForKey:@"root_url_preference"] retain];
		}	
	}
	
	return rootUrlString;
}

+(NSString *)rootUrlString {
	return [[self applicationProperty] rootUrlString];
}

+(NSString *)auditUrlString {		
	return [[self applicationProperty] auditUrlString];
}

-(NSString *)auditUrlString {		

	if (auditUrlString) {
		return auditUrlString;
	}
	
	NSString *auditPath = [configurationProperties objectForKey:@"AuditURL"];
	auditUrlString = [[NSString stringWithFormat:@"%@%@", [self rootUrlString], auditPath] retain];
	
    return auditUrlString;
}

+(NSString *)healthUrlString {		
	return [[self applicationProperty] healthUrlString];
}

-(NSString *)healthUrlString {
    
	if (healthUrlString) {
		return healthUrlString;
	}
	
	NSString *healthPath = [configurationProperties objectForKey:@"HealthURL"];
	healthUrlString = [[NSString stringWithFormat:@"%@%@", [self rootUrlString], healthPath] retain];
	
	return healthUrlString;
}

-(NSString *)feedbackUrlString {
	
	NSString *feedbackPath = [configurationProperties objectForKey:@"FeedbackURL"];
	NSString *feedbackURLString = [NSString stringWithFormat:@"%@%@", [self rootUrlString], feedbackPath];
	
	return feedbackURLString;
}

+(NSString *)feedbackUrlString {
	return [[self applicationProperty] feedbackUrlString];
}

+(NSString *)loginUrlString {
	return [[self applicationProperty] loginUrlString];
}

-(NSString *)loginUrlString {

	NSString *loginPath = [configurationProperties objectForKey:@"LoginURL"];
	NSString *loginURLString = [NSString stringWithFormat:@"%@%@", [self rootUrlString], loginPath];
	
	return loginURLString;
}

+(NSString *)connectionLibraryString {		
	return [[self applicationProperty] connectionLibraryString];
}

-(NSString *)connectionLibraryString {		
	
	if (connectionLibraryString) {
		return connectionLibraryString;
	}
	
	connectionLibraryString = [[configurationProperties objectForKey:@"ConnectionLibrary"] retain];

	if (connectionLibraryString == nil || [connectionLibraryString isEqualToString:@""]) {
		connectionLibraryString = [@"ASIConnection" retain];
	}

    return connectionLibraryString;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[configurationProperties release], configurationProperties = nil;
	[pathOfFile release], pathOfFile = nil;
	
	[super dealloc];
}

@end