//  DataLoaderIndicator.m
//  PatientViewer
//
//  Created by Grant Oladipo on 4/11/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "AppDelegate.h"
#import "DataLoaderIndicator.h"

#define IndicatorBackgroundColorAlphaComponent .3

@interface DataLoaderIndicator ()
-(void)showIndicator;
@end


@implementation DataLoaderIndicator

@synthesize indicatorView;


#pragma mark - Custom Accessors
+(id)sharedIndicator {
	
	static DataLoaderIndicator *sharedIndicator = nil;
	
	if (!sharedIndicator) {
		sharedIndicator = [[DataLoaderIndicator alloc] init];
	}
	
	return sharedIndicator;
}

-(UIView *)indicatorView {
	
	if (indicatorView) {
		return indicatorView;
	}
	
	indicatorView = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
	indicatorView.backgroundColor = [[UIColor grayColor] colorWithAlphaComponent:IndicatorBackgroundColorAlphaComponent];
	
	dataAIV = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
	
	[indicatorView addSubview:dataAIV];
	
	dataAIV.center = indicatorView.center;
	
	return indicatorView;
}


#pragma mark - Custom Action
+(void)createIndicator {
	[self sharedIndicator];
}

-(void)hideIndicator {
	
	[UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
	[dataAIV stopAnimating];
	[indicatorView removeFromSuperview];
}

-(void)processNotification:(NSNotification *)notification {
	
	if ([[notification name] isEqualToString:ApplicationDidMakeMHPRequest]) {

		if ([requests count] == 0) {
			
			[self showIndicator];
		}
		
		[requests addObject:[notification object]];
		
	} else if ([[notification name] isEqualToString:ApplicationDidFinishProcessingMHPResponse]) {

		[requests removeObject:[notification object]];
		
	}
	
	if ([requests count] == 0) {
		[self hideIndicator];
	}
}

-(void)registerForNotifications {
	
	NSNotificationCenter *defaultNC = [NSNotificationCenter defaultCenter];
	
	[defaultNC addObserver:self selector:@selector(processNotification:) name:ApplicationDidMakeMHPRequest object:nil];
	
	[defaultNC addObserver:self selector:@selector(processNotification:) name:ApplicationDidFinishProcessingMHPResponse object:nil];
}

-(void)showIndicator {
	
	[UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
	[appDelegate.window addSubview:self.indicatorView];
	[dataAIV startAnimating];
}

-(void)unregisterFromNotifications {
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}


#pragma mark - Initialization
-(id)init {
	
	self = [super init];
	
	if (self) {
		appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
		requests = [[NSMutableArray alloc] initWithCapacity:1];
		[self registerForNotifications];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[self unregisterFromNotifications];
	
	[dataAIV release], dataAIV = nil;
	[indicatorView release], indicatorView = nil;
	
	[super dealloc];
}

@end