//  DemographicsParser.m
//  PatientViewer
//
//  Created by Grant Oladipo on 5/4/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxx

#import "Address.h"
#import "Demographics.h"
#import "DemographicsParser.h"
#import "Link.h"
#import "ManagerUtility.h"
#import "NextOfKin.h"

@implementation DemographicsParser


#pragma mark - Custom Actions
-(void)parseData:(NSData *)data {
	
	[xmlParser release];
	xmlParser = [[NSXMLParser alloc] initWithData:data];
	[xmlParser setDelegate:self];
	[xmlParser setShouldProcessNamespaces:YES];
	
	if (![xmlParser parse]) {
		NSLog(@"***Parsing UNSUCCESSFUL!");
		NSLog(@"Tried to parse:\n%@", [ManagerUtility prettyPrintXML:[[[NSString alloc] initWithData:data encoding:NSStringEncodingConversionAllowLossy] autorelease]]);
	}
}


#pragma mark - Initialization
-(id)initWithDelegate:(id)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = [theDelegate retain];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[delegate release], delegate = nil;
	[xmlParser release], xmlParser = nil;
	
	[super dealloc];
}


#pragma mark - NSXMLParserDelegate
-(void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	if ([elementName isEqualToString:@"address"]) {
		
		parentElement = demographics;
		demographics.address = address;
		
	} else if ([elementName isEqualToString:@"demographics"]) {
		
		parentElement = demographics;
		[delegate addDemographics:demographics];
		
	} else if ([elementName isEqualToString:@"link"]) {
		[parentElement addLink:link];
	} else if ([elementName isEqualToString:@"nextOfKin"]) {
		
		parentElement = demographics;
		demographics.nextOfKin = nextOfKin;
	}
}

-(void)parserDidEndDocument:(NSXMLParser *)parser {
	[delegate parserFinishedCreatingResource:self];
}

-(void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {	
	
	if ([elementName isEqualToString:@"address"]) {
		
		[address release];
		address = [[Address alloc] initWithAttributeDictionary:attributeDict];
		parentElement = address;
		
	} else if ([elementName isEqualToString:@"demographics"]) {
		
		[demographics release];
		demographics = [[Demographics alloc] initWithAttributeDictionary:attributeDict];
		parentElement = demographics;
		
	} else if ([elementName isEqualToString:@"link"]) {
		
		[link release];
		link = [[Link alloc] initWithAttributeDictionary:attributeDict];
		
	} else if ([elementName isEqualToString:@"nextOfKin"]) {
		
		[nextOfKin release];
		nextOfKin = [[NextOfKin alloc] initWithAttributeDictionary:attributeDict];
		parentElement = nextOfKin;
	}
}

@end
