//  ProductionConnection.m
//  PatientViewer
//
//  Created by Grant Oladipo on 12/20/10.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "Alerter.h"
#import "AppDelegate.h"
#import "ApplicationProperty.h"
#import "IOSConnection.h"
#import "Reachability.h"
#import "UserSession.h"

@implementation IOSConnection

-(void)postWithURL:(NSURL *)URL withPostBodyString:(NSString *)postBody {
	
}

-(void)getWithURL:(NSURL *)URL usingParser:(id<Parser>)newParser {
	
	[[NSNotificationCenter defaultCenter] postNotificationName:ApplicationDidMakeMHPRequest object:self];
	
	[parser release];
	parser = [newParser retain];
	
	NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:URL cachePolicy:NSURLCacheStorageNotAllowed timeoutInterval:ConnectionTimeoutInterval];
	
	[request setValue:@"application/xml" forHTTPHeaderField:@"Accept"];
	
	[theConnection release];
	theConnection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
}

-(BOOL)statusCodeIsSuccessful:(NSInteger)statusCode {

	static NSInteger successBottomLimit = 200;
	static NSInteger successTopLimit = 300;
	
	return (statusCode >= successBottomLimit 
			&& statusCode < successTopLimit);
}

-(BOOL)internetConnectionIsReachable {
	
	Reachability *internetReach = [Reachability reachabilityForInternetConnection];
	
	return ([internetReach isReachableViaWiFi] || [internetReach isReachableViaWWAN]);
}

-(void)performHealthCheck {
	
	BOOL healthCheckFailed = NO;
	NSString *alertTitle;
	NSString *alertMessage;
	
	if (![self internetConnectionIsReachable]) {
	
		healthCheckFailed = YES;
		alertTitle = @"Unable to connect to the MHP Server!";
		alertMessage = @"Please check your network settings.";

	} else {
		
		NSInteger statusCode = [self statusCodeForURL:[NSURL URLWithString:[ApplicationProperty healthUrlString]] 
										   usingMethod:@"HEAD"];
		
		if (![self statusCodeIsSuccessful:statusCode]) {

			healthCheckFailed = YES;
			alertTitle = @"Unable to connect to the MHP Server!";
			alertMessage = [NSString stringWithFormat:@"HTTP Status %d - %@.", statusCode, [[NSHTTPURLResponse localizedStringForStatusCode:statusCode] capitalizedString]];
		}
	}
	
	if (healthCheckFailed) {
		
		[[NSNotificationCenter defaultCenter] postNotificationName:ConnectionHealthCheckFailed object:self userInfo:nil];
		
		[Alerter enqueueAlertViewWithTitle:alertTitle message:alertMessage];
		
	} else {
		[[NSNotificationCenter defaultCenter] postNotificationName:ConnectionHealthCheckSucceeded object:self userInfo:nil];			
	}
}

-(NSInteger)statusCodeForURL:(NSURL *)URL usingMethod:(NSString *)method {
	
	NSMutableURLRequest *request = [[NSMutableURLRequest alloc] initWithURL:URL cachePolicy:NSURLCacheStorageNotAllowed timeoutInterval:ConnectionTimeoutInterval];
	[request setHTTPMethod:method];
	
	NSError *error = nil;
	NSHTTPURLResponse *response = nil;
	[NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
	[request release];
	
	if (error) {
		return 408;
	} else {
		return [response statusCode];
	}
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[theConnection release], theConnection = nil;
	[receivedData release], receivedData = nil;
	[parser release], parser = nil;
	[super dealloc];
}


#pragma mark - NSURLConnectionDelegate Methods
-(void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {	
	NSLog(@"Connection failed! Error: %@, %@", error, [error userInfo]);
}

-(void)connectionDidFinishLoading:(NSURLConnection *)connection {

	[parser parseData:receivedData];
	
	[[NSNotificationCenter defaultCenter] postNotificationName:ApplicationDidFinishProcessingMHPResponse object:self];
}

-(void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge {
		
	if ([challenge previousFailureCount] > 0) {		
		[Alerter enqueueAlertViewWithTitle:@"LOGIN FAILED" message:@"Please attempt to login again" cancelButtonTitle:@"Continue"];
	} else {
	
		NSURLCredential *userCredential = [UserSession credential];
		
		if (userCredential) {
			[[challenge sender] useCredential:userCredential forAuthenticationChallenge:challenge];
		}
	}
}

-(void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	[receivedData appendData:data];
}

-(void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	[receivedData release];
	receivedData = [[NSMutableData alloc] initWithLength:0];
}

@end