//
//  ICADPatientSummaryContainer.m
//  iOSiCADLibrary
//
//  Created by Grant Oladipo on 1/3/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx
//

#import "LabMainContainer.h"
#import "LabResult.h"
#import "Medication.h"

#define InRangeKey = 0
#define OutOfRangeLowerKey = 1
#define OutOfRangeHigherKey = 2

@implementation	LabMainContainer

@synthesize referenceRangeMinOfHighRange;
@synthesize referenceRangeMaxOfLowRange;
@synthesize absoluteMaxValue;
@synthesize absoluteMinValue;
@synthesize delegate;
@synthesize currentDate;
@synthesize currentMedicationSource;
@synthesize currentRange;
@synthesize earliestDate;
@synthesize labResultsMaxValue;
@synthesize labResultsMinValue;
@synthesize labTestHasReferenceRangeHigh;
@synthesize labTestHasReferenceRangeLow;
@synthesize referenceRangeMaxValue;
@synthesize referenceRangeMinValue;
@synthesize totalNumberOfLabResults;
@synthesize totalNumberOfMedications;


#pragma mark - Custom Accessors
-(NSTimeInterval)dateRange {
	
	NSTimeInterval range;
	
	switch (currentRange) {
		case LabTestRangeDay:
			range = oneDay;
			break;
			
		case LabTestRangeWeek:
			range = oneDay*7;
			break;
			
		case LabTestRangeMonth:
			range = oneDay*30;
			break;
			
		case LabTestRangeYear:
			range = oneDay*365;
			break;
			
		case LabTestRangeTwoYear:
			range = oneDay*730;
			break;
			
		default:
			range = [self.currentDate timeIntervalSinceDate:self.earliestDate];
			break;
	}
	
	return range;
}

-(NSArray *)filteredLabResults {
	
	if ([filteredLabResults count] > 0) {
		return filteredLabResults;
	}
	
	NSDate *startDate = [self.currentDate dateByAddingTimeInterval:-1*[self dateRange]];
	
	[filteredLabResults addObjectsFromArray:[labResults filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"resultedDate >= %@", startDate]]];
	
	return filteredLabResults;
}

-(NSArray *)filteredMedications {
	
	if (filteredMedications) {
		return filteredMedications;
	}
	
	NSDate *startDate = [self.currentDate dateByAddingTimeInterval:-1*[self dateRange]];
	
	switch (currentMedicationSource) {
		case MedicationSourceInpatient:
			filteredMedications = [[medications filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"medicationSource == %@ AND endDate >= %@", [NSString stringWithString:@"INPATIENT"], startDate]] retain];
			break;
		case MedicationSourceOutpatient:
			filteredMedications = [[medications filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"medicationSource == %@ AND endDate >= %@", [NSString stringWithString:@"OUTPATIENT"], startDate]] retain];
			break;
		default:
			filteredMedications = [[medications filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"endDate >= %@", startDate]] retain];
			break;
	}
	
	return filteredMedications;
}

-(LabResult *)labResultAtIndex:(NSUInteger)index {
	return [[self filteredLabResults] objectAtIndex:index];
}

-(CGFloat)referenceRangeMinOfHighRange {
    return [[[self filteredLabResults] valueForKeyPath:@"@min.referenceHigh.floatValue"] floatValue];
}

-(CGFloat)referenceRangeMaxOfLowRange {
    return [[[self filteredLabResults] valueForKeyPath:@"@max.referenceLow.floatValue"] floatValue];
}

-(CGFloat)labResultsMaxValue {
	return [[[self filteredLabResults] valueForKeyPath:@"@max.value.floatValue"] floatValue];
}

-(CGFloat)labResultsMinValue {
	return [[[self filteredLabResults] valueForKeyPath:@"@min.value.floatValue"] floatValue];
}

-(CGFloat)referenceRangeMaxValue {
    return [[[self filteredLabResults] valueForKeyPath:@"@max.referenceHigh.floatValue"] floatValue];
}

-(CGFloat)referenceRangeMinValue {    
    return [[[self filteredLabResults] valueForKeyPath:@"@min.referenceLow.floatValue"] floatValue];
}

-(BOOL)labTestHasReferenceRangeHigh {
    BOOL result = NO;
    
    if ([[self filteredLabResults] valueForKeyPath:@"@max.referenceHigh"]) {
        result = YES;
    }
    
    return result;
}

-(BOOL)labTestHasReferenceRangeLow {
    BOOL result = NO;
    
    if ([[self filteredLabResults] valueForKeyPath:@"@min.referenceLow"]) {
        result = YES;
    }
    
    return  result;
}

-(NSInteger)numberOfLabResults {
	return [[self filteredLabResults] count];
}

-(NSInteger)numberOfMedications {
	return [[self filteredMedications] count];
}

-(Medication *)medicationAtIndex:(NSUInteger)index {
	return [[self filteredMedications] objectAtIndex:index];
}

-(CGFloat)valueRange {
    
    CGFloat maxValue = self.labResultsMaxValue;
    CGFloat minValue = self.labResultsMinValue;
    
    if (self.labTestHasReferenceRangeHigh) {
        if (self.labResultsMinValue >= self.referenceRangeMaxValue) {
            maxValue = self.labResultsMaxValue;
            minValue = self.referenceRangeMaxValue;
        }
    }
    
    if (self.labTestHasReferenceRangeLow) {
        if (self.labResultsMaxValue <= self.referenceRangeMinValue) {
            maxValue = self.referenceRangeMinValue;
            minValue = self.labResultsMinValue;
        }
    }
    
    
    if (maxValue == minValue) {
        maxValue += maxValue*.1;
        minValue -= minValue*.1;
    }

    CGFloat range = (maxValue - minValue);
    
    
    self.absoluteMaxValue = maxValue;
    self.absoluteMinValue = minValue;
    
    
    if (range >= 10) {
        range = roundf(range);
    } else if (range != 0 && range > 0.1) {
        range *= 10;
        range = roundf(range);
        range /= 10;
    } else if (range < 0.1) {
        range = maxValue;  
    } else {
        range = maxValue*0.5;
    }
    
    if (range > 1) {
        range = ceilf(range);
    }
    
    
    return range;
}


#pragma mark - Custom Actions
-(void)addLabResult:(LabResult *)theLabResult {
	
	[labResults addObject:theLabResult];
	
	if ([labResults count] == 1) {
		self.earliestDate = [theLabResult resultedDate];
	}

	self.earliestDate = [self.earliestDate earlierDate:[theLabResult resultedDate]];
	
	[labResults sortUsingDescriptors:[NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"resultedDate" ascending:YES]]];
	
	if (totalNumberOfLabResults == [labResults count]) {
		[delegate labTestGraphContainerHasLoadedLabResults:self];
	}
}

-(void)addMedication:(Medication *)theMedication {
    
	[medications addObject:theMedication];
	
	if (totalNumberOfMedications == [medications count]) {
		[delegate labTestGraphContainerHasLoadedAllMedications:self];
	}
}

-(void)refreshWithPatient:(Patient *)thePatient labTest:(LabTest *)theLabTest {
	
	[labResults removeAllObjects];
	[medications removeAllObjects];
	
	[[LabResultManager labResultManager] labResultsForLabTest:theLabTest result:self];
	[[MedicationManager medicationManager] allMedicationsforPatient:thePatient result:self];
}

-(void)setCurrentRange:(LabMainContainerRange)newCurrentRange {

	currentRange = newCurrentRange;
	
	self.currentDate = [NSDate date];
	
	[filteredLabResults removeAllObjects];
	[filteredMedications release], filteredMedications = nil;
	
	[delegate labTestGraphContainerDidChangeCurrentRange:self];
}
	      
-(void)setCurrentMedicationSource:(MedicationSource)newMedicationSource {
	
	currentMedicationSource = newMedicationSource;
	
	[filteredMedications release], filteredMedications = nil;
}

-(void)parserFinishedCreatingResource:(id<Parser>)parser {
	
}


#pragma mark - Initialization
-(id)init {
	
	self = [super init];
	
	if (self) {
		
		self.currentDate = [NSDate date];
		filteredLabResults = [[NSMutableArray alloc] initWithCapacity:1];
		filteredMedications = [[NSMutableArray alloc] initWithCapacity:1];
		labResults = [[NSMutableArray alloc] initWithCapacity:1];
		medications = [[NSMutableArray alloc] initWithCapacity:1];
		totalNumberOfLabResults = -1;
		totalNumberOfMedications = -1;
	}
	
	return self;
}

-(id)initWithDelegate:(id<LabMainContainerDelegate>)theDelegate {

	self = [self init];
	
	if (self) {
		delegate = theDelegate;
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[currentDate release], currentDate = nil;
	[earliestDate release], earliestDate = nil;
	[filteredLabResults release], filteredLabResults = nil;
	[filteredMedications release], filteredMedications = nil;
	[labResults release], labResults = nil;
	[medications release], medications = nil;
	
	[super dealloc];
}


+(NSString *)stringFromRange:(LabMainContainerRange)range {
	
	NSString *string;
	
	switch (range) {
		case LabTestRangeDay:
			string = @"Day";
			break;
			
		case LabTestRangeWeek:
			string = @"Week";
			break;
			
		case LabTestRangeMonth:
			string = @"Month";
			break;
			
		case LabTestRangeYear:
			string = @"Year";
			break;
			
		case LabTestRangeTwoYear:
			string = @"Two Year";
			break;
			
		case LabTestRangeAll:
			string = @"All";
			break;
			
		default:
			string = @"Month";
			break;
	}
	
	return string;
}
				
@end