//  labPanelViewController.m
//  PatientViewer
//
//  Created by Grant Oladipo on 3/9/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "LabResult.h"
#import "LabTest.h"
#import "LabPanelContainer.h"
#import "LabPanel.h"
#import "MetricsPusher.h"
#import "Patient.h"
#import "SectionInfo.h"
#import "UserSession.h"

@implementation LabPanel

@synthesize listTV;
@synthesize customTVC;
@synthesize titleL;
@synthesize currentTab;
@synthesize chemHemB;
@synthesize microbiologyB;
@synthesize microTV;
@synthesize labTabSC;
@synthesize microLabD;
@synthesize selectedIndexPath;

-(void)configureCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath forTableView:(UITableView *)tableView {
	
	UILabel *label;
	LabTest *labTest;
    LabResult *labResultForCell;
    LabResult *latestLabResult;
    
	static NSDateFormatter *dateFormatter = nil;
	if (!dateFormatter) {
		dateFormatter = [[NSDateFormatter alloc] init];
		[dateFormatter setDateStyle:NSDateFormatterMediumStyle];
		[dateFormatter setTimeStyle:NSDateFormatterNoStyle];
	}
	
    
	label = (UILabel *)[cell viewWithTag:1];
	

    if (tableView == listTV) {
        
        labTest = [labPanelContainer labTestAtIndex:indexPath.row inGroupSection:indexPath.section];
        latestLabResult = labTest.mostRecentLabResult;

        if (labTest) {
            label.text = [labTest testname];
            label.frame = CGRectMake(12, 4, 190, 35);

            label = (UILabel *)[cell viewWithTag:2];
            label.text = [dateFormatter stringFromDate:[latestLabResult resultedDate]];		
            label = (UILabel *)[cell viewWithTag:3];
            label.text = [latestLabResult value];
            label.superview.hidden = NO;
        } else {
            label.text = @"No Hem/Chem Lab Data";
            label.frame = CGRectMake(12, 4, 277, 35);
            
            label = (UILabel *)[cell viewWithTag:2];
            label.text = @"";		
            label = (UILabel *)[cell viewWithTag:3];
            label.superview.hidden = YES;
        }

    } else {
        labResultForCell = [labPanelContainer microLabResultAtIndex:indexPath.row inGroupSection:indexPath.section];
        if (labResultForCell) {
            label.text = [labResultForCell displayName];
            label.frame = CGRectMake(12, 4, 277, 35);

            label = (UILabel *)[cell viewWithTag:2];
            label.text = [dateFormatter stringFromDate:[labResultForCell resultedDate]];
            label.frame = CGRectMake(12, 28, 277, 22);
            
            label = (UILabel *)[cell viewWithTag:3];
            label.text =@"";
            label.superview.hidden = YES;
        } else {
            label.text = @"No Microbiology Lab Data";
            label.frame = CGRectMake(12, 4, 277, 35);
            label = (UILabel *)[cell viewWithTag:2];
            label.text = @"";		
            label = (UILabel *)[cell viewWithTag:3];
            label.superview.hidden = YES;
        }
        
    }

}

#pragma mark - Custom Accesssors
-(UITableViewCell *)customizedCellForRowAtIndexPath:(NSIndexPath *)indexPath inTableView:(UITableView *)tableView {
	//Implemented in subclass
	return nil;
}


#pragma mark - Custom Actions

-(void)displayLabResultAtIndexPath:(NSIndexPath *)indexPath {
	//Implemented in subclass
}

-(void)displayLabTestAtIndexPath:(NSIndexPath *)indexPath {
	//Implemented in subclass
}
        
        
-(void)clearChem {
    [UserSession removeSelectedLabTest];
    [UserSession removeSelectedLabResult];
    [labPanelContainer removeChemObjects];
	[sectionInfoArray removeAllObjects];
    
    [listTV reloadData];
}


-(void)clearMicro {
    [labPanelContainer removeMicroObjects];
    [UserSession removeSelectedLabResult];
    [microSectionInfoArray removeAllObjects];
    
    [microTV reloadData];
}

-(void)retrieveResults:(LabTab)labTab {
	
	Patient *patient = [UserSession selectedPatient];
	
	
	if (patient) {
        switch (labTab) {
            case ChemHem:
                [self clearChem];
                [labPanelContainer refreshChemLabsWithPatient:patient];
                break;
                
            case Microbiology:
                [self  clearMicro];
                [labPanelContainer refreshMicroLabsWithPatient:patient];
                areMicrosLoaded = YES;
                break;
                
            default:
                [labPanelContainer refreshChemLabsWithPatient:patient];
                break;
        }
        
	}
}

-(void)setTabToChemHem {
    currentTab = ChemHem;
    listTV.hidden = NO;
    microTV.hidden = YES;
    [UserSession showChemTest];
}

-(void)setTabToMicro {
    currentTab = Microbiology;
    microTV.hidden = NO;
    listTV.hidden = YES;
    [UserSession showMicroResults];
}

-(IBAction)setCurrentLabTab:(UISegmentedControl*)sender {
    switch (sender.selectedSegmentIndex) {
        case 0:
            [self setTabToChemHem];
            break;
            
        case 1:
            if (areMicrosLoaded == NO) {
                [self retrieveResults:Microbiology];
            }
            [self setTabToMicro];
                
            break;
            
        default:
            break;
    }
    
}



#pragma mark - Initialization Methods
-(void)processDidChangePatient:(NSNotification *)notification {
	shouldRefresh = YES;
    [self setTabToChemHem];
    [labTabSC setSelectedSegmentIndex:ChemHem];
}

-(void)registerForNotifications {
	
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter addObserver:self selector:@selector(processDidChangePatient:) name:UserSessionDidChangePatient object:nil];
}

-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	
	self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
	
	if (self) {
		[self registerForNotifications];
		labPanelContainer = [[LabPanelContainer alloc] initWithDelegate:self];
		shouldRefresh = YES;
        sectionInfoArray = [[NSMutableArray alloc] initWithCapacity:1];
        microSectionInfoArray = [[NSMutableArray alloc] initWithCapacity:1];
        
        microGroups = [[NSMutableArray alloc] initWithCapacity:1];
	}
	
	return self;
}


#pragma mark - LabPanelContainerDelegate Method
-(void)labPanelContainerWillAddLabTestToGroup:(LabPanelContainer *)container {
    [self.listTV beginUpdates];
}


-(void)labPanelContainer:(LabPanelContainer *)container didAddLabTestToGroupWithIndex:(NSInteger)index {	
    
	UITableView *tableView = self.listTV;
	[tableView reloadData];
	
	
	NSIndexPath *newIndexPath = [NSIndexPath indexPathForRow:0 inSection:index];
	[tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath] withRowAnimation:UITableViewRowAnimationTop];
	
	[tableView endUpdates];
}

-(void)labPanelContainerWillAddLabTestGroup:(LabPanelContainer *)container {
    [self.listTV beginUpdates];
    
}

-(void)labPanelContainer:(LabPanelContainer *)container didAddLabTestGroupAtIndex:(NSInteger)index {
    
    UITableView *tableView = self.listTV;
    
    SectionInfo *sectionInfo = [[SectionInfo alloc] init];			
    sectionInfo.open = YES;
    [sectionInfoArray addObject:sectionInfo];
    [sectionInfo release];
    
    [tableView insertSections:[NSIndexSet indexSetWithIndex:index] withRowAnimation:UITableViewRowAnimationTop];
    
    [tableView endUpdates];
}

-(void)labPanelContainerFinishedLoadingLabTests:(LabPanelContainer *)container {
	UITableView *tableView = self.listTV;
	
	if (![labPanelContainer hasLabTestGroups]) {
		
		[tableView beginUpdates];
		
        SectionInfo *sectionInfo = [[SectionInfo alloc] init];			
        sectionInfo.open = YES;
        [sectionInfoArray addObject:sectionInfo];
        [sectionInfo release];
        
		[tableView insertSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationTop];
		
		[tableView endUpdates];
	}
}


-(void)labPanelContainerWillAddLabResultToGroup:(LabPanelContainer *)container {
    //not used if sections are collapsed
}

-(void)labPanelContainer:(LabPanelContainer *)container didAddLabResultToGroupWithIndex:(NSInteger)index {
    //not used if sections are collapsed
}


-(void)labPanelContainerWillAddLabResultGroup:(LabPanelContainer *)container {
    [self.microTV beginUpdates];
}


-(void)labPanelContainer:(LabPanelContainer *)container didAddLabResultGroupAtIndex:(NSInteger)index {
    
    UITableView *tableView = self.microTV;
    
    SectionInfo *sectionInfo = [[SectionInfo alloc] init];			
    sectionInfo.open = NO;
    [microSectionInfoArray addObject:sectionInfo];
    [sectionInfo release];
    
    [tableView insertSections:[NSIndexSet indexSetWithIndex:index] withRowAnimation:UITableViewRowAnimationTop];
    
    [self.microTV endUpdates];
}


-(void)putLatestMicroDatesInSectionHeaders {
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateStyle:NSDateFormatterMediumStyle];
    [dateFormatter setTimeStyle:NSDateFormatterNoStyle];
    
    for (int section = 0; section < [microSectionInfoArray count]; section++) {
        
        SectionInfo *sectionInfo = [microSectionInfoArray objectAtIndex:section];
        LabResult *latestLabResult = [labPanelContainer microLabResultAtIndex:0 inGroupSection:section];
        
        NSString *date = [dateFormatter stringFromDate:[latestLabResult resultedDate]];	
        sectionInfo.headerView.dateLabel.text = date;
    }
}

-(void)labPanelContainerFinishedLoadingLabResults:(LabPanelContainer *)container {
    UITableView *tableView = self.microTV;
	
	if (![labPanelContainer hasLabResultGroups]) {
		
		[tableView beginUpdates];
        
        SectionInfo *sectionInfo = [[SectionInfo alloc] init];			
        sectionInfo.open = YES;
        [microSectionInfoArray addObject:sectionInfo];
        [sectionInfo release];
        
		[tableView insertSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationTop];
		
		[tableView endUpdates];
        
	}
    
    [self putLatestMicroDatesInSectionHeaders];
}


#pragma mark - Memory Management
-(void)unregisterFromNotifications {
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

-(void)dealloc {
	[self unregisterFromNotifications];
	
	labPanelContainer.delegate = nil;
	[labPanelContainer release], labPanelContainer = nil;
	
	[super dealloc];
}



#pragma mark - UITableViewDataSource Methods
-(void)configureCell:(UITableViewCell *)cell forIndexPath:(NSIndexPath *)indexPath identifier:(NSString *)cellIdentifier {
}


-(NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    NSInteger sections = 1;
    
    if (tableView == microTV) {
        sections = [microSectionInfoArray count];
    } else {
        sections = [sectionInfoArray count];
    }
    
    return sections;
}


-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    NSInteger rows = 0;
    
    if (tableView == microTV) {
        if ([microSectionInfoArray count] == 0) {
            rows = 0;
        } else {
            
            SectionInfo *sectionInfo = [microSectionInfoArray objectAtIndex:section];

            rows = sectionInfo.open ? [labPanelContainer numberofLabResultsForGroupSection:section]: 0;
        }
        
    } else if (tableView == listTV) {
        if ([sectionInfoArray count] == 0) {
            rows = 0;
        } else {
            SectionInfo *sectionInfo = [sectionInfoArray objectAtIndex:section];
            rows = sectionInfo.open ? [labPanelContainer numberOfLabTestsForGroupSection:section]: 0;
        }
    }
    
    return rows;
}


-(UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {

    LabTest *labTest;
    UITableViewCell *cell;
    NSString *cellIdentifier;
    
    if (tableView == listTV) {
        labTest = [labPanelContainer labTestAtIndex:indexPath.row inGroupSection:indexPath.section];
        LabResult *labResult = labTest.mostRecentLabResult;
        cellIdentifier = @"Unknown";
        if ([[labResult abnormalIndicatorTriFlag] isEqualToString:@"true"]) {
            cellIdentifier = @"Abnormal";
        } else if ([[labResult abnormalIndicatorTriFlag] isEqualToString:@"false"]) {
            cellIdentifier = @"Normal";
        }   
    } else {
        cellIdentifier = @"Normal";
    }
    
    
    cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if (cell == nil) {
        
        cell = [self customizedCellForRowAtIndexPath:indexPath inTableView:tableView];
    } 
    
    [self configureCell:cell forRowAtIndexPath:indexPath forTableView:tableView];
    
	return cell;
}

-(void)showLabResultOnMainForSection:(NSInteger)section {
    
    for (int index = 0; index < [microSectionInfoArray count]; index++) {
        SectionInfo *sectionInfo = [microSectionInfoArray objectAtIndex:index];
        [sectionInfo.headerView hideSelectionCover];
        
        if (index == section) {
            [sectionInfo.headerView showSelectionCover];
        }
    }
    
    [self displayLabResultAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:section]];
}


-(void)clearSectionHeaderCovers {
    for (int index = 0; index < [microSectionInfoArray count]; index++) {
        SectionInfo *sectionInfo = [microSectionInfoArray objectAtIndex:index];
        [sectionInfo.headerView hideSelectionCover];
    }
}

#pragma mark - View Lifecycle
-(void)viewWillAppear:(BOOL)animated {

	[super viewWillAppear:animated];
	
    
	if (shouldRefresh) {
        [self retrieveResults:ChemHem];
        areMicrosLoaded = NO;
		shouldRefresh = NO;
	}
    
    if (currentTab == ChemHem) {
        [UserSession showChemTest];
    } else {
        [UserSession showMicroResults];
    }

}

-(void)viewDidAppear:(BOOL)animated {

	[super viewDidAppear:animated];
    
	[MetricsPusher reportMetricsWithUserEventValue:@"LaunchLabViewer" tagDictionary:nil];
}

-(void)viewDidLoad {
	static NSString *LabPanelTitle = @"Lab Results";
	
	[super viewDidLoad];
	
	self.title = LabPanelTitle;
	self.titleL.text = LabPanelTitle;
    currentTab = ChemHem;
    [UserSession showChemTest];
}

-(void)viewDidUnload {
	
	[listTV release], listTV = nil;
	[customTVC release], customTVC = nil;
	[titleL release], titleL = nil;
    [microTV release], microTV = nil;
	[super viewDidUnload];
}

@end