//
//  ICADSearchPanelContainer.m
//  iOSiCADLibrary
//
//  Created by Grant Oladipo on 12/15/10.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx
//

#import "LabPanelContainer.h"
#import "LabTestGroup.h"
#import "LabTestGroupManager.h"
#import "Patient.h"
#import "PatientManager.h"
#import "LabResultManager.h"
#import "LabResultParser.h"
#import "MicroLabResultsManager.h"
#import "MicroLabResultParser.h"

@implementation LabPanelContainer

@synthesize delegate;
@synthesize totalNumberOfLabTests;
@synthesize totalNumberOfGroups;
@synthesize totalNumberOfMicroLabResults;
@synthesize totalNumberOfLabResultsGroups;

#pragma mark - Custom Accessors
-(BOOL)hasLabTests {
	return ([labTests count] > 0);
}

-(BOOL)hasLabTestGroups {
    return ([labTestGroups count] > 0);
}



-(NSInteger)numberOfLabTestsForGroupSection:(NSInteger)index {
    NSInteger numberOfLabTests = 1;
    
    if ([self hasLabTestGroups]) {
        numberOfLabTests = [[[labTestGroups objectAtIndex:index] labTests] count];
    } 
    NSLog(@"lab test groups %@", labTestGroups);
    
    NSLog(@"num lab test for group %d: %d", index, numberOfLabTests);
    
    return numberOfLabTests;
}

-(NSInteger)numberOfLabTestGroups {
    return [labTestGroups count];
}


-(LabTest *)labTestAtIndex:(NSInteger)index inGroupSection:(NSInteger)sectionIndex {
    if ([labTestGroups count] > sectionIndex) {
        if ([[[labTestGroups objectAtIndex:sectionIndex] labTests] count] > index) {
            return [[[labTestGroups objectAtIndex:sectionIndex] labTests] objectAtIndex:index];
        }
    }
	
    return nil;
}

-(LabTestGroup *)labTestGroupAtIndex:(NSInteger)index  {
    if  ([labTestGroups count] > index) {
        return [labTestGroups objectAtIndex:index];
    }
    
    return nil;
}



-(BOOL)hasMicroLabResults {
    return ([microLabResults count] > 0);
}

-(BOOL)hasLabResultGroups {
    return ([labResultsGroups count] > 0);
}

-(NSInteger)numberofLabResultsForGroupSection:(NSInteger)index {
    
    if ([self hasLabResultGroups]) {
        return [[[labResultsGroups objectAtIndex:index] labResults] count];
    } 
    else {
        return 1;
    }
}



-(NSInteger)numberofLabResultGroups {
    return [labResultsGroups count];
}

-(LabResultGroup *)labResultGroupAtIndex:(NSInteger)index {
    if  ([labResultsGroups count] > index) {
        return [labResultsGroups objectAtIndex:index];
    }
    
    return nil;
}


-(LabResult *)microLabResultAtIndex:(NSInteger)index inGroupSection:(NSInteger)sectionIndex {
    if  ([labResultsGroups count] > sectionIndex) {
        if ([[[labResultsGroups objectAtIndex:sectionIndex] labResults] count] > index) {
            return [[[labResultsGroups objectAtIndex:sectionIndex] labResults] objectAtIndex:index];
        }
    }
	
    return nil;
}



#pragma mark - Custom Actions
-(void)removeChemObjects {
    [labTests removeAllObjects];
	[labTestGroups removeAllObjects];
    [self setTotalNumberOfGroups:0];
	[self setTotalNumberOfLabTests:0];
}

-(void)removeMicroObjects {
    
    [labResultsGroups removeAllObjects];
    [microLabResults removeAllObjects];
    [self setTotalNumberofLabResultsGroups:0];
    [self setTotalNumberOfMicroLabResults:0];
}

-(void)removeAllObjects {
    [self removeChemObjects];
    [self removeMicroObjects];
}

-(void)refreshChemLabsWithPatient:(Patient*)patient {	
    
	[self removeChemObjects];
	
	[[LabTestGroupManager labTestGroupManager] labTestsWithGroupsForPatient:patient result:self];
}


-(void)refreshMicroLabsWithPatient:(Patient*)patient {
    [self removeMicroObjects];
    [[MicroLabResultsManager microLabResultsManager] microbiologyLabResultsForPatient:patient result:self];
}


#pragma mark - Initialization
-(id)initWithDelegate:(id<LabPanelContainerDelegate>)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = theDelegate;
		labTests = [[NSMutableArray alloc] initWithCapacity:1];
        labTestGroups = [[NSMutableArray alloc] initWithCapacity:1];
        microLabResults = [[NSMutableArray alloc] initWithCapacity:1];
        labResultsGroups = [[NSMutableArray alloc] initWithCapacity:1];
	}
	
	return self;
}



#pragma mark - LabContainer Methods
-(void)addLabTest:(LabTest *)theLabTest toGroup:(LabTestGroup *)theLabTestGroup {
    
    [delegate labPanelContainerWillAddLabTestToGroup:self];
    
    NSInteger index = [labTestGroups indexOfObject:theLabTestGroup];
    [[[labTestGroups objectAtIndex:index] labTests] addObject:theLabTest];
    
    [delegate labPanelContainer:self didAddLabTestToGroupWithIndex:index];
    
}



-(void)addLabTestGroup:(LabTestGroup *)theLabTestGroup {
	
	[delegate labPanelContainerWillAddLabTestGroup:self];
	
    theLabTestGroup.labTests = [[NSMutableArray alloc] initWithCapacity:1];
    
	[labTestGroups addObject:theLabTestGroup];
	
	NSInteger index = [labTestGroups indexOfObject:theLabTestGroup];
	
	[delegate labPanelContainer:self didAddLabTestGroupAtIndex:index];
	
}

-(void)setTotalNumberOfGroups:(NSInteger)theTotalNumberOfGroups {
	totalNumberOfGroups = theTotalNumberOfGroups;
}

#pragma mark - LabResultContainer Methods
-(void)addLabResultGroup:(LabResultGroup*)theLabResultGroup {
    [delegate labPanelContainerWillAddLabResultGroup:self];
    theLabResultGroup.labResults = [[NSMutableArray alloc] initWithCapacity:1];
    
    [labResultsGroups addObject:theLabResultGroup];
	
	NSInteger index = [labResultsGroups indexOfObject:theLabResultGroup];
	
	[delegate labPanelContainer:self didAddLabResultGroupAtIndex:index];

}

-(void)addLabResult:(LabResult*)theLabResult toGroup:(LabResultGroup*)theLabResultGroup {
    
    [delegate labPanelContainerWillAddLabResultToGroup:self];
    
    NSInteger index = [labResultsGroups indexOfObject:theLabResultGroup];
    [[[labResultsGroups objectAtIndex:index] labResults] addObject:theLabResult];
    
    [delegate labPanelContainer:self didAddLabResultToGroupWithIndex:index];
    
}

-(void)setTotalNumberofLabResultsGroups:(NSInteger)theTotalNumberOfLabResultGroups{
    totalNumberOfLabResultsGroups = theTotalNumberOfLabResultGroups;
}

#pragma mark - Memory Management
-(void)dealloc {
	
	[labTests release], labTests = nil;
	[microLabResults release], microLabResults = nil;
    
	[super dealloc];
}



#pragma mark - ResourceContainer Methods
-(void)parserFinishedCreatingResource:(id<Parser>)parser {
    if ([parser isMemberOfClass:[MicroLabResultParser class]]) {
        [delegate labPanelContainerFinishedLoadingLabResults:self];
    } else {
        [delegate labPanelContainerFinishedLoadingLabTests:self];        
    }
}

@end