//  LoginViewController.m
//  PatientViewer
//
//  Created by Grant Oladipo on 2/2/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "Alerter.h"
#import "Connection.h"
#import "LoginDisclaimer_iPhone.h"
#import "LoginMain.h"
#import "MetricsPusher.h"
#import "UserSession.h"
#import "UserSessionManager.h"

@interface LoginMain ()
-(void)performLogin;
@end

@implementation LoginMain

@synthesize keyboardHeightChange;
@synthesize loginB;
@synthesize logoIV;
@synthesize usernameTF;
@synthesize passwordTF;


-(void)registerForNotifications {
    
	NSNotificationCenter *defaultNC = [NSNotificationCenter defaultCenter];
	
	[defaultNC addObserver:self selector:@selector(keyboardWillAppear:) name:UIKeyboardWillShowNotification object:nil];
	[defaultNC addObserver:self selector:@selector(keyboardWillDisappear:) name:UIKeyboardWillHideNotification object:nil];
}

-(void)unregisterFromNotifications {
    
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	
	self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
	
	if (self) {
		[self registerForNotifications];
	}
	
	return self;
}


#pragma mark - Custom Actions
-(IBAction)readMore {

	[MetricsPusher reportMetricsWithUserEventValue:@"LaunchDisclaimer" tagDictionary:nil];
}

-(IBAction)attemptLogin {
	
	[UserSession reset];
    
	if (![usernameTF.text isEqualToString:@""] && ![passwordTF.text isEqualToString:@""]) {
		
		NSURLCredential *userCredential = [NSURLCredential credentialWithUser:usernameTF.text password:passwordTF.text persistence:NSURLCredentialPersistenceForSession];
		
		[UserSession setCredential:userCredential];
		
		[[UserSessionManager userSessionManager] refreshUserSession:self];
	
	}else{
		[Alerter enqueueAlertViewWithTitle:@"LOGIN FAILED" 
								   message:@"Username or Password should not be empty" 
						 cancelButtonTitle:@"Continue"];
    }
}

-(void)performLogin {
	
	[MetricsPusher reportMetricsWithUserEventValue:@"Logon" tagDictionary:nil];
}

#pragma mark - Keyboard Notifications
-(void)keyboardWillAppear:(NSNotification *)notification {
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:[[[notification userInfo] objectForKey:UIKeyboardAnimationDurationUserInfoKey] doubleValue]];
	[UIView setAnimationCurve:[[[notification userInfo] objectForKey:UIKeyboardAnimationCurveUserInfoKey] intValue]];
	CGRect keyboardEndingUncorrectedFrame = [[[notification userInfo] objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue];
	CGRect keyboardEndingFrame = [self.view convertRect:keyboardEndingUncorrectedFrame fromView:nil];
	float change = keyboardEndingFrame.size.height-keyboardHeightChange;
	self.view.frame = CGRectMake(self.view.frame.origin.x, self.view.frame.origin.y-change, self.view.frame.size.width, self.view.frame.size.height);
	[UIView commitAnimations];
}

-(void)keyboardWillDisappear:(NSNotification *)notification {
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:[[[notification userInfo] objectForKey:UIKeyboardAnimationDurationUserInfoKey] doubleValue]];
	[UIView setAnimationCurve:[[[notification userInfo] objectForKey:UIKeyboardAnimationCurveUserInfoKey] intValue]];
	CGRect keyboardEndingUncorrectedFrame = [[[notification userInfo] objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue];
	CGRect keyboardEndingFrame = [self.view convertRect:keyboardEndingUncorrectedFrame fromView:nil];
	float change = keyboardEndingFrame.size.height-keyboardHeightChange;
	self.view.frame = CGRectMake(self.view.frame.origin.x, self.view.frame.origin.y+change, self.view.frame.size.width, self.view.frame.size.height);
	[UIView commitAnimations];
}


#pragma mark - Memory Managment
-(void)dealloc {

	[self unregisterFromNotifications];
	
	[super dealloc];
}


#pragma mark - UITextFieldDelegate Methods
-(BOOL)textFieldShouldReturn:(UITextField *)textField {

	if (self.usernameTF == textField) {
		[self.passwordTF becomeFirstResponder];
	} else {
		[self attemptLogin];
	}

	return NO;
}


#pragma mark - View Lifecycle
-(void)viewDidLoad {
	[super viewDidLoad];
}

-(void)viewWillAppear:(BOOL)animated {

	self.usernameTF.text = @"";
	self.passwordTF.text = @"";
}

-(void)viewDidAppear:(BOOL)animated {
	[super viewDidAppear:animated];
}

-(void)viewWillDisappear:(BOOL)animated {
    
    [self.usernameTF resignFirstResponder];
    [self.passwordTF resignFirstResponder];
    
    [super viewWillDisappear:animated];
}

- (void)viewDidUnload {
		
	self.logoIV = nil;
	self.passwordTF = nil;
	self.usernameTF = nil;
	
	[super viewDidUnload];
}

@end