//
//  MedicationMain.m
//  PatientViewer
//
//  Created by Grant Oladipo on 4/28/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "Medication.h"
#import "MedicationMain.h"
#import "MedicationMainContainer.h"
#import "UserSession.h"

@implementation MedicationMain

@synthesize medicationMainContainer;


#pragma mark - Custom Actions
-(void) hideView {
}

-(void)showView {
}

-(void)removeData {
	
	titleL.text = @"";
	quantityL.text = @"";
	lastFilledL.text = @"";
	refillsL.text = @"";
	expiresL.text = @"";
	sigTV.text = @"";
    medicationDetailTV.text = @"";
    orderedL.text = @"";
    statusL.text = @"";
    
}

-(void)changeStatusColor {
   //Implemented in subclass
}


-(void)populateData {
	
	static NSDateFormatter *dateFormatter = nil;
	
	if (!dateFormatter) {
		dateFormatter = [[NSDateFormatter alloc] init];
		[dateFormatter setDateFormat:@"MM/dd/yyyy"];
	}
	
	Medication *medication = [UserSession selectedMedication];
	
    orderedL.text = [dateFormatter stringFromDate:[medication startDate]];
	titleL.text = [[medication drugName] capitalizedString];
    
    statusL.text = [medication status];
	
    [self changeStatusColor];
    
    expiresL.text = [dateFormatter stringFromDate:[medication endDate]];
	sigTV.text = [medication sig];
    medicationDetailTV.text = [medication medicationDetail];
        
    if ([[medication medicationSource] isEqualToString:@"INPATIENT"]) {
        
        quantityL.hidden = YES;
        quantityHeaderL.hidden = YES;
        quantityL.text = @"";
        
        lastFilledL.hidden = YES;
        lastFilledHeaderL.hidden = YES;
        lastFilledL.text = @"";
        
        refillsL.hidden = YES;
        refillsHeaderL.hidden = YES;
        refillsL.text = @"";

    } else {
        quantityL.hidden = NO;
        quantityHeaderL.hidden = NO;
        quantityL.text = [medication displayQuantity];

        lastFilledL.hidden = NO;
        lastFilledHeaderL.hidden = NO;
        lastFilledL.text = [medication lastFilledDate];

        refillsL.hidden = NO;
        refillsHeaderL.hidden = NO;
        refillsL.text = [medication refill];
    }
    
}
-(void) refreshView {
    	
	[self hideView];
	
	[self removeData];
	
	if ([UserSession selectedMedication]) {
		
		[self populateData];		
		
		[self showView];
        
        [self.medicationMainContainer refreshMedicationDetails];
	}
    
    shouldRefresh = NO;
}

-(void)processDidChangePatient:(NSNotification *)notification {
	shouldRefresh = YES;
}

-(void)processDidChangeCurrentMedications:(NSNotification *)notification {
	shouldRefresh = YES;
}

-(void)processDidChangeMedication:(NSNotification *)notification {
	[self refreshView];
}

-(void)registerForNotifications {
	
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter addObserver:self selector:@selector(processDidChangePatient:) name:UserSessionDidChangePatient object:nil];
	[defaultCenter addObserver:self selector:@selector(processDidChangeCurrentMedications:) name:UserSessionDidChangeCurrentMedications object:nil];
	[defaultCenter addObserver:self selector:@selector(processDidChangeMedication:) name:UserSessionDidChangeMedication object:nil];
}

-(void)unregisterFromNotifications {
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark - Medication Main Delegate Methods
-(void)medicationMainContainerFinishedLoadingMedicationDetails:(MedicationMainContainer *)container {
}

-(void)medicationMainContainerWillChangeMedicationDetails:(MedicationMainContainer *)container {
    
}

-(void)medicationMainContainerDidChangeMedicationDetails:(MedicationMainContainer *)container {
    [self populateData];
}




#pragma mark - Initialization
-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	
	self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
	
	if (self) {
		
		[self registerForNotifications];
		
		self.title = @"Medication Details";
		
        self.medicationMainContainer = [[[MedicationMainContainer alloc] initWithDelegate:self] autorelease];
        
		shouldRefresh = YES;
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[self unregisterFromNotifications];
	
	[super dealloc];
}


#pragma mark - View Lifecycle 
-(void)viewWillAppear:(BOOL)animated {
	
	[super viewWillAppear:animated];
	
//	if (shouldRefresh) {
//		
//		[self refreshView];
//		
//		shouldRefresh = NO;
//	}
}

-(void)viewDidLoad {
    [super viewDidLoad];
}

-(void)viewDidUnload {
	
	[titleL release], titleL = nil;;
	[quantityL release], quantityL = nil;
	[lastFilledL release], lastFilledL = nil;
	[refillsL release], refillsL = nil;
	[expiresL release], expiresL = nil;
	[sigTV release], sigTV = nil;
	[orderedL release], orderedL = nil;
    [statusL release], statusL = nil;
    
	[super viewDidUnload];
}

@end




