//
//  MedicationPanel.m
//  PatientViewer
//
//  Created by Grant Oladipo on 4/28/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "MedicationPanel.h"
#import "UserSession.h"
#import "Patient.h"
#import "MedicationPanelContainer.h"
#import "SectionInfo.h"

@implementation MedicationPanel

@synthesize numberOfInpatientMedications;
@synthesize numberOfOutpatientMedications;
@synthesize numberOfNonVAMedications;
@synthesize inpatientIndex;
@synthesize outpatientIndex;
@synthesize nonVAIndex;
@synthesize listTV;
@synthesize medicationPanelContainer;

#pragma mark - Custom Actions
-(NSString *)titleForHeaderInSection:(NSInteger)section {
    NSString *result = @"";
    
    if ([self isInpatient]) {
        
        switch (section) {
            case inpatientIndexForInpatient:
                result = [NSString stringWithFormat:@"%d Inpatient Medication%@", [self.medicationPanelContainer numberOfInpatientMedications], ([self.medicationPanelContainer numberOfInpatientMedications]!=1?@"s":@"")];
                break;
            case nonVAIndexForInpatient:
                result = [NSString stringWithFormat:@"%d Non-VA Medication%@", [self.medicationPanelContainer numberOfNonVAMedications], ([self.medicationPanelContainer numberOfNonVAMedications]!=1?@"s":@"")];
                break;
            case outpatientIndexForInpatient:
                result = [NSString stringWithFormat:@"%d Outpatient Medication%@", [self.medicationPanelContainer numberOfOutpatientMedications], ([self.medicationPanelContainer numberOfOutpatientMedications]!=1?@"s":@"")];
                break;
                
            default:
                break;
        }
        
        
    } else {
        switch (section) {
            case inpatientIndexForOutpatient:
                result = [NSString stringWithFormat:@"%d Inpatient Medication%@", [self.medicationPanelContainer numberOfInpatientMedications], ([self.medicationPanelContainer numberOfInpatientMedications]!=1?@"s":@"")];
                break;
            case nonVAIndexForOutpatient:
                result = [NSString stringWithFormat:@"%d Non-VA Medication%@", [self.medicationPanelContainer numberOfNonVAMedications], ([self.medicationPanelContainer numberOfNonVAMedications]!=1?@"s":@"")];
                break;
            case outpatientIndexForOutpatient:
                result = [NSString stringWithFormat:@"%d Outpatient Medication%@", [self.medicationPanelContainer numberOfOutpatientMedications], ([self.medicationPanelContainer numberOfOutpatientMedications]!=1?@"s":@"")];
                break;
                
            default:
                break;
        }
        
    }
    
    return result;	
}

-(void)registerForNotifications {
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter addObserver:self selector:@selector(processDidChangePatient:) name:UserSessionDidChangePatient object:nil];
}

-(void)performClear {
	
	[UserSession removeSelectedMedication];
	
    self.numberOfNonVAMedications = 0;
    self.numberOfInpatientMedications = 0;
    self.numberOfOutpatientMedications = 0;
    
    [sectionInfoArray removeAllObjects];
    
	[self.medicationPanelContainer clearMedicationLists];

    
}

-(void)retrieveResults {
    
	Patient *patient = [UserSession selectedPatient];
	
	
	if (patient) {
        [self performClear];

		[self.medicationPanelContainer refreshWithUserSession];
        [self.listTV reloadData];

	}
    

}

-(BOOL)isInpatient {
    BOOL result = NO;
    
    if ([[[UserSession selectedPatient] status] isEqualToString:@"true"]) { 
        inpatientIndex = 0;
        nonVAIndex = 1;
        outpatientIndex = 2;
        result = YES;
    } else {
        outpatientIndex = 0;
        nonVAIndex = 1;
        inpatientIndex = 2;
    }
    
    return result;
}

-(NSInteger)getNumberOfRowsForGroupInSection:(NSInteger)section {
    NSInteger result = 0;
    
    if ([self isInpatient]) {
        
        switch (section) {
            case inpatientIndexForInpatient:
                result = [medicationPanelContainer numberOfInpatientMedications];
                break;
            case nonVAIndexForInpatient:
                result = [medicationPanelContainer numberOfNonVAMedications];
                break;
            case outpatientIndexForInpatient:
                result = [medicationPanelContainer numberOfOutpatientMedications];
                break;
                
            default:
                break;
        }
        
        
    } else {
        switch (section) {
            case inpatientIndexForOutpatient:
                result = [medicationPanelContainer numberOfInpatientMedications];
                break;
            case nonVAIndexForOutpatient:
                result = [medicationPanelContainer numberOfNonVAMedications];
                break;
            case outpatientIndexForOutpatient:
                result = [medicationPanelContainer numberOfOutpatientMedications];
                break;
                
            default:
                break;
        }
        
        
    }
    
    return result;
    
}

-(Medication*)getMedicationAtIndexPath:(NSIndexPath*)indexPath {
    Medication *medication = [[[Medication alloc] init] autorelease];
    
    if ([self isInpatient]) {
        
        switch (indexPath.section) {
            case inpatientIndexForInpatient:
                medication = [medicationPanelContainer inpatientMedicationAtIndex:indexPath.row];
                break;
            case nonVAIndexForInpatient:
                medication = [medicationPanelContainer nonVAMedicationAtIndex:indexPath.row];
                break;
            case outpatientIndexForInpatient:
                medication = [medicationPanelContainer outpatientMedicationAtIndex:indexPath.row];
                break;
                
            default:
                break;
        }
        
        
    } else {
        switch (indexPath.section) {
            case inpatientIndexForOutpatient:
                medication = [medicationPanelContainer inpatientMedicationAtIndex:indexPath.row];
                break;
            case nonVAIndexForOutpatient:
                medication = [medicationPanelContainer nonVAMedicationAtIndex:indexPath.row];
                break;
            case outpatientIndexForOutpatient:
                medication = [medicationPanelContainer outpatientMedicationAtIndex:indexPath.row];
                break;
                
            default:
                break;
        }
        
        
    }
    
    return medication;
}

#pragma mark - Initialization
-(void)processDidChangePatient:(NSNotification *)notification {
	shouldRefresh = YES;
}

-(void)createSectionInfoArray {
    sectionInfoArray = [[NSMutableArray alloc] initWithCapacity:3];
}

-(void)updateSectionInfoArray {
    static NSInteger firstIndex = 0;
    static NSInteger secondIndex = 1;
    static NSInteger thirdIndex = 2;
    
    
    
    [sectionInfoArray removeAllObjects];
    SectionInfo *inpatientInfo = [[SectionInfo alloc] init];
    SectionInfo *outpatientInfo = [[SectionInfo alloc] init];
    SectionInfo *nonVAInfo = [[SectionInfo alloc] init];
   
    inpatientInfo.open = NO;
    outpatientInfo.open = NO;
    nonVAInfo.open = NO;

    if ([self isInpatient]) {
        inpatientInfo.open = YES;
        
        inpatientInfo.groupName = [self titleForHeaderInSection:firstIndex];
        nonVAInfo.groupName = [self titleForHeaderInSection:secondIndex];
        outpatientInfo.groupName = [self titleForHeaderInSection:thirdIndex];
        
        [sectionInfoArray addObject:inpatientInfo];
        [sectionInfoArray addObject:nonVAInfo];
        [sectionInfoArray addObject:outpatientInfo];
    } else {
        outpatientInfo.open = YES;

        outpatientInfo.groupName = [self titleForHeaderInSection:firstIndex];
        nonVAInfo.groupName = [self titleForHeaderInSection:secondIndex];
        inpatientInfo.groupName = [self titleForHeaderInSection:thirdIndex];
        
        [sectionInfoArray addObject:outpatientInfo];
        [sectionInfoArray addObject:nonVAInfo];
        [sectionInfoArray addObject:inpatientInfo];
    }
    
    [inpatientInfo release];
    [nonVAInfo release];
    [outpatientInfo release];
}

-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
	
	if (self) {
        medicationPanelContainer = [[MedicationPanelContainer alloc] initWithDelegate:self];
        [self registerForNotifications];
        [self createSectionInfoArray];
    }
	
    return self;
}

#pragma mark - Memory Management
-(void)unregisterFromNotifications {
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}


- (void)dealloc {
	
	[self unregisterFromNotifications];
    medicationPanelContainer.delegate = nil;
    [medicationPanelContainer release], medicationPanelContainer = nil;
    
    [super dealloc];
}

#pragma mark - Medication Container Methods
-(void)medicationPanelContainerWillAddInpatientMedication:(MedicationPanelContainer *)container {
	[self.listTV beginUpdates];
}

-(void)medicationPanelContainerWillAddOutpatientMedication:(MedicationPanelContainer *)container {
	[self.listTV beginUpdates];
}
-(void)medicationPanelContainerWillAddNonVAMedication:(MedicationPanelContainer *)container{
	[self.listTV beginUpdates];
}


-(void)medicationPanelContainer:(MedicationPanelContainer *)container didAddInpatientMedicationAtIndex:(NSInteger)index {
    
	
	UITableView *tableView = self.listTV;
	
	if (self.numberOfInpatientMedications == 0) {
        //		[tableView insertSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationNone];
        //		numberOfSections++;
	}
    self.numberOfInpatientMedications++;
	NSIndexPath *indexPathForNewPatient = [NSIndexPath indexPathForRow:index inSection:self.inpatientIndex];
	[tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:indexPathForNewPatient] withRowAnimation:UITableViewRowAnimationTop];
	
	[tableView endUpdates];
}

-(void)medicationPanelContainer:(MedicationPanelContainer *)container didAddOutpatientMedicationAtIndex:(NSInteger)index {	
	UITableView *tableView = self.listTV;
	
	if (self.numberOfOutpatientMedications == 0) {
        //		[tableView insertSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationNone];
        //		numberOfSections++;
	}
    
	self.numberOfOutpatientMedications++;
	NSIndexPath *indexPathForNewPatient = [NSIndexPath indexPathForRow:index inSection:self.outpatientIndex];
	[tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:indexPathForNewPatient] withRowAnimation:UITableViewRowAnimationTop];
	
	[tableView endUpdates];
}


-(void)medicationPanelContainer:(MedicationPanelContainer *)container didAddNonVAMedicationAtIndex:(NSInteger)index {	
	UITableView *tableView = self.listTV;
	if (self.numberOfNonVAMedications == 0) {
        //		[tableView insertSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationNone];
        //		numberOfSections++;
	}
    
    self.numberOfNonVAMedications++;
	NSIndexPath *indexPathForNewPatient = [NSIndexPath indexPathForRow:index inSection:self.nonVAIndex];
	[tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:indexPathForNewPatient] withRowAnimation:UITableViewRowAnimationTop];
    
	[tableView endUpdates];
}

-(void)medicationPanelContainerFinishedLoadingInpatientMedications:(MedicationPanelContainer *)container {
}

-(void)medicationPanelContainerFinishedLoadingOutpatientMedications:(MedicationPanelContainer *)container {
}

-(void)medicationPanelContainerFinishedLoadingNonVAMedications:(MedicationPanelContainer *)container {
}

-(void)medicationPanelContainerFinishedLoadingMedications:(MedicationPanelContainer *)container  {
    self.numberOfNonVAMedications = [medicationPanelContainer numberOfNonVAMedications];
    self.numberOfOutpatientMedications = [medicationPanelContainer numberOfOutpatientMedications];
    self.numberOfInpatientMedications = [medicationPanelContainer numberOfInpatientMedications];
    
    [self updateSectionInfoArray];
}


@end