//
//  MedicationPanelContainer.m
//  MedicationViewer
//
//  Created by Grant Oladipo on 4/27/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "MedicationPanelContainer.h"
#import "Medication.h"
#import "MedicationManager.h"
#import "UserSession.h"

@implementation MedicationPanelContainer

@synthesize delegate;
@synthesize totalNumberOfMedications;
@synthesize medications;
@synthesize inpatientMedications;
@synthesize outpatientMedications;
@synthesize nonVAMedications;


#pragma mark - Custom Accessors
-(void)addInpatientMedications {
}

-(void)addOutpatientMedications {
}

-(void)addNonVAMedications {
}


-(BOOL)hasMedications {
	return ([self.medications count] > 0);
}

-(NSInteger)numberOfMedications {
	return [self.medications count];
}

-(NSInteger)numberOfInpatientMedications {
    return [self.inpatientMedications count];
}

-(NSInteger)numberOfOutpatientMedications {
    return [self.outpatientMedications count];
}

-(NSInteger)numberOfNonVAMedications {
    return [self.nonVAMedications count];
}

-(Medication *)medicationAtIndex:(NSInteger)index {
	
	if ([self.medications count] > index) {
		return [self.medications objectAtIndex:index];
	}
    
	return nil;
}

-(Medication *)inpatientMedicationAtIndex:(NSInteger)index {
	
	if ([self.inpatientMedications count] > index) {
		return [self.inpatientMedications objectAtIndex:index];
	}
    
	return nil;
}

-(Medication *)outpatientMedicationAtIndex:(NSInteger)index {
	
	if ([self.outpatientMedications count] > index) {
		return [self.outpatientMedications objectAtIndex:index];
	}
    
	return nil;
}

-(Medication *)nonVAMedicationAtIndex:(NSInteger)index {
	
	if ([self.nonVAMedications count] > index) {
		return [self.nonVAMedications objectAtIndex:index];
	}
    
	return nil;
}

#pragma mark - Custom Actions
-(void)clearMedicationLists {
	

	[self.inpatientMedications removeAllObjects];
    [self.outpatientMedications removeAllObjects];
    [self.nonVAMedications removeAllObjects];
    [self.medications removeAllObjects];
    
	[self setTotalNumberOfMedications:0];
}

-(void)refreshWithUserSession {

	[self clearMedicationLists];
	
    
    
	NSArray *newMedications = [UserSession currentMedications];
        
    NSPredicate *inpatientPredicate = [NSPredicate predicateWithFormat:@"medicationSource == [c] %@", @"inpatient"];  
    
    NSPredicate *outpatientPredicate = [NSPredicate predicateWithFormat:@"medicationSource == [c] %@", @"outpatient"];  
        
    NSPredicate *nonVAPredicate = [NSPredicate predicateWithFormat:@"medicationSource != [c] %@ AND medicationSource != [c] %@", @"inpatient", @"outpatient"];  
    
    
    self.inpatientMedications = nil;
    self.outpatientMedications = nil;
    self.nonVAMedications = nil;
    
    self.inpatientMedications = [[NSMutableArray alloc] initWithArray:[newMedications filteredArrayUsingPredicate:inpatientPredicate]];
    self.outpatientMedications = [[NSMutableArray alloc] initWithArray:[newMedications filteredArrayUsingPredicate:outpatientPredicate]];
    self.nonVAMedications = [[NSMutableArray alloc] initWithArray:[newMedications filteredArrayUsingPredicate:nonVAPredicate]];
    self.medications = [[NSMutableArray alloc] initWithArray:newMedications];
    
 
        
	[self setTotalNumberOfMedications:[newMedications count]];
    
	[delegate medicationPanelContainerFinishedLoadingMedications:self];

}


#pragma mark - Initialization
-(id)initWithDelegate:(id<MedicationPanelContainerDelegate>)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = theDelegate;
		self.medications = [[NSMutableArray alloc] initWithCapacity:1];
        self.inpatientMedications = [[NSMutableArray alloc] initWithCapacity:1];
        self.outpatientMedications = [[NSMutableArray alloc] initWithCapacity:1];
        self.nonVAMedications = [[NSMutableArray alloc] initWithCapacity:1];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[self.medications release], self.medications = nil;
	[self.inpatientMedications release], self.inpatientMedications = nil;
    [self.outpatientMedications release], self.outpatientMedications = nil;
    [self.nonVAMedications release], self.nonVAMedications = nil;
	[super dealloc];
}


#pragma MedicationContainer Methods

-(void)setTotalNumberOfMedications:(int)theTotalNumberOfMedications {
	totalNumberOfMedications = theTotalNumberOfMedications;
}


#pragma mark - ResourceContainer Methods
-(void)parserFinishedCreatingResource:(id<Parser>)parser {
	[delegate medicationPanelContainerFinishedLoadingMedications:self];
}


@end