//
//  MyClass.m
//  PatientViewer
//
//  Created by Grant Oladipo on 5/4/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
#import "Link.h"
#import "ManagerUtility.h"
#import "Medication.h"
#import "MedicationParser.h"


@implementation MedicationParser

#pragma mark - Custom Actions
-(void)parseData:(NSData *)data {
	
	[xmlParser release];
	xmlParser = [[NSXMLParser alloc] initWithData:data];
	[xmlParser setDelegate:self];
	[xmlParser setShouldProcessNamespaces:YES];
	
	
	if (![xmlParser parse]) {
		NSLog(@"***Parsing UNSUCCESSFUL!");
		NSLog(@"Tried to parse:\n%@", [ManagerUtility prettyPrintXML:[[[NSString alloc] initWithData:data encoding:NSStringEncodingConversionAllowLossy] autorelease]]);
	}
}


#pragma mark - Initialization
-(id)initWithDelegate:(id)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = [theDelegate retain];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[delegate release], delegate = nil;
	[xmlParser release], xmlParser = nil;
	
	[super dealloc];
}



#pragma mark -  NSXMLParserDelegate
-(void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	
	if ([elementName isEqualToString:@"link"]) {
		[parentElement addLink:link];
	} else if ([elementName isEqualToString:@"medication"]){
		
		parentElement = delegate;
		[delegate addMedication:medication];
	}
}

-(void)parserDidEndDocument:(NSXMLParser *)parser {
	
	[delegate parserFinishedCreatingResource:self];
}

-(void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {	
	
	if ([elementName isEqualToString:@"link"]) {
		
		[link release];
		link = [[Link alloc] initWithAttributeDictionary:attributeDict];
		
	} else if ([elementName isEqualToString:@"medication"]) {
		
		[medication release];
		medication = [[Medication alloc] initWithAttributeDictionary:attributeDict];
		parentElement = medication;
		
	} else if ([elementName isEqualToString:@"medications"]) {
		
		parentElement = delegate;
		
		if([delegate respondsToSelector:@selector(setTotalNumberOfMedications:)]) {
			[delegate setTotalNumberOfMedications:[[attributeDict objectForKey:@"size"] integerValue]];
		}
	}
}


@end
