//  PatientSummaryContainer.m
//  Patient Viewer
//
//  Created by Grant Oladipo on 1/3/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "AllergyManager.h"
#import "DemographicsManager.h"
#import "ImageManager.h"
#import "MedicationManager.h"
#import "PatientMainContainerDelegate.h"
#import "ProblemManager.h"
#import "Resource.h"

@class Allergy;
@class Demographics;
@class Medication;
@class Patient;
@class Problem;

@interface PatientMainContainer : Resource <AllergyContainer, DemographicsContainer, ImageContainer, MedicationContainer, ProblemContainer> {

	Demographics *demographics;
	Patient *patient;	
	UIImage *patientImage;

	NSMutableArray *allergies;
	NSMutableArray *inpatientMedications;
	NSMutableArray *outpatientMedications;
	NSMutableArray *nonVAMedications;
	NSMutableArray *medications;
	NSMutableArray *problems;
}

@property (nonatomic, assign) id<PatientMainContainerDelegate> delegate;

@property (nonatomic) NSInteger totalNumberOfAllergies;
@property (nonatomic) NSInteger totalNumberOfInpatientMedications;
@property (nonatomic) NSInteger totalNumberOfOutpatientMedications;
@property (nonatomic) NSInteger totalNumberOfNonVAMedications;
@property (nonatomic) NSInteger totalNumberOfMedications;
@property (nonatomic) NSInteger totalNumberOfProblems;

-(BOOL)hasAllergies;
-(BOOL)hasInpatientMedications;
-(BOOL)hasOutpatientMedications;
-(BOOL)hasNonVAMedications;
-(BOOL)hasMedications;
-(BOOL)hasProblems;

-(NSInteger)numberOfAllergies;
-(NSInteger)numberOfMedications;
-(NSInteger)numberOfInpatientMedications;
-(NSInteger)numberOfOutpatientMedications;
-(NSInteger)numberOfNonVAMedications;
-(NSInteger)numberOfProblems;

-(Allergy *)allergyAtIndex:(NSInteger)index;
-(Medication *)medicationAtIndex:(NSInteger)index;
-(Medication *)inpatientMedicationAtIndex:(NSInteger)index;
-(Medication *)outpatientMedicationAtIndex:(NSInteger)index;
-(Medication *)nonVAMedicationAtIndex:(NSInteger)index;
-(Problem *)problemAtIndex:(NSInteger)index;

-(Demographics *)demographics;
-(Patient *)patient;
-(UIImage *)patientImage;

-(void)refreshWithPatient:(Patient *)thePatient;
-(void)removeAllObjects;

-(id)initWithDelegate:(id<PatientMainContainerDelegate>)theDelegate;

@end