//  PatientMainContainer.m
//  Patient Viewer
//
//  Created by Grant Oladipo on 1/3/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "Allergy.h"
#import "AllergyParser.h"
#import "Demographics.h"
#import "DemographicsParser.h"
#import "Medication.h"
#import "MedicationParser.h"
#import "Patient.h"
#import "PatientMainContainer.h"
#import "Problem.h"
#import "ProblemParser.h"
#import "UserSession.h"

@implementation PatientMainContainer

@synthesize delegate;
@synthesize totalNumberOfAllergies;
@synthesize totalNumberOfInpatientMedications;
@synthesize totalNumberOfOutpatientMedications;
@synthesize totalNumberOfNonVAMedications;
@synthesize totalNumberOfMedications;
@synthesize totalNumberOfProblems;


#pragma mark - Custom Accessors
-(BOOL)hasAllergies {
	return ([allergies count] > 0);
}

-(BOOL)hasMedications {
	return ([medications count] > 0);
}

-(BOOL)hasInpatientMedications {
	return ([inpatientMedications count] > 0);
}

-(BOOL)hasOutpatientMedications {
	return ([outpatientMedications count] > 0);
}

-(BOOL)hasNonVAMedications {
	return ([nonVAMedications count] > 0);
}

-(BOOL)hasProblems {
	return ([problems count] > 0);
}

-(NSInteger)numberOfAllergies {
	return [allergies count];
}

-(NSInteger)numberOfMedications {
	return [medications count];
}

-(NSInteger)numberOfInpatientMedications {
	return [inpatientMedications count];
}

-(NSInteger)numberOfOutpatientMedications {
	return [outpatientMedications count];
}

-(NSInteger)numberOfNonVAMedications {
	return [nonVAMedications count];
}

-(NSInteger)numberOfProblems {
	return [problems count];
}

-(Allergy *)allergyAtIndex:(NSInteger)index {
	
	if ([allergies count] > index) {
		return [allergies objectAtIndex:index];
	}
	return nil;
}

-(Medication *)medicationAtIndex:(NSInteger)index {
	
	if ([medications count] > index) {
		return [medications objectAtIndex:index];
	}
	return nil;
}

-(Medication *)inpatientMedicationAtIndex:(NSInteger)index {
	
	if ([inpatientMedications count] > index) {
		return [inpatientMedications objectAtIndex:index];
	}
	return nil;
}

-(Medication *)outpatientMedicationAtIndex:(NSInteger)index {
	
	if ([outpatientMedications count] > index) {
		return [outpatientMedications objectAtIndex:index];
	}
	return nil;
}

-(Medication *)nonVAMedicationAtIndex:(NSInteger)index {
	
	if ([nonVAMedications count] > index) {
		return [nonVAMedications objectAtIndex:index];
	}
	return nil;
}

-(Problem *)problemAtIndex:(NSInteger)index {
	
	if ([problems count] > index) {
		return [problems objectAtIndex:index];
	}
	return nil;
}

-(Demographics *)demographics {
	return demographics;
}

-(Patient *)patient {
	return patient;
}

-(UIImage *)patientImage {
	return patientImage;
}


#pragma mark - Custom Actions
-(void)removeAllObjects {
	
	[demographics release], demographics = nil;
	[patient release], patient = nil;
	[patientImage release], patientImage = nil;
	
	[allergies removeAllObjects];
	[inpatientMedications removeAllObjects];
	[outpatientMedications removeAllObjects];
	[nonVAMedications removeAllObjects];
    [medications removeAllObjects];

	[problems removeAllObjects];
	
	[self setTotalNumberOfAllergies:0];
	[self setTotalNumberOfMedications:0];
	[self setTotalNumberOfProblems:0];
}

-(void)refreshWithPatient:(Patient *)thePatient {
	
	[self removeAllObjects];
	
	patient = [thePatient retain];
	
	[[DemographicsManager demographicsManager] demographicsForPatient:thePatient result:self];
	[[ProblemManager problemManager] activeProblemsForPatient:thePatient result:self];
	[[AllergyManager allergyManager] allergiesForPatient:thePatient result:self];
	[[MedicationManager medicationManager] activeMedicationsForPatient:thePatient result:self];
	[[ImageManager imageManager] patientImageForPatient:thePatient result:self];
}


#pragma mark - Initialization
-(id)initWithDelegate:(id<PatientMainContainerDelegate>)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = theDelegate;
		allergies = [[NSMutableArray alloc] initWithCapacity:1];
		inpatientMedications = [[NSMutableArray alloc] initWithCapacity:1];
		outpatientMedications = [[NSMutableArray alloc] initWithCapacity:1];
		nonVAMedications = [[NSMutableArray alloc] initWithCapacity:1];		
		medications = [[NSMutableArray alloc] initWithCapacity:1];
        problems = [[NSMutableArray alloc] initWithCapacity:1];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[allergies release], allergies = nil;
	[demographics release], demographics = nil;
	[inpatientMedications release], inpatientMedications = nil;
	[outpatientMedications release], outpatientMedications = nil;
	[nonVAMedications release], nonVAMedications = nil;
	[medications release], medications = nil;
	[patient release], patient = nil;
	[patientImage release], patientImage = nil;
	[problems release], problems = nil;
	
	[super dealloc];
}


#pragma AllergyContainer Methods
-(void)addAllergy:(Allergy *)theAllergy {
	
    if ([delegate respondsToSelector:@selector(patientMainContainerWillAddAllergy:)]) {
        [delegate patientMainContainerWillAddAllergy:self];
	}
    
	[allergies addObject:theAllergy];
	
	NSInteger index = [allergies indexOfObject:theAllergy];
    
    if ([delegate respondsToSelector:@selector(patientMainContainer:didAddAllergyAtIndex:)]) {
        [delegate patientMainContainer:self didAddAllergyAtIndex:index];
    }
}

-(void)setTotalNumberOfAllergies:(NSInteger)theTotalNumberOfAllergies{
	totalNumberOfAllergies = theTotalNumberOfAllergies;
}


#pragma mark - DemographicsContainer Methods
-(void)addDemographics:(Demographics *)theDemographics {
	
    if ([delegate respondsToSelector:@selector(patientMainContainerWillChangeDemographics:)]) {
        [delegate patientMainContainerWillChangeDemographics:self];
    } 
    
	[demographics release];
	demographics = [theDemographics retain];

    if ([delegate respondsToSelector:@selector(patientMainContainerDidChangeDemographics:)]) {
        [delegate patientMainContainerDidChangeDemographics:self];
    }
}


#pragma mark - ImageContainer Methods
-(void)addImage:(UIImage *)theImage {
   
    if ([delegate respondsToSelector:@selector(patientMainContainerWillChangePatientImage:)]) {
        [delegate patientMainContainerWillChangePatientImage:self];
	}
    
	[patientImage release];
	patientImage = [theImage retain];

    if ([delegate respondsToSelector:@selector(patientMainContainerDidChangePatientImage:)]) {
        [delegate patientMainContainerDidChangePatientImage:self];
    }
}


#pragma MedicationContainer Methods
-(void)addMedication:(Medication *)theMedication {
	static NSString *inpatientSource = @"inpatient";
    static NSString *outpatientSource = @"outpatient";

    [medications addObject:theMedication];
	NSInteger index;

    
    if ([theMedication.medicationSource caseInsensitiveCompare:inpatientSource] == NSOrderedSame ) {

        if ([delegate respondsToSelector:@selector(patientMainContainerWillAddInpatientMedication:)]) {
            [delegate patientMainContainerWillAddInpatientMedication:self];
        }

        [inpatientMedications addObject:theMedication];
        index = [inpatientMedications indexOfObject:theMedication];
        
        if ([delegate respondsToSelector:@selector(patientMainContainer:didAddInpatientMedicationAtIndex:)]) {   
            [delegate patientMainContainer:self didAddInpatientMedicationAtIndex:index];
        }
    } else if ([theMedication.medicationSource caseInsensitiveCompare:outpatientSource] == NSOrderedSame ) {
        
        if ([delegate respondsToSelector:@selector(patientMainContainerWillAddOutpatientMedication:)]) {
            [delegate patientMainContainerWillAddOutpatientMedication:self];
        }
        
        [outpatientMedications addObject:theMedication];
        index = [outpatientMedications indexOfObject:theMedication];
        
        if ([delegate respondsToSelector:@selector(patientMainContainer:didAddOutpatientMedicationAtIndex:)]) {   
            [delegate patientMainContainer:self didAddOutpatientMedicationAtIndex:index];
        }
    } else {
        
        if ([delegate respondsToSelector:@selector(patientMainContainerWillAddNonVAMedication:)]) {
            [delegate patientMainContainerWillAddNonVAMedication:self];
        }
        
        [nonVAMedications addObject:theMedication];
        index = [nonVAMedications indexOfObject:theMedication];
        
        if ([delegate respondsToSelector:@selector(patientMainContainer:didAddNonVAMedicationAtIndex:)]) {   
            [delegate patientMainContainer:self didAddNonVAMedicationAtIndex:index];
        }
    } 
    
}

-(void)setTotalNumberOfMedications:(NSInteger)theTotalNumberOfMedications {
	totalNumberOfMedications = theTotalNumberOfMedications;
}

-(void)setTotalNumberOfInpatientMedications:(NSInteger)theTotalNumberOfMedications {
	totalNumberOfInpatientMedications = theTotalNumberOfMedications;
}

-(void)setTotalNumberOfOutpatientMedications:(NSInteger)theTotalNumberOfMedications {
	totalNumberOfOutpatientMedications = theTotalNumberOfMedications;
}

-(void)setTotalNumberOfNonVAMedications:(NSInteger)theTotalNumberOfMedications {
	totalNumberOfNonVAMedications = theTotalNumberOfMedications;
}


#pragma ProblemContainer Methods
-(void)addProblem:(Problem *)theProblem {
    
    if ([delegate respondsToSelector:@selector(patientMainContainerWillAddProblem:)]) {
        [delegate patientMainContainerWillAddProblem:self];
    }
    
	[problems addObject:theProblem];
	
	NSInteger index = [problems indexOfObject:theProblem];
    
    if ([delegate respondsToSelector:@selector(patientMainContainer:didAddProblemAtIndex:)]) {
        [delegate patientMainContainer:self didAddProblemAtIndex:index];
    }
}

-(void)setTotalNumberOfProblems:(NSInteger)theTotalNumberOfProblems {
	totalNumberOfProblems = theTotalNumberOfProblems;
}


#pragma mark - ResourceContainer Methods
-(void)parserFinishedCreatingResource:(id<Parser>)parser {

	if ([parser isKindOfClass:[AllergyParser class]]) {
		[delegate patientMainContainerFinishedLoadingAllergies:self];
	} else if ([parser isKindOfClass:[MedicationParser class]]) {
		
		[UserSession setCurrentMedications:medications];
		
        if ([delegate respondsToSelector:@selector(patientMainContainerFinishedLoadingInpatientMedications:)]) {
            [delegate patientMainContainerFinishedLoadingInpatientMedications:self];
        }
        
        if ([delegate respondsToSelector:@selector(patientMainContainerFinishedLoadingOutpatientMedications:)]) {
            [delegate patientMainContainerFinishedLoadingOutpatientMedications:self];
        }
        
        if ([delegate respondsToSelector:@selector(patientMainContainerFinishedLoadingNonVAMedications:)]) {
            [delegate patientMainContainerFinishedLoadingNonVAMedications:self];
        }
        
        if ([delegate respondsToSelector:@selector(patientMainContainerFinishedLoadingMedications:)]) {
            [delegate patientMainContainerFinishedLoadingMedications:self];
        }
        
	} else if ([parser isKindOfClass:[ProblemParser class]]) {
		[delegate patientMainContainerFinishedLoadingProblems:self];
	} else if ([parser isKindOfClass:[DemographicsParser class]]) {
        if ([delegate respondsToSelector:@selector(patientMainFinishedLoadingDemographics:)]) {
            [delegate patientMainContainerFinishedLoadingDemographics:self];
        }
        
    }
}
				
@end