//  SearchPanelViewController.m
//  PatientViewer
//
//  Created by Grant Oladipo on 1/25/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "Alerter.h"
#import "Link.h"
#import "MetricsPusher.h"
#import "Patient.h"
#import "PatientPanelContainer.h"
#import "PatientPanel.h"
#import "UserSession.h"

@implementation PatientPanel

@synthesize customTVC;
@synthesize listTV;
@synthesize titleL;
//@synthesize searchStringSB;
@synthesize searchTF;

#pragma mark - Custom Actions
-(void)retrieveResults {
	
	static NSString *AlertTitle = @"No Search Criteria Entered";
	static NSString *AlertMessage = @"Please search using SSN and/or patient last name.";
	static NSString *AlertButtonTitle = @"CONTINUE";
	
	[self performClear];
    
	if (searchTF.text && ![searchTF.text isEqualToString:@""]) {
		[patientPanelContainer refreshWithSearchString:searchTF.text];
	} else {
		[Alerter enqueueAlertViewWithTitle:AlertTitle message:AlertMessage cancelButtonTitle:AlertButtonTitle];
	}
    
    self.listTV.hidden = NO;
}

-(NSString*)getPatientCellName {
    //implemented in subclass
    return nil;
}

-(NSString*)getNoPatientCellName {
    //implemented in subclass
    return nil;
}


-(UIImage*)getSectionHeaderImage {
    //implemented in subclass
    return nil;
}


-(void)disableInput {
    searchTF.userInteractionEnabled = NO;
}

-(void)enableInput {
	searchTF.userInteractionEnabled = YES;
}

-(void)performSearch {
    
	[self disableInput];
	[self retrieveResults];
	[self enableInput];
}

-(void)performClear {
    
    self.listTV.hidden = YES;
    
	[UserSession removeSelectedPatient];
	
	[patientPanelContainer removeAllObjects];
	numberOfSections = 0;
	[listTV reloadData];
}


#pragma mark - Initialization
-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	
	self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
	
	if (self) {
		patientPanelContainer = [[PatientPanelContainer alloc] initWithDelegate:self];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	patientPanelContainer.delegate = nil;
	[patientPanelContainer release], patientPanelContainer = nil;
	
	[super dealloc];
}


#pragma mark - PatientPanelContainerDelegate Methods
-(void)patientPanelContainerWillAddPatient:(PatientPanelContainer *)container {
	[self.listTV beginUpdates];
}

-(void)patientPanelContainer:(PatientPanelContainer *)container didAddPatientAtIndex:(NSInteger)index {
	
	UITableView *tableView = self.listTV;
	
	if (numberOfSections == 0) {
		[tableView insertSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationNone];
		numberOfSections++;
	}
	
	NSIndexPath *indexPathForNewPatient = [NSIndexPath indexPathForRow:index inSection:0];
	[tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:indexPathForNewPatient] withRowAnimation:UITableViewRowAnimationTop];
	
	[tableView endUpdates];
}

-(void)patientPanelContainerFinishedLoadingPatients:(PatientPanelContainer *)container {
	
	UITableView *tableView = self.listTV;

	if (![patientPanelContainer hasPatients]) {

		[tableView beginUpdates];
		
		[tableView insertSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationTop];
		numberOfSections++;
		
		[tableView endUpdates];
	}
}


#pragma  mark - UITextFieldDelegatemethods 
-(BOOL)textFieldShouldReturn:(UITextField*)textField {
    [self performSearch];
    
    return YES;
}

-(BOOL)textFieldShouldClear:(UITextField *)textField {
    
    [self performClear];
    
    return YES;
}


#pragma mark - UITableViewDataSource Methods
-(void)configureCell:(UITableViewCell *)cell forIndexPath:(NSIndexPath *)indexPath identifier:(NSString *)cellIdentifier {
	
	if ([cellIdentifier isEqualToString:(NSString *)NoPatientsCellIdentifier]) {
		
		UILabel *label = (UILabel *)[cell viewWithTag:1];
		label.text = [NSString stringWithFormat:@"\"%@\"", [patientPanelContainer searchString]];
		
	} else {
		
		Patient *patient = [patientPanelContainer patientAtIndex:indexPath.row];
		UILabel *nameLabel = (UILabel *)[cell viewWithTag:1];
		UILabel *ssnLabel = (UILabel *)[cell viewWithTag:2];
		UILabel *dobLabel = (UILabel *)[cell viewWithTag:3];
		UILabel *genderLabel = (UILabel *)[cell viewWithTag:4];
        
		nameLabel.text = [[patient displayName] capitalizedString];
		ssnLabel.text = [patient ssn];
		dobLabel.text = @"";
		genderLabel.text = @"";
        
		if (![@"" isEqualToString:[patient dateOfBirth]]) {
			dobLabel.text = [NSString stringWithFormat:@"%@ (%@)", [patient dateOfBirth], [patient age]];	
		}
        
        if ([[patient gender] length] > 1) {
            genderLabel.text = [[patient gender] substringToIndex:1];
        }
	}
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSString *PatientCellNibName = [self getPatientCellName];
    NSString *NoPatientCellNibName = [self getNoPatientCellName];
	
	NSString *cellIdentifier = [self cellIdentifierForContainer:patientPanelContainer];
    
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    
	if (cell == nil) {
		
		if ([cellIdentifier isEqualToString:PatientCellIdentifier]) {
            
			[[NSBundle bundleForClass:[self class]] loadNibNamed:PatientCellNibName owner:self options:nil];
			
		} else {
			[[NSBundle bundleForClass:[self class]] loadNibNamed:NoPatientCellNibName owner:self options:nil];
		}
		
		cell = [[self.customTVC retain] autorelease];
		self.customTVC = nil;
	}
    
    
	[self configureCell:cell forIndexPath:indexPath identifier:cellIdentifier];
	
	return cell;
}


-(NSString *)cellIdentifierForContainer:(PatientPanelContainer *)container {
	
	if ([container hasPatients]) {
		return (NSString *)PatientCellIdentifier;
    } else {
		return (NSString *)NoPatientsCellIdentifier;
    }
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return numberOfSections;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	
	static NSInteger NumberOfRowsForNoPatientsFound = 1;
	
	if ([patientPanelContainer hasPatients]) {
		return [patientPanelContainer numberOfPatients];
	} else {
		return NumberOfRowsForNoPatientsFound;
	}
}

-(UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
    
    if([patientPanelContainer hasPatients]){
        UILabel *title = [[UILabel alloc] initWithFrame:CGRectMake(15, 4, 200, 22)];
        
        title.backgroundColor = [UIColor clearColor];
        title.font = [UIFont fontWithName:@"Arial-BoldMT" size:15];
        title.textColor = [UIColor colorWithRawValuesForRed:24 green:38 blue:86 alpha:1.0];
        title.shadowColor = [UIColor whiteColor];
        title.shadowOffset = CGSizeMake(0, 1);
        
        title.text = [NSString stringWithFormat:@"%d Match%@ for \"%@\"", [patientPanelContainer totalNumberOfPatients], ([patientPanelContainer totalNumberOfPatients]!=1?@"es":@""), [patientPanelContainer searchString]];
        
        UIImageView *header = [[[UIImageView alloc] initWithImage:[self getSectionHeaderImage]] autorelease];
        
        [header addSubview:title];
        
        [title release];
        
        return  header;
        
    } else{
        return  nil;
	}	
}

#pragma mark - UITableViewDelegate Methods
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
	Patient *patient = [patientPanelContainer patientAtIndex:indexPath.row];
	
	NSDictionary *tagDict = [NSDictionary dictionaryWithObjectsAndKeys:[[[[patient links] objectForKey:ResourceSelfLinkKey] href] absoluteString], @"Patient.uri", nil];
	[MetricsPusher reportMetricsWithUserEventValue:@"SelectPatient" tagDictionary:tagDict];
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {

	static CGFloat HeightForPatientResultsTitleSection = 29.0;
	return HeightForPatientResultsTitleSection;
}


#pragma mark - View Lifecycle
-(void)viewDidLoad {
    
	static NSString *PatientPanelTitle = @"Patient Search";
	
	[super viewDidLoad];
    
    self.title = PatientPanelTitle;
	self.titleL.text = PatientPanelTitle;
	
	[self.searchTF becomeFirstResponder];
}

-(void)viewDidUnload {
	
	[customTVC release], customTVC = nil;
	[listTV release], listTV = nil;
	[titleL release], titleL = nil;
	
    searchTF.delegate = nil;
    [searchTF release], searchTF = nil;
    
	[super viewDidUnload];
}

-(void)viewWillDisappear:(BOOL)animated {
    
    [searchTF resignFirstResponder];
    
    [super viewWillDisappear:animated];
}

@end