//  SearchPanelContainer.m
//  Patient Viewer
//
//  Created by Grant Oladipo on 12/15/10.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "PatientPanelContainer.h"
#import "Patient.h"
#import "PatientManager.h"

@implementation PatientPanelContainer

@synthesize delegate;
@synthesize totalNumberOfPatients;


#pragma mark - Custom Accessors
-(BOOL)hasPatients {
	return ([patients count] > 0);
}

-(NSInteger)numberOfPatients {
	return [patients count];
}

-(Patient *)patientAtIndex:(NSInteger)index {
	
	if ([patients count] > index) {
		return [patients objectAtIndex:index];
	}
	return nil;
}

-(NSString *)searchString {
	return searchString;
}


#pragma mark - Custom Actions
-(void)removeAllObjects {
	
	[searchString release], searchString = nil;
	
	[patients removeAllObjects];
	
	[self setTotalNumberOfPatients:0];
}

-(void)refreshWithSearchString:(NSString *)theSearchString {
	
	[self removeAllObjects];
	
	searchString = [theSearchString retain];
	
	[[PatientManager patientManager]  patientsWithSearchString:theSearchString result:self];
}


#pragma mark - Initialization
-(id)initWithDelegate:(id<PatientPanelContainerDelegate>)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = theDelegate;
		patients = [[NSMutableArray alloc] initWithCapacity:1];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[searchString release], searchString = nil;
	[patients release], patients = nil;
	
	[super dealloc];
}


#pragma PatientContainer Methods
-(void)addPatient:(Patient *)aPatient {
	
	[delegate patientPanelContainerWillAddPatient:self];
	
	[patients addObject:aPatient];
	
	NSInteger index = [patients indexOfObject:aPatient];
	
	[delegate patientPanelContainer:self didAddPatientAtIndex:index];
}

-(void)setTotalNumberOfPatients:(int)theTotalNumberOfPatients {
	totalNumberOfPatients = theTotalNumberOfPatients;
}


#pragma mark - ResourceContainer Methods
-(void)parserFinishedCreatingResource:(id<Parser>)parser {
	[delegate patientPanelContainerFinishedLoadingPatients:self];
}


@end