//  PatientParser.m
//  PatientViewer
//
//  Created by Grant Oladipo on 5/4/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxx

#import "Link.h"
#import "ManagerUtility.h"
#import "Patient.h"
#import "PatientPanelContainer.h"
#import "PatientParser.h"

@implementation PatientParser


#pragma mark - Custom Actions
-(void)parseData:(NSData *)data {
	
	[xmlParser release];
	xmlParser = [[NSXMLParser alloc] initWithData:data];
	[xmlParser setDelegate:self];
	[xmlParser setShouldProcessNamespaces:YES];
	
	if (![xmlParser parse]) {
		NSLog(@"***Parsing UNSUCCESSFUL!");
		NSLog(@"Tried to parse:\n%@", [ManagerUtility prettyPrintXML:[[[NSString alloc] initWithData:data encoding:NSStringEncodingConversionAllowLossy] autorelease]]);
	}
}

#pragma mark - Initialization
-(id)initWithDelegate:(id)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = [theDelegate retain];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[delegate release], delegate = nil;
	[xmlParser release], xmlParser = nil;
	
	[super dealloc];
}


#pragma mark - NSXMLParserDelegate Methods
-(void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	
	if ([elementName isEqualToString:@"link"]) {
		[parentElement addLink:myLink];
	} else if ([elementName isEqualToString:@"patient"]){
		
		parentElement = delegate;
		[delegate addPatient:patient];
	} 
}

-(void)parserDidEndDocument:(NSXMLParser *)parser {
	[delegate parserFinishedCreatingResource:self];
}

-(void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {	
	
	if ([elementName isEqualToString:@"link"]) {
		
		[myLink release];
		myLink = [[Link alloc] initWithAttributeDictionary:attributeDict];
		
	} else if ([elementName isEqualToString:@"patient"]) {
		
		[patient release];
		patient = [[Patient alloc] initWithAttributeDictionary:attributeDict];
		parentElement = patient;
		
	} else if ([elementName isEqualToString:@"patients"]) {
		
		parentElement = delegate;
		[delegate setTotalNumberOfPatients:[[attributeDict objectForKey:@"size"] integerValue]];
	}
}

@end