//
//  ProblemParser.m
//  PatientViewer
//
//  Created by Grant Oladipo on 5/4/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "Link.h"
#import "ManagerUtility.h"
#import "Problem.h"
#import "ProblemParser.h"


@implementation ProblemParser

#pragma mark - Custom Actions
-(void)parseData:(NSData *)data {
	
	[xmlParser release];
	xmlParser = [[NSXMLParser alloc] initWithData:data];
	[xmlParser setDelegate:self];
	[xmlParser setShouldProcessNamespaces:YES];

	
	if (![xmlParser parse]) {
		NSLog(@"***Parsing UNSUCCESSFUL!");
		NSLog(@"Tried to parse:\n%@", [ManagerUtility prettyPrintXML:[[[NSString alloc] initWithData:data encoding:NSStringEncodingConversionAllowLossy] autorelease]]);
	}
}

#pragma mark - Initialization
-(id)initWithDelegate:(id)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = [theDelegate retain];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[delegate release], delegate = nil;
	[xmlParser release], xmlParser = nil;
	

	[super dealloc];
}


#pragma mark - NSXMLParserDelegate
-(void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	
	if ([elementName isEqualToString:@"link"]) {
		[parentElement addLink:myLink];
	} else if ([elementName isEqualToString:@"problem"]){
		
		parentElement = delegate;
		[delegate addProblem:problem];
		
	} else if ([elementName isEqualToString:@"problems"]) {
	} 
}

-(void)parserDidEndDocument:(NSXMLParser *)parser {
	[delegate parserFinishedCreatingResource:self];
}

-(void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {	
	
	if ([elementName isEqualToString:@"link"]) {
		
		[myLink release];
		myLink = [[Link alloc] initWithAttributeDictionary:attributeDict];
		
	} else if ([elementName isEqualToString:@"problem"]) {
		
		[problem release];
		problem = [[Problem alloc] initWithAttributeDictionary:attributeDict];
		parentElement = problem;
		
	} else if ([elementName isEqualToString:@"problems"]) {
		
		parentElement = delegate;
		[delegate setTotalNumberOfProblems:[[attributeDict objectForKey:@"size"] integerValue]];
	}
}

@end
