#import "SectionHeaderView.h"

@implementation SectionHeaderView


@synthesize titleLabel, disclosureButton, delegate, section, dateLabel, selectionCover, imageView;

-(UIImage *)getSectionImage {
    //implemented in subclass
    return nil;
}

-(UIImage *)getOpenCarat {
    //implemented in subclass
    return nil;
}

-(UIImage *)getClosedCarat {
    //implemented in subclass
    return nil;
}

-(void)createDisclosureButton {
    //implemented in subclass
}

-(UIColor *)getTextColor {
    //implemented in subclass
    return nil;
}

-(UIImage *)getSelectedSectionImage {
    //implemented in subclass    
    return nil;
}

-(void)createGestureRecognizer {
    // Set up the tap gesture recognizer.
    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(toggleOpen:)];
    [self addGestureRecognizer:tapGesture];
    [tapGesture release];
}

-(void)createMicroGestureRecognizers {
    // Set up the tap gesture recognizer.
    titleLabel.userInteractionEnabled = YES;

    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(showLabResult:)];
    [titleLabel addGestureRecognizer:tapGesture];
    [tapGesture release];
    
    UITapGestureRecognizer *tapButtonGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(toggleOpen:)];
    [disclosureButton addGestureRecognizer:tapButtonGesture];
    [tapButtonGesture release];
}

-(void)createMicroTitleLabelWithTitle:(NSString *)title {
    CGRect titleLabelFrame = self.bounds;
    titleLabelFrame.origin.x += 30.0;
    titleLabelFrame.size.width -= 114.0;
    CGRectInset(titleLabelFrame, 0.0, 5.0);
    titleLabel = [[UILabel alloc] initWithFrame:titleLabelFrame];
    titleLabel.text = title;
    titleLabel.font = [UIFont fontWithName:@"Arial-BoldMT" size:15];
    titleLabel.textColor = [self getTextColor];
    titleLabel.backgroundColor = [UIColor clearColor];
}

-(void)createTitleLabelWithTitle:(NSString *)title {
    CGRect titleLabelFrame = self.bounds;
    titleLabelFrame.origin.x += 30.0;
    titleLabelFrame.size.width -= 30.0;
    CGRectInset(titleLabelFrame, 0.0, 5.0);
    titleLabel = [[UILabel alloc] initWithFrame:titleLabelFrame];
    titleLabel.text = title;
    titleLabel.font = [UIFont fontWithName:@"Arial-BoldMT" size:15];
    titleLabel.textColor = [self getTextColor];
    titleLabel.backgroundColor = [UIColor clearColor];
}

-(void)createDateLabel {
    CGRect dateLabelFrame = self.bounds;
    dateLabelFrame.origin.x += 216.0;
    dateLabelFrame.size.width -= 216.0;
    CGRectInset(dateLabelFrame, 0.0, 5.0);
    dateLabel = [[UILabel alloc] initWithFrame:dateLabelFrame];
    dateLabel.font = [UIFont fontWithName:@"Arial-BoldMT" size:12];
    dateLabel.textColor = [self getTextColor];
    dateLabel.backgroundColor = [UIColor clearColor];
}

-(void)createSelectionCover {
    CGRect coverFrame = self.bounds;
    selectionCover = [[UIView alloc] initWithFrame:coverFrame];
    selectionCover.backgroundColor = [UIColor greenColor];
    selectionCover.alpha = 0.4;
    selectionCover.hidden = YES;
}


-(id)initWithFrame:(CGRect)frame title:(NSString *)title section:(NSInteger)sectionNumber delegate:(id <SectionHeaderViewDelegate>)aDelegate {
    
    self = [super initWithFrame:frame];
    
    if (self != nil) {
        
        [self createGestureRecognizer];
        
        delegate = aDelegate;        
        
        self.userInteractionEnabled = YES;
        titleLabel.userInteractionEnabled = YES;
        
        section = sectionNumber;
        
        imageView = [[UIImageView alloc] initWithImage:[self getSectionImage]];
        imageView.frame = self.frame;
        [self addSubview:imageView];
        [imageView release];
        
        [self createTitleLabelWithTitle:title];
        [self addSubview:titleLabel];
        
        [self createDisclosureButton];
        [self addSubview:disclosureButton];
    }
    
    return self;
}

-(id)initForMicroWithFrame:(CGRect)frame title:(NSString*)title section:(NSInteger)sectionNumber delegate:(id <SectionHeaderViewDelegate>)aDelegate {
    
    self = [super initWithFrame:frame];
    
    if (self != nil) {
        
        delegate = aDelegate;        
        
        self.userInteractionEnabled = YES;
        
        section = sectionNumber;
        
        imageView = [[UIImageView alloc] initWithImage:[self getSectionImage]];
        imageView.frame = self.frame;
        [self addSubview:imageView];
        [imageView release];
        
        [self createSelectionCover];
        [self addSubview:selectionCover];
        
        [self createDateLabel];
        [self addSubview:dateLabel];
        
        [self createMicroTitleLabelWithTitle:title];
        [self addSubview:titleLabel];
        
        [self createDisclosureButton];
        [self addSubview:disclosureButton];
        [self.disclosureButton setSelected:NO];
        
        [self createMicroGestureRecognizers];
    }
    
    return self;
}


-(IBAction)toggleOpen:(id)sender {
    
    [self toggleOpenWithUserAction:YES];
}

-(IBAction)showLabResult:(id)sender {
    [self showSelectionCover];
    [delegate showLabResultOnMainForSection:section];
}

-(void)hideSelectionCover {
    //implemented in subclass
}

-(void)showSelectionCover {
    //implemented in subclass
}

-(void)toggleOpenWithUserAction:(BOOL)userAction {
    
    disclosureButton.selected = !disclosureButton.selected;
    
    if (userAction) {
        if (disclosureButton.selected) {
            if ([delegate respondsToSelector:@selector(sectionHeaderView:sectionOpened:)]) {
                [delegate sectionHeaderView:self sectionOpened:section];
            }
            
        } else if ([delegate respondsToSelector:@selector(sectionHeaderView:sectionClosed:)]) {
            [delegate sectionHeaderView:self sectionClosed:section];
        }
    }
}



-(void)dealloc {
    
    [titleLabel release];
    [disclosureButton release];
    [super dealloc];
}

@end
