#import "AppDelegate.h"
#import "ApplicationProperty.h"
#import "Clipboard.h"
#import "MedicationPanel_iPad.h"
#import "LabMain_iPad.h"
#import "LabPanel_iPad.h"
#import "MHPUser.h"
#import "PatientMain_iPad.h"
#import "Patient.h"
#import "PatientPanel_iPad.h"
#import "UserSession.h"


@implementation Clipboard

@synthesize contentV;
@synthesize contentVC;
@synthesize loggedInStatusL;
@synthesize patientDOBL;
@synthesize patientFullNameL;
@synthesize patientSSNL;
@synthesize titleL;
@synthesize genderL;

#pragma mark - Custom Actions
-(void)refreshContentArea {
	
	if (self.contentV != self.contentVC.view) {

		[contentVC view];
		
		CGRect frame = contentV.frame;
		[self.contentV removeFromSuperview];
		
		self.contentV=self.contentVC.view;
		self.contentV.frame = frame;
		
		/**
		 Insert above the background but below all other views
		 **/
		[self.view insertSubview:self.contentV atIndex:1];
	}
}

-(void)refreshTitleArea {
    
	Patient *selectedPatient = [UserSession selectedPatient];
	
	self.titleL.text = self.contentVC.title;
	
	if(selectedPatient) {
		self.patientDOBL.text = [NSString stringWithFormat:@"%@ (%@)", [selectedPatient dateOfBirth], [selectedPatient age]];
		self.patientFullNameL.text = [[selectedPatient displayName] capitalizedString];
		self.patientSSNL.text = [selectedPatient ssn];
        if ([selectedPatient gender] && [[selectedPatient gender] length] > 0) { 
            self.genderL.text = [[selectedPatient gender] substringWithRange:NSMakeRange(0, 1)];
        }

	} else {
		self.patientDOBL.text = @"";
		self.patientFullNameL.text = @"";
		self.patientSSNL.text = @"";
        self.genderL.text = @"";
	}
}

-(void)refreshSettingsArea {
	self.loggedInStatusL.text = [NSString stringWithFormat:@"Logged in as %@", [[UserSession mhpUser] displayName]];
}

-(void)refreshView {
	
	[self view];
		
	[self refreshTitleArea];
	[self refreshContentArea];
	[self refreshSettingsArea];	
}

-(void)setContentVC:(UIViewController *)viewController {
	
	[contentVC release];
	contentVC = [viewController retain];
	
	[viewController setClipboard:self];
	
	[self refreshView];
}

-(void)processDidChangePatient:(NSNotification *)notification {
	[self refreshView];
}

-(void)registerForNotifications {
	
	NSNotificationCenter *defaultCenter  = [NSNotificationCenter defaultCenter];
		
	[defaultCenter addObserver:self selector:@selector(processDidChangePatient:) name:UserSessionDidChangePatient object:nil];
}

-(void)unregisterFromNotifications {
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}


#pragma mark - Initialization
-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	
	self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
	
	if (self) {
		[self registerForNotifications];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[self unregisterFromNotifications];
	
	[contentVC release], contentVC = nil;
	
	[super dealloc];
}


#pragma mark - Orientation
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return UIInterfaceOrientationIsLandscape(interfaceOrientation);
}


#pragma mark - View Lifecycle
-(void)viewDidLoad {
	[super viewDidLoad];
}

-(void)viewDidAppear:(BOOL)animated {
	
	[super viewDidAppear:animated];
	
	[self.contentVC viewDidAppear:animated];
}

- (void)viewDidUnload {
	
	[contentV release], contentV = nil;
	[loggedInStatusL release], loggedInStatusL = nil;
	[patientDOBL release], patientDOBL = nil;
	[patientFullNameL release], patientFullNameL = nil;
	[patientSSNL release], patientSSNL = nil;
	[titleL release], titleL = nil;
	[genderL release], genderL = nil;
    
    [super viewDidUnload];
}


-(IBAction)launchFeedback:(id)sender {

	[AppDelegate launchFeedback];
}

@end