//  LabTestGraphViewController_iPad.m
//  PatientViewer
//
//  Created by Matt Tea on 3/22/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "LabMain_iPad.h"
#import "LabMainContainer.h"
#import "LabTest.h"
#import "UserSession.h"

@implementation LabMain_iPad


#pragma mark - Custom Actions

-(CGFloat)getGraphWidth {
    return 573;
}

-(void)drawVerticalSelectionLineInSpace:(CPPlotSpace *)space withEvent:(id)event atPoint:(CGPoint)point  {
    
    const static float TextLayerPointYLocation = 445.0;
    
    [self clearPlotAnnotations];
    CPScatterPlot *plot = (CPScatterPlot *)[scatterPlotGraph  plotWithIdentifier:@"labResult"]; 
    
    NSUInteger index = [plot indexOfVisiblePointClosestToPlotAreaPointByXValues:[plot.graph convertPoint:point toLayer:plot]]; 
    
    CGPoint textLayerPoint = CGPointMake(0, TextLayerPointYLocation);
    NSDecimal *decimalPoint = malloc(sizeof(NSDecimal) * 2);
    textLayerPoint = [plot.graph convertPoint:textLayerPoint toLayer:plot];
    
    [plot.plotSpace plotPoint:decimalPoint forPlotAreaViewPoint:textLayerPoint];
    
    
    NSDecimal decimalPointY = decimalPoint[1];
    
    NSDecimalNumber *decimalNumberY = [NSDecimalNumber decimalNumberWithDecimal:decimalPointY];
    
    
    NSArray *textAnchorPointArray = [NSArray arrayWithObjects:[self numberForPlot:plot field:CPScatterPlotFieldX recordIndex:index], decimalNumberY , nil]; 
    [self addTextAnnotationsToPlot:plot atIndex:index withAnchorPointArray:textAnchorPointArray];    
    
    
    NSArray *dataAnchorPointArray = [NSArray arrayWithObjects:[self numberForPlot:plot field:CPScatterPlotFieldX recordIndex:index], [self numberForPlot:plot field:CPScatterPlotFieldY recordIndex:index], nil]; 
    
    [self addLineAnnotationToPlot:plot withAnchorPointArray:dataAnchorPointArray withIndex:index];    
    
    [self addPointSymbolToPlot:plot withAnchorPointArray:dataAnchorPointArray];
    
    [super drawVerticalSelectionLineInSpace:space withEvent:event atPoint:point];
}



-(CGFloat)getLayerWidth {
    return 150.0;
}

-(CGFloat)getLayerHeight {
    return 26.0;
}

-(CGFloat)getTextFontSize {
    return 18.0;
}



-(void)processDidChangePatient:(NSNotification *)notification {
	shouldRefresh = YES;
    [super processDidChangePatient:notification];
}

-(void)processDidChangeLabTest:(NSNotification *)notification {
    
	LabTest *labTest = [UserSession selectedLabTest];
	
	[self hideGraphs];
	
	if (labTest) {
		[self showLabTest:labTest];
	}
    
    [super processDidChangeLabTest:notification];
}

-(void)processWillChangeLabTest:(NSNotification *)notification {
	[self clearGraphs];
}

-(void)processDidChangeLabResult:(NSNotification *)notification {
    
    [super processDidChangeLabResult:notification];
}

-(void)setPaddingForGraph:(CPXYGraph *)graph {
    
	//static CGFloat GraphPadding = 10.0;
	
    graph.paddingTop = 5.0;
    graph.paddingLeft = 11.0;
    graph.paddingRight = 13.0;
    graph.paddingBottom = 0.0;
    
    graph.plotAreaFrame.paddingTop = 13.0;
    graph.plotAreaFrame.paddingBottom = 18.0;
    graph.plotAreaFrame.paddingLeft = 35.0;
    graph.plotAreaFrame.paddingRight = 35.0;
}

-(void)reloadHistogramGraph {
	
	static CGFloat HistogramWidth = 664;
	
	CGFloat height = 80*[self.labTestGraphContainer numberOfMedications] + 50;
	
	self.histogramSV.contentSize = CGSizeMake(HistogramWidth, height);
	self.histogramGHV.frame = CGRectMake(0, 0, HistogramWidth, height);
	
	[super reloadHistogramGraph];
}

-(void)setGraphIntervals {
    [super setGraphIntervals];

    NSTimeInterval dateRange = [self.labTestGraphContainer dateRange];
    
    
    switch (self.labTestGraphContainer.currentRange) {
        case LabTestRangeDay:
            self.xUnit = (oneDay/4);
            [self.dateFormatter setDateFormat:@"HH:mm"]; 
            break;
            
        case LabTestRangeWeek:
            self.xUnit = (oneDay);
            [self.dateFormatter setDateFormat:@"MM/dd"];
            break;
            
        case LabTestRangeMonth:
            self.xUnit = (oneDay*6);
            [self.dateFormatter setDateFormat:@"MM/dd"];
            break;
            
        case LabTestRangeYear:
            self.xUnit = (oneDay*73);
            [self.dateFormatter setDateFormat:@"MM/dd/YY"];
            break;
            
        case LabTestRangeTwoYear:
            self.xUnit = (oneDay*146);
            [self.dateFormatter setDateFormat:@"MM/dd/YY"];
            break;
            
        case LabTestRangeAll:
            self.xUnit = dateRange/10;
            [self.dateFormatter setDateFormat:@"MM/dd/YY"];
            break;
            
        default:
            self.xUnit = dateRange/10;
            [self.dateFormatter setDateFormat:@"MM/dd/YY"];
            break;
    }
    
}

@end