//  MainNavigationController.m
//  PatientViewer
//
//  Created by Grant Oladipo on 2/22/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "Clipboard.h"
#import "Connection.h" 
#import "MedicationMain_iPad.h"
#import "MedicationPanel_iPad.h"
#import "LabMain_iPad.h"
#import "LabPanel_iPad.h"
#import "LoginMain_iPad.h"
#import "MainNavigation.h"
#import "NavigationSideBar.h"
#import "PatientMain_iPad.h"
#import "PatientPanel_iPad.h"
#import "LoginDisclaimer_iPad.h"

#pragma mark - Constants
#define HiddenPanelRect				CGRectMake(0,0,0,0)
#define LeftPanelRect				CGRectMake(0,0,60,748)
#define MiddlePanelRect				CGRectMake(60,0,300,748)
#define FullRightPanelRect			CGRectMake(0,0,1024,748)
#define ExpandedRightPanelRect		CGRectMake(60,0,964,748)
#define RightPanelRect				CGRectMake(360,0,664,748)

@implementation MainNavigationController

@synthesize leftPanelV;
@synthesize middlePanelV;
@synthesize rightPanelV;

@synthesize leftPanelVC;
@synthesize middlePanelVC;
@synthesize rightPanelVC;
@synthesize newLeftPanelVC;
@synthesize newMiddlePanelVC;
@synthesize newRightPanelVC;

@synthesize clipboard;
@synthesize medicationMain;
@synthesize medicationPanel;
@synthesize labMain;
@synthesize labPanel;
@synthesize loginMain;
@synthesize navigationBar;
@synthesize patientMain;
@synthesize patientPanel;

@synthesize viewMode;
@synthesize clipboardState;
@synthesize leftPanelState;
@synthesize middlePanelState;
@synthesize rightPanelState;


#pragma mark - Login Disclaimer
-(void)showDisclaimer {
    LoginDisclaimer_iPad *disclaimer = [[LoginDisclaimer_iPad alloc] initWithNibName:@"LoginDisclaimer_iPad" bundle:[NSBundle bundleForClass:[self class]]];
    UINavigationController *navController = [[UINavigationController alloc]
											 initWithRootViewController:disclaimer];
    
    navController.modalPresentationStyle = UIModalPresentationFormSheet;
    [self presentModalViewController:navController animated:YES];
    
    [navController release];
    [disclaimer release];
}

#pragma mark - Cached View Controllers
-(id)viewControllerFromCacheWithNibName:(NSString *)nibName class:(Class)class {
	
	UIViewController *viewController = [cacheDict objectForKey:class];
	
	if (viewController) {
		return viewController;
	}

	viewController = [[class alloc] initWithNibName:nibName bundle:[NSBundle bundleForClass:[self class]]];
	[cacheDict setObject:viewController forKey:class];
	return [viewController autorelease];
}

-(Clipboard *)clipboard {
    NSString *nibName = @"Clipboard";
	return [self viewControllerFromCacheWithNibName:nibName class:[Clipboard class]];
}

-(MedicationPanel_iPad *)medicationPanel {
    NSString *nibName = @"MedicationPanel_iPad";
	return [self viewControllerFromCacheWithNibName:nibName class:[MedicationPanel_iPad class]];
}

-(MedicationMain_iPad *)medicationMain {
    NSString *nibName = @"MedicationMain_iPad";
	return [self viewControllerFromCacheWithNibName:nibName class:[MedicationMain_iPad class]];
}

-(LabMain_iPad *)labMain {
    NSString *nibName = @"LabMain_iPad";
	return [self viewControllerFromCacheWithNibName:nibName class:[LabMain_iPad class]];
}

-(LabPanel_iPad *)labPanel {
    NSString *nibName = @"LabPanel_iPad";
	return [self viewControllerFromCacheWithNibName:nibName class:[LabPanel_iPad class]];
}

-(LoginMain_iPad *)loginMain {
    NSString *nibName = @"LoginMain_iPad";
	return [self viewControllerFromCacheWithNibName:nibName class:[LoginMain_iPad class]];
}

-(NavigationSideBar *)navigationBar {
    NSString *nibName = @"NavigationSideBar";
	return [self viewControllerFromCacheWithNibName:nibName class:[NavigationSideBar class]];
}

-(PatientMain_iPad *)patientMain {
    NSString *nibName = @"PatientMain_iPad";
	return [self viewControllerFromCacheWithNibName:nibName class:[PatientMain_iPad class]];
}

-(PatientPanel_iPad *)patientPanel {
    NSString *nibName = @"PatientPanel_iPad";
    return [self viewControllerFromCacheWithNibName:nibName class:[PatientPanel_iPad class]];
}


#pragma mark - Cached Panel Views
-(UIView *)viewPanel:(NSString *)panelName fromCacheBelowSubview:(UIView *)subView {
	
	UIView *panelView = [cacheDict objectForKey:panelName];
	
	if (panelView) {
		return panelView;
	}
		
	panelView = [[UIView alloc] init];
	panelView.clipsToBounds = YES;
	
	if (subView) {
		[self.view insertSubview:panelView belowSubview:subView];
	} else {
		[self.view addSubview:panelView];
	}
	
	[cacheDict setObject:panelView forKey:panelName];
	
	return [panelView autorelease];
}

-(UIView *)leftPanelV {
    NSString *panelName = @"left";
	return [self viewPanel:panelName fromCacheBelowSubview:self.middlePanelV];
}

-(UIView *)middlePanelV {
    NSString *panelName = @"middle";
	return [self viewPanel:panelName fromCacheBelowSubview:self.rightPanelV];
}

-(UIView *)rightPanelV {
    NSString *panelName = @"right";
	return [self viewPanel:panelName fromCacheBelowSubview:nil];
}


#pragma mark - View Controller Handles
-(void)setPanelVC:(UIViewController **)oldViewController newViewController:(UIViewController *)newViewController {
	
	[*oldViewController release];
	[newViewController view];
	*oldViewController = newViewController;
	[*oldViewController retain];
	[*oldViewController setMainNavigationController:self];
}

-(void)setLeftPanelVC:(UIViewController *)viewController {
	[self setPanelVC:&leftPanelVC newViewController:viewController];
}

-(void)setMiddlePanelVC:(UIViewController *)viewController {
	[self setPanelVC:&middlePanelVC newViewController:viewController];
}

-(void)setRightPanelVC:(UIViewController *)viewController {
	[self setPanelVC:&rightPanelVC newViewController:viewController];
}

-(void)setNewLeftPanelVC:(UIViewController *)viewController {
	
	[newLeftPanelVC release];
	newLeftPanelVC = [viewController retain];
}

-(void)setNewMiddlePanelVC:(UIViewController *)viewController {
	
	[newMiddlePanelVC release];
	newMiddlePanelVC = [viewController retain];
}

-(void)setNewRightPanelVC:(UIViewController *)viewController {

	[newRightPanelVC release];
	newRightPanelVC = [viewController retain];
}


#pragma mark - Custom Actions
-(BOOL)isLoginMode {
	return (viewMode == MainNavigationViewModeLogin);
}

-(void)setStatesandControllersForLeftPanel:(UIViewController *)newLeftPanelViewController
							   middlePanel:(UIViewController *)newMiddlePanelViewController
								rightPanel:(UIViewController *)newRightPanelViewController {
	
	/**
    currently if you are in login mode you can determine state of clipboard and panels and the same vice versa
	When other modes are introduced, this test needs to be revisited
    **/
    
	if ([self isLoginMode]) {
		
		self.clipboardState = MainNavigationPanelStateHidden;
		self.leftPanelState = MainNavigationPanelStateHidden;
		self.middlePanelState = MainNavigationPanelStateHidden;
		self.rightPanelState = MainNavigationPanelStateFull;

	} else {

		self.clipboardState = MainNavigationPanelStateDefault;
		self.leftPanelState = MainNavigationPanelStateDefault;
		self.middlePanelState = MainNavigationPanelStateDefault;
		self.rightPanelState = MainNavigationPanelStateDefault;
	}
	
	self.newLeftPanelVC = newLeftPanelViewController;
	self.newMiddlePanelVC = newMiddlePanelViewController;
	self.newRightPanelVC = newRightPanelViewController;
}

-(void)setViewMode:(MainNavigationControllerViewMode)newViewMode animated:(BOOL)animated {
	
	viewMode = newViewMode;
	
	if (shouldClearCache) {
		
		[cacheDict removeAllObjects];
		[self.view removeAllSubviews];
		
		shouldClearCache = NO;
	}
	
	switch (viewMode) {

		case MainNavigationViewModeLogin:
			[self setStatesandControllersForLeftPanel:nil
										  middlePanel:nil 
										   rightPanel:self.loginMain];
			break;
			
		case MainNavigationViewModePatient:
			[self setStatesandControllersForLeftPanel:self.navigationBar
										  middlePanel:self.patientPanel
										   rightPanel:self.patientMain];			
			break;
			
		case MainNavigationViewModeLabTest:
			[self setStatesandControllersForLeftPanel:self.navigationBar
										  middlePanel:self.labPanel
										   rightPanel:self.labMain];			
			break;
		case MainNavigationViewModeMedication:
			[self setStatesandControllersForLeftPanel:self.navigationBar
										  middlePanel:self.medicationPanel
										   rightPanel:self.medicationMain];
	}
	/**	
     Left Panel
	**/
    UIView *panelView = self.leftPanelV;
	
	[self.leftPanelVC viewWillDisappear:animated];
	
	[panelView removeAllSubviews];
	
	[self.leftPanelVC viewDidDisappear:animated];
	
	self.leftPanelVC = newLeftPanelVC;
	
	switch (self.leftPanelState) {
		case MainNavigationPanelStateHidden:
			panelView.frame = HiddenPanelRect;
			break;
		default:
			panelView.frame = LeftPanelRect;
			break;
	}
	
	if (self.leftPanelVC) {
		self.leftPanelVC.view.frame = panelView.bounds;
	}
	
	[self.leftPanelVC viewWillAppear:animated];
	
	[panelView addSubview:self.leftPanelVC.view];
	
	[self.leftPanelVC viewDidAppear:animated];
	
	/**
     Middle Panel
	**/
    panelView = self.middlePanelV;
	
    
	[self.middlePanelVC viewWillDisappear:animated];

	[panelView removeAllSubviews];
		
	[self.middlePanelVC viewDidDisappear:animated];
	
	self.middlePanelVC = newMiddlePanelVC;
	
	switch (self.middlePanelState) {
		case MainNavigationPanelStateHidden:
			panelView.frame = HiddenPanelRect;
			break;
		default:
			panelView.frame = MiddlePanelRect;
			break;
	}
	
	if (self.middlePanelVC) {
		self.middlePanelVC.view.frame = panelView.bounds;
	}
	
	[self.middlePanelVC viewWillAppear:animated];
	
	[panelView addSubview:self.middlePanelVC.view];
	
    
	[self.middlePanelVC viewDidAppear:animated];

	/**
	 Right Panel
	**/
    panelView = self.rightPanelV;
	
	[self.rightPanelVC viewWillDisappear:animated];

	[panelView removeAllSubviews];	
	
	[self.rightPanelVC viewDidDisappear:animated];
	
	if (self.clipboardState != MainNavigationPanelStateHidden) {
		
		self.clipboard.contentVC = newRightPanelVC;
		self.rightPanelVC = self.clipboard;
		
	} else {
		self.rightPanelVC = newRightPanelVC;
	}
	
	switch (self.rightPanelState) {
		case MainNavigationPanelStateFull:
			panelView.frame = FullRightPanelRect;
			break;
		default:
			panelView.frame = RightPanelRect;
			break;
	}
	
	if (self.rightPanelVC) {
		self.rightPanelVC.view.frame = panelView.bounds;
	}
	
	[self.rightPanelVC viewWillAppear:animated];
	
	[panelView addSubview:self.rightPanelVC.view];
	
	[self.rightPanelVC viewDidAppear:animated]; 
	
}

-(void)setViewMode:(MainNavigationControllerViewMode)newViewMode {
	[self setViewMode:newViewMode animated:NO];
}

-(void)collapseMiddlePanel:(BOOL)animated {
	//TODO: COLLAPSE MIDDLE PANEL
}

-(void)expandMiddlePanel:(BOOL)animated {
	//TODO: EXPAND MIDDLE PANEL
}


#pragma mark - Initialization
-(void)processShouldWipeData:(NSNotification *)notification {
	
	[self setViewMode:MainNavigationViewModeLogin];
	
	shouldClearCache = YES;
}

-(void)registerForNotifications {
	
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	
	[defaultCenter addObserver:self selector:@selector(processShouldWipeData:) name:ConnectionShouldWipeData object:nil];
}

-(void)unregisterFromNotifications {
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

-(id)initWithViewMode:(MainNavigationControllerViewMode)startingViewMode {
	
	self = [super init];
	
	if (self) {
		
		cacheDict = [[NSMutableDictionary alloc] initWithCapacity:1];
		
		/*
		 Sets up the initial views/controllers of the main navigation controller, all initializiation should be done before
		 */
        self.viewMode = MainNavigationViewModeLogin;
		
		[self registerForNotifications];
	}
	
	return self;
}

-(void)loadView {

	UIView *mainView = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] bounds]];		
	self.view = mainView;
	[mainView release];
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[self unregisterFromNotifications];

	[cacheDict release], cacheDict = nil;
	
	[leftPanelVC release], leftPanelVC = nil;
	[middlePanelVC release], middlePanelVC = nil;
	[rightPanelVC release], rightPanelVC = nil;
	
	[super dealloc];
}

-(void)didReceiveMemoryWarning {
		
	[cacheDict removeAllObjects];
	
    [super didReceiveMemoryWarning];
}


#pragma mark - Orientation
-(void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
	
	[leftPanelVC didRotateFromInterfaceOrientation:fromInterfaceOrientation];
	[middlePanelVC didRotateFromInterfaceOrientation:fromInterfaceOrientation];
	[rightPanelVC didRotateFromInterfaceOrientation:fromInterfaceOrientation];
}

-(BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {	
	
	[leftPanelVC shouldAutorotateToInterfaceOrientation:interfaceOrientation];
	[middlePanelVC shouldAutorotateToInterfaceOrientation:interfaceOrientation];
	[rightPanelVC shouldAutorotateToInterfaceOrientation:interfaceOrientation];
	
	return UIInterfaceOrientationIsLandscape(interfaceOrientation);
}


#pragma mark - View Lifecycle
-(void)viewDidLoad {
	[super viewDidLoad];
    
    self.middlePanelV.clipsToBounds = NO;
}

@end