//
//  LabTestGraphViewController_iPhone.m
//  PatientViewer
//
//  Created by Tea, Matt on 4/19/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx
//

#import "AppDelegate.h"
#import "LabMain_iPhone.h"
#import "LabMainContainer.h"
#import "LabTest.h"
#import "Patient.h"
#import "UserSession.h"

@implementation LabMain_iPhone


#pragma mark - Custom Actions
-(CGFloat)getGraphWidth {
    return 261;
}

-(void)refreshTitleView {
    Patient *thePatient = [UserSession selectedPatient];
    
    nameL.text =    [thePatient displayName];
    ssnL.text =     [thePatient ssn];
    dobL.text =     [NSString stringWithFormat:@"%@ (%@)",[thePatient dateOfBirth], [thePatient age]];
    
    if ([thePatient gender]) { 
        genderL.text = [[thePatient gender] substringWithRange:NSMakeRange(0, 1)];
    }
}

-(void)drawVerticalSelectionLineInSpace:(CPPlotSpace *)space withEvent:(id)event atPoint:(CGPoint)point  {
    
    const static float TextLayerPointYLocation = 232.0;
    
    [self clearPlotAnnotations];
    CPScatterPlot *plot = (CPScatterPlot *)[scatterPlotGraph  plotWithIdentifier:@"labResult"]; 
    
    NSUInteger index = [plot indexOfVisiblePointClosestToPlotAreaPointByXValues:[plot.graph convertPoint:point toLayer:plot]]; 
    
    CGPoint textLayerPoint = CGPointMake(0, TextLayerPointYLocation);
    NSDecimal *decimalPoint = malloc(sizeof(NSDecimal) * 2);
    textLayerPoint = [plot.graph convertPoint:textLayerPoint toLayer:plot];
    
    [plot.plotSpace plotPoint:decimalPoint forPlotAreaViewPoint:textLayerPoint];
    
    
    NSDecimal decimalPointY = decimalPoint[1];
    
    NSDecimalNumber *decimalNumberY = [NSDecimalNumber decimalNumberWithDecimal:decimalPointY];
    
    
    NSArray *textAnchorPointArray = [NSArray arrayWithObjects:[self numberForPlot:plot field:CPScatterPlotFieldX recordIndex:index], decimalNumberY , nil]; 
    [self addTextAnnotationsToPlot:plot atIndex:index withAnchorPointArray:textAnchorPointArray];    
    
    
    NSArray *dataAnchorPointArray = [NSArray arrayWithObjects:[self numberForPlot:plot field:CPScatterPlotFieldX recordIndex:index], [self numberForPlot:plot field:CPScatterPlotFieldY recordIndex:index], nil]; 
    [self addLineAnnotationToPlot:plot withAnchorPointArray:dataAnchorPointArray withIndex:index];    
    
    [self addPointSymbolToPlot:plot withAnchorPointArray:dataAnchorPointArray];
    
    [super drawVerticalSelectionLineInSpace:space withEvent:event atPoint:point];
}


-(CGFloat)getLayerWidth {
    return 80.0;
}

-(CGFloat)getLayerHeight {
    return 18.0;
}

-(CGFloat)getTextFontSize {
    return 12.0;
}

-(void)processDidChangePatient:(NSNotification *)notification {
    [self.navigationController popToRootViewControllerAnimated:NO];
	shouldRefresh = YES;
    [super processDidChangePatient:notification];
}

-(void)processDidChangeLabTest:(NSNotification *)notification {
    shouldRefresh = YES;
    [super processDidChangeLabTest:notification];
    
    self.title = [[UserSession selectedLabTest] testname];     
    [self showLabTest:[UserSession selectedLabTest]];
}

-(void)processDidChangeLabResult:(NSNotification *)notification {
    [super processDidChangeLabResult:notification];
    
}

-(void)reloadHistogramGraph {
	
    static CGFloat Multiplier = 60.0;
    static CGFloat Buffer = 50.0;
    static CGFloat Width = 320.0;
    
	CGFloat height = Multiplier*[self.labTestGraphContainer numberOfMedications]+Buffer;
	
	self.histogramSV.contentSize = CGSizeMake(Width, height);

	self.histogramGHV.frame = CGRectMake(0, 0, Width, height);
	
	
	[super reloadHistogramGraph];
}

-(void)setPaddingForGraph:(CPXYGraph *)graph {
    static CGFloat SmallPadding = 25.0;
    static CGFloat LeftPadding = 35.0;
    
    graph.paddingTop = 0.0;
    graph.paddingLeft = 0.0;
    graph.paddingRight = 0.0;
    graph.paddingBottom = 0.0;
    
    graph.plotAreaFrame.paddingTop = SmallPadding;
    graph.plotAreaFrame.paddingBottom = SmallPadding;
    graph.plotAreaFrame.paddingLeft = LeftPadding;
    graph.plotAreaFrame.paddingRight = SmallPadding;
}

-(void)setGraphIntervals {

    [super setGraphIntervals];
    
    
    NSTimeInterval dateRange = [self.labTestGraphContainer dateRange];
    
    
    switch (self.labTestGraphContainer.currentRange) {
        case LabTestRangeDay:
            self.xUnit = (oneDay/4);
            [self.dateFormatter setDateFormat:@"HH:mm"]; 
            break;
            
        case LabTestRangeWeek:
            self.xUnit = (oneDay);
            [self.dateFormatter setDateFormat:@"MM/dd"];
            break;
            
        case LabTestRangeMonth:
            self.xUnit = (oneDay*6);
            [self.dateFormatter setDateFormat:@"MM/dd"];
            break;
            
        case LabTestRangeYear:
            self.xUnit = (oneDay*73);
            [self.dateFormatter setDateFormat:@"MM/dd/YY"];
            break;
            
        case LabTestRangeTwoYear:
            self.xUnit = (oneDay*146);
            [self.dateFormatter setDateFormat:@"MM/dd/YY"];
            break;
            
        case LabTestRangeAll:
            self.xUnit = dateRange/5;
            [self.dateFormatter setDateFormat:@"MM/dd/YY"];
            break;
            
        default:
           self.xUnit = dateRange/5;
            [self.dateFormatter setDateFormat:@"MM/dd/YY"];
            break;
    }

}


#pragma mark - Orientation
-(BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return UIInterfaceOrientationIsPortrait(interfaceOrientation);
}

#pragma mark - View Lifecycle
-(void)viewDidLoad {
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Feedback" 
																			   style:UIBarButtonItemStylePlain 
																			  target:[AppDelegate class] 
																			  action:@selector(launchFeedback)] autorelease];
}

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self refreshTitleView];
    
}

#pragma mark - Custom Actions

-(void)showLabTest:(LabTest *)theLabTest {

	self.histogramGHV.hidden = YES;
	self.scatterPlotGHV.hidden = NO;
	self.noMedicationsL.hidden = YES;
    
	self.inpatientB.selected = NO;
	self.outpatientB.selected = NO;	
    
	labTestGraphContainer.currentRange = LabTestRangeTwoYear;
    
    [super showLabTest:theLabTest];
}

@end