//
//  LabTestPanelViewController_iPhone.m
//  PatientViewer
//
//  Created by Tea, Matt on 3/31/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx
//

#import "AppDelegate_iPhone.h"
#import "LabMain_iPhone.h"
#import "LabPanel_iPhone.h"
#import "LabPanelContainer.h"
#import "LabResult.h"
#import "LabTest.h"
#import "Link.h"
#import "MetricsPusher.h"
#import "Patient.h"
#import "PatientMain_iPhone.h"
#import "SectionInfo.h"
#import "UserSession.h"


@implementation LabPanel_iPhone


#pragma mark Custom Accessors
-(UITableViewCell *)customizedCellForRowAtIndexPath:(NSIndexPath *)indexPath inTableView:(UITableView *)tableView {
	
    if (tableView == self.listTV) {
        
        LabTest *labTest = [labPanelContainer labTestAtIndex:indexPath.row inGroupSection:indexPath.section];
        LabResult *labResult = labTest.mostRecentLabResult;
        
        if ([[labResult abnormalIndicatorTriFlag] isEqualToString:@"true"]) {
            [[NSBundle mainBundle] loadNibNamed:@"AbnormalCell_iPhone" owner:self options:nil];
        } else if ([[labResult abnormalIndicatorTriFlag] isEqualToString:@"false"]) {
            [[NSBundle mainBundle] loadNibNamed:@"NormalCell_iPhone" owner:self options:nil];
        } else {
            [[NSBundle mainBundle] loadNibNamed:@"UnknownCell_iPhone" owner:self options:nil];
        }
        
        
    } else {
        [[NSBundle mainBundle] loadNibNamed:@"NormalCell_iPhone" owner:self options:nil];
        
    }
    
	UITableViewCell *cell = [self.customTVC retain];
	self.customTVC = nil;	
	
	return [cell autorelease];
}


#pragma mark Custom Actions
-(void)refreshTitleView {
    Patient *thePatient = [UserSession selectedPatient];
    
    nameL.text =    [thePatient displayName];
    ssnL.text =     [thePatient ssn];
    dobL.text =     [NSString stringWithFormat:@"%@ (%@)",[thePatient dateOfBirth], [thePatient age]];
    
    if ([thePatient gender]) { 
        genderL.text = [[thePatient gender] substringWithRange:NSMakeRange(0, 1)];
    }
}

#pragma mark - Orientation
-(BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return UIInterfaceOrientationIsPortrait(interfaceOrientation);
}


#pragma mark - UITableViewDelegate Methods


-(void)displayLabTestAtIndexPath:(NSIndexPath *)indexPath {
    
    if ([labPanelContainer hasLabTestGroups]) {
        
        LabMain_iPhone *labMain_iPhone = [[LabMain_iPhone alloc] initWithNibName:@"LabMain_iPhone" bundle:[NSBundle bundleForClass:[self class]]];
        [self.navigationController pushViewController:labMain_iPhone animated:YES];
        
        [labMain_iPhone release];
        
        LabTest *labTest = [labPanelContainer labTestAtIndex:indexPath.row inGroupSection:indexPath.section];
        
        NSDictionary *tagDict = [NSDictionary dictionaryWithObjectsAndKeys:[[[[labTest links] objectForKey:LabTestLabResultsKey] href] absoluteString], 
                                 @"LabResults.uri", nil];
        
        [MetricsPusher reportMetricsWithUserEventValue:@"SelectLabTestOnLabViewer" tagDictionary:tagDict];
        
        
        [UserSession setSelectedLabTest:labTest];
        
    }
}

-(void)displayLabResultAtIndexPath:(NSIndexPath *)indexPath {
    
    if ([labPanelContainer hasLabResultGroups]) {
        LabMain_iPhone *labMain_iPhone = [[LabMain_iPhone alloc] initWithNibName:@"LabMain_iPhone" bundle:[NSBundle bundleForClass:[self class]]];
        [self.navigationController pushViewController:labMain_iPhone animated:YES];
        
        [labMain_iPhone release];
        
        Patient *patient = [UserSession selectedPatient];
        
        NSDictionary *tagDict = [NSDictionary dictionaryWithObjectsAndKeys:[[[[patient links] objectForKey:PatientMicrobiologyLabResultsKey] href] absoluteString], 
                                 @"MicroLabResult.uri", nil];
        
        [MetricsPusher reportMetricsWithUserEventValue:@"SelectMicroLabResultOnLabViewer" tagDictionary:tagDict];
        
        LabResult *labResult = [labPanelContainer microLabResultAtIndex:indexPath.row inGroupSection:indexPath.section];
        [UserSession setSelectedLabResult:labResult];
        
    }
    
}

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (tableView == self.listTV) {
        [self displayLabTestAtIndexPath:indexPath];
    } else {
        self.selectedIndexPath = indexPath;
        [self clearSectionHeaderCovers];
        [self displayLabResultAtIndexPath:indexPath];
    }
}



#pragma mark  - Section Header Delegate
-(void)sectionHeaderView:(SectionHeaderView *)sectionHeaderView sectionOpened:(NSInteger)sectionOpened {
        
    if (self.currentTab == ChemHem) {
        
        SectionInfo *sectionInfo = [sectionInfoArray objectAtIndex:sectionOpened];
        
        sectionInfo.open = YES;
        
        
        NSInteger countOfRowsToInsert = [labPanelContainer numberOfLabTestsForGroupSection:sectionOpened];
        NSMutableArray *indexPathsToInsert = [[NSMutableArray alloc] init];
        for (NSInteger i = 0; i < countOfRowsToInsert; i++) {
            [indexPathsToInsert addObject:[NSIndexPath indexPathForRow:i inSection:sectionOpened]];
        }
        
        
        UITableViewRowAnimation insertAnimation;
        
        insertAnimation = UITableViewRowAnimationTop;
        
        UITableView *tableView = self.listTV;
        
        [tableView beginUpdates];
        
        [tableView insertRowsAtIndexPaths:indexPathsToInsert withRowAnimation:insertAnimation];
        [tableView endUpdates];
        
        [indexPathsToInsert release];
    } else if (self.currentTab == Microbiology) {
        
        
        SectionInfo *sectionInfo = [microSectionInfoArray objectAtIndex:sectionOpened];
        
        sectionInfo.open = YES;
        
        
        NSInteger countOfRowsToInsert = [labPanelContainer numberofLabResultsForGroupSection:sectionOpened];
        NSMutableArray *indexPathsToInsert = [[NSMutableArray alloc] init];
        for (NSInteger i = 0; i < countOfRowsToInsert; i++) {
            [indexPathsToInsert addObject:[NSIndexPath indexPathForRow:i inSection:sectionOpened]];
        }
        
        UITableViewRowAnimation insertAnimation;
        
        insertAnimation = UITableViewRowAnimationTop;
        
        UITableView *tableView = self.microTV;
        
        [tableView beginUpdates];
        
        [tableView insertRowsAtIndexPaths:indexPathsToInsert withRowAnimation:insertAnimation];
        [tableView endUpdates];
        
        [indexPathsToInsert release];
    }

}


-(void)sectionHeaderView:(SectionHeaderView *)sectionHeaderView sectionClosed:(NSInteger)sectionClosed {
    
    if (self.currentTab == ChemHem) {
        
        SectionInfo *sectionInfo = [sectionInfoArray objectAtIndex:sectionClosed];
        
        sectionInfo.open = NO;
        NSInteger countOfRowsToDelete = [labPanelContainer numberOfLabTestsForGroupSection:sectionClosed];
        
        if (countOfRowsToDelete > 0) {
            NSMutableArray *indexPathsToDelete = [[NSMutableArray alloc] init];
            for (NSInteger i = 0; i < countOfRowsToDelete; i++) {
                [indexPathsToDelete addObject:[NSIndexPath indexPathForRow:i inSection:sectionClosed]];
            }
            [self.listTV beginUpdates];
            [self.listTV deleteRowsAtIndexPaths:indexPathsToDelete withRowAnimation:UITableViewRowAnimationTop];
            
            [self.listTV endUpdates];
            
            [indexPathsToDelete release];
        }
    } else if (self.currentTab == Microbiology) {
        
        SectionInfo *sectionInfo = [microSectionInfoArray objectAtIndex:sectionClosed];
        
        sectionInfo.open = NO;
        NSInteger countOfRowsToDelete = [labPanelContainer numberofLabResultsForGroupSection:sectionClosed];
        
        if (countOfRowsToDelete > 0) {
            NSMutableArray *indexPathsToDelete = [[NSMutableArray alloc] init];
            for (NSInteger i = 0; i < countOfRowsToDelete; i++) {
                [indexPathsToDelete addObject:[NSIndexPath indexPathForRow:i inSection:sectionClosed]];
            }
            [self.microTV beginUpdates];
            [self.microTV deleteRowsAtIndexPaths:indexPathsToDelete withRowAnimation:UITableViewRowAnimationTop];
            
            [self.microTV endUpdates];
            
            [indexPathsToDelete release];
        }
    }
    
}


-(UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
    if (tableView == self.listTV) {
        
        if ([labPanelContainer hasLabTestGroups]) {
            
            SectionInfo *sectionInfo = [sectionInfoArray objectAtIndex:section];
            if (!sectionInfo.headerView) {
                NSString *groupName = [[labPanelContainer labTestGroupAtIndex:section] groupName];
                sectionInfo.headerView = [[[SectionHeaderView_iPhone alloc] initWithFrame:CGRectMake(0.0, 0.0, self.listTV.bounds.size.width, self.listTV.sectionHeaderHeight) title:groupName section:section delegate:self] autorelease];
            }
            
            return sectionInfo.headerView;
        }
        else { 
            return nil;
        }
    } else {
        if ([labPanelContainer hasLabResultGroups]) {
            
            SectionInfo *sectionInfo = [microSectionInfoArray objectAtIndex:section];
            if (!sectionInfo.headerView) {
                NSString *groupName = [[labPanelContainer labResultGroupAtIndex:section] groupName];

                sectionInfo.headerView = [[[SectionHeaderView_iPhone alloc] initForMicroWithFrame:CGRectMake(0.0, 0.0, self.microTV.bounds.size.width, self.microTV.sectionHeaderHeight) title:groupName section:section delegate:self] autorelease];
            }
            
            return sectionInfo.headerView;
        }
        else { 
            return nil;
        }
    }
}


#pragma mark - View Lifecycle
-(void)viewDidLoad {
    [super viewDidLoad];
    self.navigationController.navigationBar.topItem.title = @"Lab Results";
   
    UINavigationBar *bar = [self.navigationController navigationBar]; 
    [bar setTintColor:[UIColor colorWithRawValuesForRed:175 green:129 blue:29 alpha:1.0]];     

	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Feedback" 
																			   style:UIBarButtonItemStylePlain 
																			  target:[AppDelegate class] 
																			  action:@selector(launchFeedback)] autorelease];
}

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    [self refreshTitleView];
}

@end