//
//  patientMainDetailViewController_iPhone.m
//  PatientViewer
//
//  Created by Tea, Matt on 4/5/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "Allergy.h"
#import "Link.h"
#import "Medication.h"
#import "MedicationPanel_iPhone.h"
#import "MetricsPusher.h"
#import "Patient.h"
#import "PatientMainContainer.h"
#import "PatientMainContainerDelegate.h"
#import "PatientMain_iPhone.h"
#import "Problem.h"
#import "SectionHeaderView_iPhone.h"
#import "SectionInfo.h"
#import "UserSession.h"


@implementation MedicationPanel_iPhone

@synthesize patientMainContainer;
@synthesize detailTVC;
@synthesize medicationMainViewController;
@synthesize hasMedications;

#pragma mark - Custom Actions
-(NSInteger)getNumberOfRowsForGroupInSection:(NSInteger)section {
    return [super getNumberOfRowsForGroupInSection:section];
}

-(void)retrieveResults {
    [super retrieveResults];

}


-(void)processDidChangePatient:(NSNotification *)notification {
	shouldRefresh = YES;
}

-(void)refreshTitleView {
    Patient *thePatient = [UserSession selectedPatient];
    
    nameL.text =    [thePatient displayName];
    ssnL.text =     [thePatient ssn];
    dobL.text =     [NSString stringWithFormat:@"%@ (%@)",[thePatient dateOfBirth], [thePatient age]];
    
    if ([thePatient gender]) { 
        genderL.text = [[thePatient gender] substringWithRange:NSMakeRange(0, 1)];
    }
}


- (void)dealloc {
    [patientMainContainer release], patientMainContainer = nil;
    [medicationMainViewController release], medicationMainViewController = nil;

    [super dealloc];
}

#pragma mark  - Section Header Delegate
-(void)sectionHeaderView:(SectionHeaderView*)sectionHeaderView sectionOpened:(NSInteger)sectionOpened {
	
	SectionInfo *sectionInfo = [sectionInfoArray objectAtIndex:sectionOpened];
	
	sectionInfo.open = YES;
    
    
    NSInteger countOfRowsToInsert = [self getNumberOfRowsForGroupInSection:sectionOpened];
   
    NSMutableArray *indexPathsToInsert = [[NSMutableArray alloc] init];
    for (NSInteger i = 0; i < countOfRowsToInsert; i++) {
        [indexPathsToInsert addObject:[NSIndexPath indexPathForRow:i inSection:sectionOpened]];
    }
    
    
    UITableViewRowAnimation insertAnimation;
    
    insertAnimation = UITableViewRowAnimationTop;
    
    UITableView *tableView = self.listTV;
    
    [tableView beginUpdates];
    
    [tableView insertRowsAtIndexPaths:indexPathsToInsert withRowAnimation:insertAnimation];
    [tableView endUpdates];
    
    [indexPathsToInsert release];
}


-(void)sectionHeaderView:(SectionHeaderView*)sectionHeaderView sectionClosed:(NSInteger)sectionClosed {
    
    
	SectionInfo *sectionInfo = [sectionInfoArray objectAtIndex:sectionClosed];
	
    sectionInfo.open = NO;
    NSInteger countOfRowsToDelete = [self getNumberOfRowsForGroupInSection:sectionClosed];
    
    if (countOfRowsToDelete > 0) {
        NSMutableArray *indexPathsToDelete = [[NSMutableArray alloc] init];
        for (NSInteger i = 0; i < countOfRowsToDelete; i++) {
            [indexPathsToDelete addObject:[NSIndexPath indexPathForRow:i inSection:sectionClosed]];
        }
        [self.listTV beginUpdates];
        [self.listTV deleteRowsAtIndexPaths:indexPathsToDelete withRowAnimation:UITableViewRowAnimationTop];
        
        [self.listTV endUpdates];
        
        [indexPathsToDelete release];
    }
}
-(UIView*)tableView:(UITableView*)tableView viewForHeaderInSection:(NSInteger)section {
    
    
    SectionInfo *sectionInfo = [sectionInfoArray objectAtIndex:section];
    
    if (!sectionInfo.headerView) {
        NSString *groupName = [sectionInfo groupName];
        sectionInfo.headerView = [[[SectionHeaderView_iPhone alloc] initWithFrame:CGRectMake(0.0, 0.0, self.listTV.bounds.size.width, self.listTV.sectionHeaderHeight) title:groupName section:section delegate:self] autorelease];
        
        if (section==0) {
            sectionInfo.headerView.disclosureButton.selected = YES;
        } else {
            sectionInfo.headerView.disclosureButton.selected = NO;
        }
        
    }
    
    return sectionInfo.headerView;
    
}

#pragma mark - MedicationPanelContainerDelegate Methos
-(void)medicationPanelContainerWillAddInpatientMedication:(MedicationPanelContainer *)container {
    [super medicationPanelContainerWillAddInpatientMedication:container];
}

-(void)medicationPanelContainerWillAddOutpatientMedication:(MedicationPanelContainer *)container {
    [super medicationPanelContainerWillAddOutpatientMedication:container];
}

-(void)medicationPanelContainerWillAddNonVAMedication:(MedicationPanelContainer *)container{
    [super medicationPanelContainerWillAddNonVAMedication:container];
}

-(void)medicationPanelContainer:(MedicationPanelContainer *)container didAddInpatientMedicationAtIndex:(NSInteger)index {
    [super medicationPanelContainer:container didAddInpatientMedicationAtIndex:index];
}

-(void)medicationPanelContainer:(MedicationPanelContainer *)container didAddOutpatientMedicationAtIndex:(NSInteger)index {	
    [super medicationPanelContainer:container didAddOutpatientMedicationAtIndex:index];
}

-(void)medicationPanelContainer:(MedicationPanelContainer *)container didAddNonVAMedicationAtIndex:(NSInteger)index {	
    [super medicationPanelContainer:container didAddNonVAMedicationAtIndex:index];
}

-(void)medicationPanelContainerFinishedLoadingInpatientMedications:(MedicationPanelContainer *)container {
}

-(void)medicationPanelContainerFinishedLoadingOutpatientMedications:(MedicationPanelContainer *)container {
}

-(void)medicationPanelContainerFinishedLoadingNonVAMedications:(MedicationPanelContainer *)container {    
}

-(void)medicationPanelContainerFinishedLoadingMedications:(MedicationPanelContainer *)container  {
    [super medicationPanelContainerFinishedLoadingMedications:container];
}


#pragma mark - Initialization
-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
	
	if (self) {
        shouldRefresh = YES;
    }
	
    return self;
}

#pragma mark - View lifecycle
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];

    [self refreshTitleView];
    if (shouldRefresh) {
		[self retrieveResults];
		shouldRefresh=NO;
	}

    [listTV reloadData];
}

-(void)viewDidDissappear:(BOOL)animated {
    [self.navigationController popViewControllerAnimated:NO]; 
}

-(void)viewDidLoad {
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Feedback" 
																			   style:UIBarButtonItemStylePlain 
																			  target:[AppDelegate class] 
																			  action:@selector(launchFeedback)] autorelease];
}

#pragma mark - Orientation
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    
	if (interfaceOrientation == UIInterfaceOrientationPortrait || interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown) {
		return YES;
	}
    return NO;
}


#pragma mark - UITableViewDataSource Methods
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    numberOfSections = 3;
    return numberOfSections;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    NSInteger result = 0;
    
    
    if ([self isInpatient]) {
        
        switch (section) {
            case inpatientIndexForInpatient:
                result = self.numberOfInpatientMedications;
                break;
            case nonVAIndexForInpatient:
                result = self.numberOfNonVAMedications;
                break;
            case outpatientIndexForInpatient:
                result = self.numberOfOutpatientMedications;
                break;
                
            default:
                break;
        }
        
        
    } else {
        
        switch (section) {
            case inpatientIndexForOutpatient:
                result = self.numberOfInpatientMedications;

                break;
            case nonVAIndexForOutpatient:
                result = self.numberOfNonVAMedications;   
                
                break;
            case outpatientIndexForOutpatient:   
                result = self.numberOfOutpatientMedications;
                break;
                
            default:
                break;
        }
        
        
    }
    
    SectionInfo *sectionInfo = [sectionInfoArray objectAtIndex:section];
    
    return sectionInfo.open ?result: 0;
}   

-(void)configureCell:(UITableViewCell *)cell forTableView:(UITableView *)tableView atIndexPath:(NSIndexPath *)indexPath {
    Medication *medication;
   
    if (hasMedications) {
        
        
        medication = [self getMedicationAtIndexPath:indexPath];

        UILabel *medNameL = (UILabel *)[cell viewWithTag:1];
        UILabel *quantityL = (UILabel *)[cell viewWithTag:2];
        UILabel *lastFilledL = (UILabel *)[cell viewWithTag:3];
        UILabel *sigL = (UILabel *)[cell viewWithTag:4];
        UILabel *statusL = (UILabel *)[cell viewWithTag:5];
        
        if ([[medication status] length] > 1) {
            statusL.text =[[medication status] substringToIndex:1];
            if ([statusL.text caseInsensitiveCompare:@"A"] == NSOrderedSame) {
                statusL.textColor = [UIColor blackColor];
                statusL.font = [UIFont systemFontOfSize:17];
            } else {
                statusL.textColor = [UIColor blueColor];
                statusL.font = [UIFont boldSystemFontOfSize:17];
            }
        } else {
            statusL.text = @"";
        }
        
        medNameL.adjustsFontSizeToFitWidth = NO;
        medNameL.text = [[medication drugName] capitalizedString];
        if ([[medication medicationSource] isEqualToString:@"OUTPATIENT"]) {
            quantityL.text = [NSString stringWithFormat:@"Quantity: %@",[medication displayQuantity]];
            lastFilledL.text = [NSString stringWithFormat:@"Last Filled Date: %@",[medication lastFilledDate]];
        } else {
            quantityL.text = @"";
            lastFilledL.text = @"";
        }
        sigL.text = [NSString stringWithFormat:@"Sig: %@",[medication sig]];
        
    } else {            
        cell.textLabel.text = @"No Data Found";
        cell.contentView.backgroundColor = [UIColor colorWithRawValuesForRed:240 green:240 blue:240 alpha:1.0];
    }
    
    
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	static NSString *CellIdentifier = @"Cell";
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	
	if (cell == nil) {
        if (hasMedications) {
            [[NSBundle bundleForClass:[self class]] loadNibNamed:@"PatientSummaryMedicationsCell" owner:self options:nil];
        } else {
            self.detailTVC = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier];
		}
		
        cell = self.detailTVC;
        self.detailTVC = nil;
        
	}
	

	[self configureCell:cell forTableView:tableView atIndexPath:indexPath];
	
	return cell;
    
}    

	
-(NSString *)titleForHeaderInSection:(NSInteger)section {
    return [super titleForHeaderInSection:section];
}



#pragma mark - Table view delegate
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (hasMedications) {
        
        Medication *medication = [self getMedicationAtIndexPath:indexPath];

        if (medicationMainViewController==nil) {
			
			
			NSDictionary *tagDict = [NSDictionary dictionaryWithObjectsAndKeys:[[[[medication links] objectForKey:ResourceSelfLinkKey] href] absoluteString], 
									 @"Medication.uri", nil];

			[MetricsPusher reportMetricsWithUserEventValue:@"SelectMedicationOnMedicationViewer" tagDictionary:tagDict];
			
			medicationMainViewController = [[MedicationMain_iPhone alloc] initWithNibName:@"MedicationMain_iPhone" bundle:[NSBundle bundleForClass:[self class]]];
			[UserSession setSelectedMedication:medication];

        } else if ([UserSession selectedMedication] != medication) {
			[UserSession setSelectedMedication:medication];
		}
       
        
        [self.navigationController pushViewController:medicationMainViewController animated:YES];
        
    }
}

@end