//  patientMainViewController_iPhone.m
//  PatientViewer
//
//  Created by Matt Tea on 3/10/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "PatientMain_iPhone.h"
#import "Address.h"
#import "Allergy.h"
#import "AllergyPanel_iPhone.h"
#import "AppDelegate_iPhone.h"
#import "PatientMainTableViewWrapper.h"
#import "PatientMainCell.h"
#import "Demographics.h"
#import "LabMain_iPhone.h"
#import "LabPanel_iPhone.h"
#import "Medication.h"
#import "NextOfKin.h"
#import "Patient.h"
#import "PatientMainContainer.h"
#import "MedicationPanel_iPhone.h"
#import "PatientMain.h"
#import "Problem.h"
#import "ProblemPanel_iPhone.h"
#import "UserSession.h"
#import "MetricsPusher.h"

@implementation PatientMain_iPhone


#pragma mark - Custom Actions
-(void) drawUnderlineForLabel:(UILabel *)label {
    
    CGSize size = [label.text sizeWithFont:[label font]];
    
    CGRect underlineFrame = CGRectMake(0, 17, size.width, 1);
    
    UILabel *lineLabel = [[UILabel alloc] initWithFrame: underlineFrame];
    lineLabel.backgroundColor = [UIColor blueColor];
    [label addSubview: lineLabel];
    [lineLabel release];
}

-(void)removeUnderlines {
    [phoneL removeAllSubviews];
    [officeL removeAllSubviews];
    [cellL removeAllSubviews];
    [nextOfKinPhoneL removeAllSubviews];
}

-(void)processDidChangePatient:(NSNotification *)notification {
    [self.navigationController popToRootViewControllerAnimated:NO];
    shouldRefresh = YES;

    hasAllergySet = NO;
    hasProblemSet = NO;
    hasMedicationSet = NO;
    
    [self removeUnderlines];
    
}


-(void)restoreButtonNumberColors {
    UIColor *defaultColor = [UIColor colorWithRawValuesForRed:95 green:61 blue:18 alpha:1.0];
    
    allergyNumL.textColor = defaultColor;
    problemNumL.textColor = defaultColor;
    medicationNumL.textColor = defaultColor;
}

-(IBAction)touchOutsideButton:(id)sender{
    [self restoreButtonNumberColors];

}

-(IBAction)buttonHighlighted:(id)sender {
    int domain = [sender tag];
    
    UIColor *defaultColor = [UIColor colorWithRawValuesForRed:95 green:61 blue:18 alpha:1.0];
    
    switch (domain) {
            
        case PatientSummaryAllergyDomain:
            
            allergyNumL.textColor = [UIColor whiteColor];
            problemNumL.textColor = defaultColor;
            medicationNumL.textColor = defaultColor;
          
            break;
            
        case PatientSummaryMedicationDomain:
            
            
            allergyNumL.textColor = defaultColor;
            problemNumL.textColor = defaultColor;
            medicationNumL.textColor = [UIColor whiteColor];            
            
            break;
            
        case PatientSummaryProblemDomain:
            
            allergyNumL.textColor = defaultColor;
            problemNumL.textColor = [UIColor whiteColor];
            medicationNumL.textColor = defaultColor;            
            break;    
            
        default:
            break;
    }
    
}

-(IBAction)buttonSelected:(id)sender {
    
    int domain = [sender tag];
    
    AllergyPanel_iPhone *allergyVC;
    MedicationPanel_iPhone *medicationVC;
    ProblemPanel_iPhone *problemVC;
    
    switch (domain) {
        
            
        case PatientSummaryAllergyDomain:
            
             allergyVC = [[AllergyPanel_iPhone alloc] initWithNibName:@"AllergyPanel_iPhone" bundle:[NSBundle bundleForClass:[self class]]];
            
            if (hasAllergySet == YES) {
                
                if ([self.patientMainContainer hasAllergies]) {
                    allergyVC.hasAllergies = YES;
                    allergyVC.title = @"Allergies";
                } else {
                    allergyVC.hasAllergies = NO;                
                }
            }
        
            allergyVC.patientMainContainer = self.patientMainContainer;
            
            [self.navigationController pushViewController:allergyVC animated:YES];
            
            [allergyVC release];
            
            break;
            
        case PatientSummaryMedicationDomain:
            medicationVC = [[MedicationPanel_iPhone alloc] initWithNibName:@"MedicationPanel_iPhone" bundle:[NSBundle bundleForClass:[self class]]];
            
            if (hasMedicationSet == YES) {
                
                
				NSDate *date = [NSDate date];
				[MetricsPusher reportMetricsWithEventTitle:@"Launched Medication Viewer" startDate:date endDate:nil tagDictionary:nil];
				
                if ([self.patientMainContainer hasMedications]) {
                    medicationVC.title = @"Medications";
                    medicationVC.hasMedications = YES;
                } else {
                    medicationVC.hasMedications = NO;
                }
            
            }
           
            
            medicationVC.patientMainContainer = self.patientMainContainer;
            
            [self.navigationController pushViewController:medicationVC animated:YES];
            
            [medicationVC release];
            break;
            
        case PatientSummaryProblemDomain:
            problemVC = [[ProblemPanel_iPhone alloc] initWithNibName:@"ProblemPanel_iPhone" bundle:[NSBundle bundleForClass:[self class]]];
            
            if (hasProblemSet == YES) {
                if ([self.patientMainContainer hasProblems]) {
                    problemVC.title = @"Problems";
                    problemVC.hasProblems = YES;
                } else {
                    problemVC.hasProblems = NO;
                }
            }
            
            
            problemVC.patientMainContainer = self.patientMainContainer;
            
            [self.navigationController pushViewController:problemVC animated:YES];
            
            [problemVC release];
            break;    
            
        default:
            break;
    }
    
   
} 



-(void)makeCallToNumberString:(UITapGestureRecognizer *)gestureRecognizer {
    
    NSString *number = ((UILabel*)gestureRecognizer.view).text;
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Are you sure you want to make phone call?" message:number delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Place Call", nil];
    [alert show];
    [alert release];
    
}

- (void)alertView:(UIAlertView *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {
    if (buttonIndex == 1) {
        NSURL *phoneNumberURL = [NSURL URLWithString:[NSString stringWithFormat:@"tel:%@",actionSheet.message]];
        [[UIApplication sharedApplication] openURL:phoneNumberURL];
    }
}


#pragma mark - Domain Buttons
-(void)updateProblemButton {
    hasProblemSet = YES;
    switch ([self.patientMainContainer numberOfProblems]) {
        case 0:
                        
            [problemB setTitle:@"Problems" forState:UIControlStateNormal];
            problemNumL.text = [NSString stringWithFormat:@"%d", [self.patientMainContainer numberOfProblems]];
            problemIV.hidden = YES;
            break;
      
        case 1:
            [problemB setTitle:@"Problem" forState:UIControlStateNormal];
            problemNumL.text = [NSString stringWithFormat:@"%d", [self.patientMainContainer numberOfProblems]];
            problemIV.hidden = NO;
            break;
    
        default:
            [problemB setTitle:@"Problems" forState:UIControlStateNormal];
            problemNumL.text = [NSString stringWithFormat:@"%d", [self.patientMainContainer numberOfProblems]];
            problemIV.hidden = NO;
            break;
    }    
}

-(void)updateAllergyButton {
    hasAllergySet = YES;
    
    switch ([self.patientMainContainer numberOfAllergies]) {
        case 0:
            [allergyB setTitle:@"Allergies" forState:UIControlStateNormal];
            allergyNumL.text = [NSString stringWithFormat:@"%d", [self.patientMainContainer numberOfAllergies]];
            allergyIV.hidden = YES;
            break;
     
        case 1:
            [allergyB setTitle:@"Allergy" forState:UIControlStateNormal];
            allergyNumL.text = [NSString stringWithFormat:@"%d", [self.patientMainContainer numberOfAllergies]];
            allergyIV.hidden = NO;
            break;
     
        default:
            [allergyB setTitle:@"Allergies" forState:UIControlStateNormal];
            allergyNumL.text = [NSString stringWithFormat:@"%d", [self.patientMainContainer numberOfAllergies]];
            allergyIV.hidden = NO;
            break;
    }
}

-(void)updateMedicationButton {
    hasMedicationSet = YES;
    
    switch ([self.patientMainContainer numberOfMedications]) {
        case 0:
            [medicationB setTitle:@"Medications" forState:UIControlStateNormal];
            medicationNumL.text = [NSString stringWithFormat:@"%d", [self.patientMainContainer numberOfMedications]];
            medicationIV.hidden = YES;
            break;
       
        case 1:
            [medicationB setTitle:@"Medication" forState:UIControlStateNormal];
            medicationNumL.text = [NSString stringWithFormat:@"%d", [self.patientMainContainer numberOfMedications]];
            medicationIV.hidden = NO;
            break;
       
        default:
            [medicationB setTitle:@"Medications" forState:UIControlStateNormal];
            medicationNumL.text = [NSString stringWithFormat:@"%d", [self.patientMainContainer numberOfMedications]];
            medicationIV.hidden = NO;
            break;
    }
}


-(void)resetButtons {
    [problemB setTitle:@"Problems" forState:UIControlStateNormal];
    [allergyB setTitle:@"Allergies" forState:UIControlStateNormal];
    [medicationB setTitle:@"Medications" forState:UIControlStateNormal];
    
    problemIV.hidden = YES;
    allergyIV.hidden = YES;
    medicationIV.hidden = YES;
  
    problemNumL.text = @"-";
    allergyNumL.text = @"-";
    medicationNumL.text = @"-";
    
}


#pragma mark - Initialization
-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    
	self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
	
	if (self) {
        
        hasAllergySet = NO;
        hasProblemSet = NO;
        hasMedicationSet = NO;
        
		self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"Feedback" 
																				   style:UIBarButtonItemStylePlain 
																				  target:[AppDelegate class] 
																				  action:@selector(launchFeedback)] autorelease];
	}
	
	return self;
}


#pragma mark - Demographic Labels
-(void)updateAddressLabel {

    Demographics *demographics = [self.patientMainContainer demographics];
    
    addressL.text = @"";
    address2L.text = @"";
    
    if ([demographics address] && [[[demographics address] formattedDisplayAddress1] isEqualToString:@""] == NO) {
        addressL.text = [[[demographics address] formattedDisplayAddress1] capitalizedString];
        addressL.numberOfLines = 1;
        addressL.adjustsFontSizeToFitWidth = YES;
        
        if ([[[demographics address] formattedDisplayAddress2] isEqualToString:@""] == NO)  {
            address2L.numberOfLines = 1;
            address2L.adjustsFontSizeToFitWidth =YES;
            address2L.text = [[[demographics address] formattedDisplayAddress2] capitalizedString];
        }
    }
}

-(void)updatePhoneLabel {
    
    UITapGestureRecognizer *singletap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(makeCallToNumberString:)];

    Demographics *demographics = [self.patientMainContainer demographics];

    phoneL.textColor = [UIColor blueColor];
    phoneL.text = @"";
    if ([demographics phoneNumberHome] && [[demographics phoneNumberHome] length] > 0) {
        phoneL.text = [demographics phoneNumberHome];
        [phoneL setUserInteractionEnabled:YES];
        [self drawUnderlineForLabel:phoneL];
        [phoneL addGestureRecognizer:singletap];                    
    }				
    [singletap release];
}

-(void)updateOfficeLabel {
    UITapGestureRecognizer *singletap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(makeCallToNumberString:)];
    
    Demographics *demographics = [self.patientMainContainer demographics];

    officeL.textColor = [UIColor blueColor];

    officeL.text = @"";
    if ([demographics phoneNumberWork] && [[demographics phoneNumberWork] length] > 0) {
        officeL.text = [demographics phoneNumberWork];

        [officeL setUserInteractionEnabled:YES];
        [self drawUnderlineForLabel:officeL];
        [officeL addGestureRecognizer:singletap];
    }
    [singletap release];

}

-(void)updateCellLabel {
    
    UITapGestureRecognizer *singletap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(makeCallToNumberString:)];

    Demographics *demographics = [self.patientMainContainer demographics];

    cellL.textColor = [UIColor blueColor];
    cellL.text = @"";
    
    if ([demographics phoneNumberMobile] && [[demographics phoneNumberMobile] length] > 0) {
        cellL.text = [demographics phoneNumberMobile];

        [cellL setUserInteractionEnabled:YES];
        [self drawUnderlineForLabel:cellL];
        [cellL addGestureRecognizer:singletap];
    }
    [singletap release];

}

-(void)updateNextOfKinLabels {
    
    UITapGestureRecognizer *singletap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(makeCallToNumberString:)];
    
    Demographics *demographics = [self.patientMainContainer demographics];
    
    nextOfKinL.text = @"";
    nextOfKinPhoneL.text = @"";
    
    if ([demographics nextOfKin] && [[[demographics nextOfKin] name] length] > 0)  {
        
        if ([[[demographics nextOfKin] relationship] isEqualToString:@""] == NO) {
            nextOfKinL.text = [NSString stringWithFormat:@"%@ (%@)", [[[demographics nextOfKin] name] capitalizedString], [[[demographics nextOfKin] relationship] capitalizedString]];
        } else {
            nextOfKinL.text = [[NSString stringWithFormat:@"%@", [[demographics nextOfKin] name]] capitalizedString];
        }
        
        if ([[demographics nextOfKin] phoneNumber] && [[[demographics nextOfKin] phoneNumber] length] > 0) {
            
            nextOfKinPhoneL.textColor = [UIColor blueColor];
            nextOfKinPhoneL.text = [[demographics nextOfKin] phoneNumber];
            [self drawUnderlineForLabel:nextOfKinPhoneL];

            [nextOfKinPhoneL setUserInteractionEnabled:YES];
            
            [nextOfKinPhoneL addGestureRecognizer:singletap];
        }
    }
    [singletap release];

}

-(void)updateRoomLabel {
    Patient *thePatient = [UserSession selectedPatient];
    
    roomHeaderL.hidden = YES;
    roomL.text = @"";
    if ([[thePatient status] isEqualToString:@"true"]) {
        roomL.text = [thePatient roombed];
        roomHeaderL.hidden = NO;
    }
}

-(void)updateWardLabel {
    Patient *thePatient = [UserSession selectedPatient];
    
    wardL.text = @"";
    wardHeaderL.hidden = YES;
    if ([[thePatient status] isEqualToString:@"true"]) {
        wardL.text = [thePatient ward];
        wardHeaderL.hidden = NO;
    }
}

-(void)updateDemographics {
    [self updateAddressLabel];
    [self updatePhoneLabel];
    [self updateOfficeLabel];
    [self updateCellLabel];
    [self updateNextOfKinLabels];
    [self updateRoomLabel];
    [self updateWardLabel];
}

#pragma mark - Refresh
-(void)refreshTitleView {
    Patient *thePatient = [UserSession selectedPatient];
    
    nameL.text =    [thePatient displayName];
    ssnL.text =     [thePatient ssn];
    dobL.text =     [NSString stringWithFormat:@"%@ (%@)",[thePatient dateOfBirth], [thePatient age]];
    
    if ([thePatient gender]) { 
        genderL.text = [[thePatient gender] substringWithRange:NSMakeRange(0, 1)];
    }
}


-(void)refresh {
	
	Patient *selectedPatient = [UserSession selectedPatient];
    
	if (selectedPatient) {  
        
        [self resetButtons];

		[self.patientMainContainer refreshWithPatient:selectedPatient];
        
        [self updateDemographics];        
        
        [self refreshTitleView];
	}
    
}



#pragma mark - patientMainContainerDelegate Methods

-(void)patientMainContainerDidChangeDemographics:(PatientMainContainer *)container {
    [self updateDemographics];
}

-(void)patientMainContainerFinishedLoadingAllergies:(PatientMainContainer *)container {
    [self updateAllergyButton];
}

-(void)patientMainContainerFinishedLoadingMedications:(PatientMainContainer *)container {
    [self updateMedicationButton];
}

-(void)patientMainContainerFinishedLoadingProblems:(PatientMainContainer *)container {
    [self updateProblemButton];
}


#pragma mark - Orientation
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    
	if (interfaceOrientation == UIInterfaceOrientationPortrait || interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown) {
		return YES;
	}
    return NO;
}

#pragma mark - View Lifecycle
 -(void)viewDidLoad {
     self.navigationController.delegate = self;
     
     UINavigationBar *bar = [self.navigationController navigationBar]; 
     [bar setTintColor:[UIColor colorWithRawValuesForRed:175 green:129 blue:29 alpha:1.0]]; 
     
     [super viewDidLoad];
 }

-(void)viewWillAppear:(BOOL)animated {
    if (shouldRefresh) {
        [self refresh];
        shouldRefresh = NO;
    }
    [self restoreButtonNumberColors];
    
    [super viewWillAppear:animated];
}


@end