//  MetricsPusher.m
//  PatientViewer
//
//  Created by Grant Oladipo on 5/31/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxx

#import "ApplicationProperty.h"
#import "ConnectionFactory.h"
#import "MetricsPusher.h"
#import "MHPUser.h"
#import "UserSession.h"

@implementation MetricsPusher

-(void)reportMetricWithXMLString:(NSString *)theString {
	
	[connection release];
	connection = [[ConnectionFactory connection] retain];
	
	NSURL *postURL = [NSURL URLWithString:[ApplicationProperty auditUrlString]];
	
	[connection postWithURL:postURL withPostBodyString:theString];
}

+(NSString *)xmlStringForMetricsWithEventTitle:(NSString *)eventTitle startDate:(NSDate *)startDate endDate:(NSDate *)endDate tagDictionary:(NSDictionary *)tagDict {
	
	static NSDateFormatter *dateFormatter = nil;
	if (dateFormatter == nil) {
		dateFormatter = [[NSDateFormatter alloc] init];
	}
	[dateFormatter setDateFormat:@"MM/dd/YYYY HH:mm:ss"];
	
	NSString *startDateString = [dateFormatter stringFromDate:startDate];
	NSString *endDateString = [dateFormatter stringFromDate:endDate];
	NSMutableString *xmlString = [[[NSMutableString alloc] initWithCapacity:10] autorelease];
	
	int timingDelta = [endDate timeIntervalSinceDate:startDate]*1000+0.5;
	
	[xmlString appendFormat:@"<?xml version=\"1.0\"?><m:timing xmlns:m=\"http://healthcare.agilex.com/mobilehealthplatform\" event=\"%@\" startDate=\"%@\" endDate=\"%@\" timing=\"%d\">", eventTitle, startDateString, endDateString, timingDelta];

	for (NSString *key in [tagDict allKeys]) {
		[xmlString appendFormat:@"<tag value=\"%@\" key=\"%@\"/>", [tagDict valueForKey:key], key];
	}
	
	
	[xmlString appendFormat:@"<tag value=\"%@\" key=\"%@\"/>", [ApplicationProperty applicationName], @"Application.name"];
	[xmlString appendFormat:@"<tag value=\"%@\" key=\"%@\"/>", [ApplicationProperty applicationBuildVersion], @"Application.version"];
	[xmlString appendFormat:@"<tag value=\"%@\" key=\"%@\"/>", [UserSession sessionId], @"MHPSessionId"];
	[xmlString appendFormat:@"<tag value=\"%@\" key=\"%@\"/>", [[UserSession mhpUser] displayName], @"User.name"];
	[xmlString appendFormat:@"<tag value=\"%@\" key=\"%@\"/>", [[UserSession mhpUser] userId], @"User.id"];
	[xmlString appendFormat:@"<tag value=\"%@\" key=\"%@\"/>", [[UIDevice currentDevice] uniqueIdentifier], @"Device.id"];
	[xmlString appendFormat:@"<tag value=\"%@\" key=\"%@\"/>", [[UIDevice currentDevice] model], @"Device.model"];
	[xmlString appendFormat:@"<tag value=\"%@\" key=\"%@\"/>", [[UIDevice currentDevice] systemVersion], @"Device.os"];
	[xmlString appendString:@"</m:timing>"];
	
	return xmlString;
}

+(void)reportMetricsWithEventTitle:(NSString *)eventTitle startDate:(NSDate *)startDate endDate:(NSDate *)endDate tagDictionary:(NSDictionary *)tagDict {
	
	MetricsPusher *metricsPusher = [[[MetricsPusher alloc] init] autorelease];
	
	NSString *xmlString = [self xmlStringForMetricsWithEventTitle:eventTitle startDate:startDate endDate:endDate tagDictionary:tagDict];
	[metricsPusher reportMetricWithXMLString:xmlString];
}

+(void)reportMetricsWithUserEventValue:(NSString *)value tagDictionary:(NSDictionary *)tagDict {
	
	NSDate *startDate = [NSDate date];
	
	NSMutableDictionary *tagDictionary = [NSMutableDictionary dictionaryWithDictionary:tagDict];
	[tagDictionary setValue:value forKey:@"userEvent"];
	
	[self reportMetricsWithEventTitle:@"iOS.userEvent" startDate:startDate endDate:nil tagDictionary:tagDictionary];
}

@end