//
//  MicroLabResultParser.m
//  PatientViewer
//
//  Created by Tea, Matt on 10/4/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "MicroLabResultParser.h"
#import "LabResult.h"
#import "LabResultGroup.h"
#import "Link.h"
#import "ManagerUtility.h"

@implementation MicroLabResultParser


#pragma mark - Custom Actions
-(void)parseData:(NSData *)data {
	
	[xmlParser release];
	xmlParser = [[NSXMLParser alloc] initWithData:data];
	[xmlParser setDelegate:self];
	[xmlParser setShouldProcessNamespaces:YES];
	
	if (![xmlParser parse]) {
		NSLog(@"***Parsing UNSUCCESSFUL!");
		NSLog(@"Tried to parse:\n%@", [ManagerUtility prettyPrintXML:[[[NSString alloc] initWithData:data encoding:NSStringEncodingConversionAllowLossy] autorelease]]);
	}
}


#pragma mark - Initialization
-(id)initWithDelegate:(id)theDelegate {
	
	self = [super init];
	
	if (self) {
		delegate = [theDelegate retain];
	}
	
	return self;
}


#pragma mark - Memory Management
-(void)dealloc {
	
	[delegate release], delegate = nil;
	[xmlParser release], xmlParser = nil;
	
	[super dealloc];
}

#pragma mark - NSXMLParserDelegate
-(void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	
	if ([elementName isEqualToString:@"link"]) {
		[parentElement addLink:link];
	} else if ([elementName isEqualToString:@"labResult"]) {
		
		//parentElement = labTest;
		[delegate addLabResult:labResult toGroup:labResultGroup];
		
	} else if ([elementName isEqualToString:@"labTest"]) {
		
		parentElement = delegate;
	} 
}

-(void)parserDidEndDocument:(NSXMLParser *)parser {
	
	[delegate parserFinishedCreatingResource:self];
}

-(void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {	
	
	if ([elementName isEqualToString:@"labTests"]) {
		
		parentElement = delegate;
        [delegate setTotalNumberofLabResultsGroups:[[attributeDict objectForKey:@"size"] integerValue]];
		
	} else if ([elementName isEqualToString:@"link"]) {
		
		[link release];
		link = [[Link alloc] initWithAttributeDictionary:attributeDict];
		
	} else if ([elementName isEqualToString:@"labTest"]) {
		
		[labResultGroup release];
		labResultGroup = [[LabResultGroup alloc] initWithAttributeDictionary:attributeDict];
		[delegate addLabResultGroup:labResultGroup];

		parentElement = labResultGroup;
        
		
	} else if ([elementName isEqualToString:@"labResult"]) {
		
		[labResult release];
		labResult = [[LabResult alloc] initWithAttributeDictionary:attributeDict];
	}
}

@end
