//  PatientMainViewController_iPad.m
//  PatientViewer
//
//  Created by Matt Tea on 3/10/11.
//  xxxxxxxxx xxxx xxxxxxx xxx xxxxxx xxxxxxxxxxxxxxxxxxxxxx

#import "Address.h"
#import "AppDelegate_iPad.h"
#import "Allergy.h"
#import "Alerter.h"
#import "PatientMainTableViewWrapper.h"
#import "Demographics.h"
#import "LabMain_iPad.h"
#import "LabPanel_iPad.h"
#import "Medication.h"
#import "NextOfKin.h"
#import "PatientMainContainer.h"
#import "PatientMain_iPad.h"
#import "Problem.h"
#import "Patient.h"
#import "UserSession.h"


@implementation PatientMain_iPad

@synthesize clinicalDataV;
@synthesize customHeaderV;
@synthesize customTVC;
@synthesize fullNameL;
@synthesize patientDataSV;
@synthesize patientDataV;
@synthesize patientIV;

@synthesize ssnL;
@synthesize dobL; 
@synthesize genderL;
@synthesize addressL1;
@synthesize addressL2;
@synthesize phoneL;
@synthesize officeL;
@synthesize cellL;
@synthesize nextOfKinL1;
@synthesize nextOfKinL2;
@synthesize roomL;
@synthesize wardL;
@synthesize roomHeader;
@synthesize wardHeader;

///*************************************************************************/
////PROOF OF CONCEPT TO BE REMOVED
//-(void)goToThirdPartyApp:(UITapGestureRecognizer *)tapGestureRecognizer {
//	
//	UIApplication *application = [UIApplication sharedApplication];
//	Patient *patient = [UserSession selectedPatient];
//
//	NSString *urlString = [NSString stringWithFormat:@"vapv1://?patientId=%@", [patient ssn]];
//	urlString = [urlString stringByAddingPercentEscapesUsingEncoding:NSStringEncodingConversionAllowLossy];
//
//	NSURL *thirdPartyAppURL = [NSURL URLWithString:urlString];
//	
//	if ([application canOpenURL:thirdPartyAppURL]) {
//		[application openURL:thirdPartyAppURL];
//	} else {
//		[Alerter enqueueAlertViewWithTitle:@"Cannot Open URL" message:urlString];
//	}
//}
///*************************************************************************/

#pragma mark - Custom Actions
-(UITableView *)createTableViewAtPosition:(NSUInteger)position {
	
	UITableView *tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, ClinicalDataTableWidth, InitialTableViewHeight) style:UITableViewStylePlain];
	tableView.delegate = self;
	tableView.dataSource = self;
	tableView.backgroundView = nil;
	tableView.backgroundColor = [UIColor clearColor];
	tableView.alwaysBounceVertical = NO;
	tableView.alwaysBounceHorizontal = NO;
	tableView.scrollEnabled  = NO;
	tableView.autoresizesSubviews = YES;
	tableView.separatorColor = [UIColor colorWithRawValuesForRed:203 green:165 blue:109 alpha:1];
	tableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
	tableView.layer.borderColor = [UIColor colorWithRawValuesForRed:203 green:165 blue:109 alpha:1].CGColor;
	tableView.layer.borderWidth = 1;
	tableView.layer.cornerRadius = 5;
    
	[clinicalDataV addTableView:tableView position:position];
	
	patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
										   InitialScrollViewContentHeight+clinicalDataV.frame.size.height);
	
    return [tableView autorelease];
}

-(void)createAllergyTableView {
	allergyTV = [[self createTableViewAtPosition:1] retain];
}

-(void)createInpatientMedicationTableView {
	inpatientMedicationTV = [[self createTableViewAtPosition:2] retain];
}

-(void)createOutpatientMedicationTableView {
	outpatientMedicationTV = [[self createTableViewAtPosition:3] retain];
}

-(void)createNonVAMedicationTableView {
	nonVAMedicationTV = [[self createTableViewAtPosition:4] retain];
}

-(void)createProblemTableView {
	problemTV = [[self createTableViewAtPosition:0] retain];
}

-(void)createTableViews {
	
	[self createAllergyTableView];
	[self createInpatientMedicationTableView];
    [self createOutpatientMedicationTableView];
    [self createNonVAMedicationTableView];
	[self createProblemTableView];
}

-(void)hideDataViewElements {
	
	patientDataV.hidden = YES;
	self.fullNameL.text = @"";
}

-(void)showDataViewElements {
	
	patientDataV.hidden = NO;
	[patientDataSV scrollRectToVisible:CGRectMake(0, 0, 10, 10) animated:YES];
}

-(void)refreshDemographicData {
	
	Demographics *demographics = [self.patientMainContainer demographics];
	Patient *thePatient = [self.patientMainContainer patient];

	self.ssnL.text = [thePatient ssn];
	
	NSString *dateOfBirth = [thePatient dateOfBirth];
	self.dobL.text = @"";

	if (dateOfBirth) {
		self.dobL.text = [NSString stringWithFormat:@"%@ (%@)", dateOfBirth, [thePatient age]];
	}

	self.genderL.text = [thePatient gender];

	Address *address = [demographics address];
	self.addressL1.text = @"";
	self.addressL2.text = @"";

	if (address) {
		self.addressL1.text = [address formattedDisplayAddress1];
		self.addressL2.text = [address formattedDisplayAddress2];
	}

	self.phoneL.text = [demographics phoneNumberHome];
	self.officeL.text = [demographics phoneNumberWork];
	self.cellL.text = [demographics phoneNumberMobile];

	NextOfKin *nextOfKin = [demographics nextOfKin];
	self.nextOfKinL1.text = @"";
	self.nextOfKinL2.text = @"";

	if (nextOfKin) {

		self.nextOfKinL1.text = [nextOfKin name];
		
		if ([[nextOfKin relationship] isEqualToString:@""] == NO) {
			self.nextOfKinL1.text = [NSString stringWithFormat:@"%@ (%@)", self.nextOfKinL1.text, [nextOfKin relationship]];
		}
		
		self.nextOfKinL2.text = [nextOfKin phoneNumber];
	}
    
    self.roomL.text = @"";
    self.wardL.text = @"";
    self.wardHeader.hidden = YES;
    self.roomHeader.hidden = YES;

    if ([[thePatient status] isEqualToString:@"true"]) {
        self.wardHeader.hidden = NO;
        self.roomHeader.hidden = NO;
        self.roomL.text = [thePatient roombed];
        self.wardL.text = [thePatient ward];
    }

}

-(void)refresh {
	
	Patient *selectedPatient = [UserSession selectedPatient];
	
	if (selectedPatient) {
	
		self.fullNameL.text = [[selectedPatient displayName] capitalizedString];
		
		[self.patientMainContainer removeAllObjects];
		
		dataRefreshFinished = NO;
		
		allergyTV.frame = CGRectMake(0, 0, ClinicalDataTableWidth, InitialTableViewHeight);
		inpatientMedicationTV.frame = CGRectMake(0, 0, ClinicalDataTableWidth, InitialTableViewHeight);
		outpatientMedicationTV.frame = CGRectMake(0, 0, ClinicalDataTableWidth, InitialTableViewHeight);
		nonVAMedicationTV.frame = CGRectMake(0, 0, ClinicalDataTableWidth, InitialTableViewHeight);
        problemTV.frame = CGRectMake(0, 0, ClinicalDataTableWidth, InitialTableViewHeight);
		[self.clinicalDataV refreshTableViews];
		patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
											   InitialScrollViewContentHeight+clinicalDataV.frame.size.height);
		
		[self refreshDemographicData];
		[allergyTV reloadData];
		[inpatientMedicationTV reloadData];
        [outpatientMedicationTV reloadData];
		[nonVAMedicationTV reloadData];
		[problemTV reloadData];
		self.patientIV.image = nil;
		
		[self showDataViewElements];

		[self.patientMainContainer refreshWithPatient:selectedPatient];        
		
	} else {
		[self hideDataViewElements];
	}
}

-(void)processDidChangePatient:(NSNotification *)notification {
	[self refresh];
}


#pragma mark - Initialization
-(id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
	
	self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
	
	if (self) {

		self.title = @"Patient Summary";
		
		self.patientMainContainer = [[[PatientMainContainer alloc] initWithDelegate:self] autorelease];
	}
	
	return self;
}


#pragma mark - Memory Management
- (void)dealloc {
	
	[allergyTV release], allergyTV = nil;
	[inpatientMedicationTV release], inpatientMedicationTV = nil;
	[outpatientMedicationTV release], outpatientMedicationTV = nil;
	[nonVAMedicationTV release], nonVAMedicationTV = nil;
	[problemTV release], problemTV = nil;
	
	[super dealloc];
}


#pragma mark - Orientation
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return UIInterfaceOrientationIsLandscape(interfaceOrientation);
}


#pragma mark - PatientMainContainerDelegate Method
-(void)patientMainContainerDidChangeDemographics:(PatientMainContainer *)container {
	[self refreshDemographicData];
}

-(void)patientMainContainerWillChangePatientImage:(PatientMainContainer *)container {
}

-(void)patientMainContainerDidChangePatientImage:(PatientMainContainer *)container {
	self.patientIV.image = [self.patientMainContainer patientImage];	
}

-(void)patientMainContainerWillAddAllergy:(PatientMainContainer *)container {
	[allergyTV beginUpdates];
}

-(void)patientMainContainer:(PatientMainContainer *)container didAddAllergyAtIndex:(NSInteger)index {
	
	allergyTV.frame = CGRectMake(allergyTV.frame.origin.x, 
								 allergyTV.frame.origin.y, 
								 ClinicalDataTableWidth, 
								 allergyTV.frame.size.height+HeightForClinicalDataRow);
	
	[self.clinicalDataV refreshTableViews];
	patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
										   patientDataSV.contentSize.height+HeightForClinicalDataRow);

	
	[allergyTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:index inSection:0]] 
					 withRowAnimation:UITableViewRowAnimationTop];
	
	[allergyTV endUpdates];
}

-(void)patientMainContainerFinishedLoadingAllergies:(PatientMainContainer *)container {

	dataRefreshFinished = YES;
	
	if (![self.patientMainContainer hasAllergies]) {
		
		[allergyTV beginUpdates];
		
		allergyTV.frame = CGRectMake(allergyTV.frame.origin.x, allergyTV.frame.origin.y, ClinicalDataTableWidth, allergyTV.frame.size.height+HeightForClinicalDataRow);
		[self.clinicalDataV refreshTableViews];
		patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
											   patientDataSV.contentSize.height+HeightForClinicalDataRow);
		
		[allergyTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:0 inSection:0]] withRowAnimation:UITableViewRowAnimationTop];
		
		[allergyTV endUpdates];
	}
}

-(void)patientMainContainerWillAddInpatientMedicationTV:(PatientMainContainer *)container {
	[inpatientMedicationTV beginUpdates];
}

-(void)patientMainContainer:(PatientMainContainer *)container didAddInpatientMedicationAtIndex:(NSInteger)index {
	
	inpatientMedicationTV.frame = CGRectMake(inpatientMedicationTV.frame.origin.x, inpatientMedicationTV.frame.origin.y, ClinicalDataTableWidth, inpatientMedicationTV.frame.size.height+HeightForClinicalDataRow);
	[self.clinicalDataV refreshTableViews];
	patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
										   patientDataSV.contentSize.height+HeightForClinicalDataRow);
	
	[inpatientMedicationTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:index inSection:0]] 
					 withRowAnimation:UITableViewRowAnimationTop];
	
	[inpatientMedicationTV endUpdates];
}

-(void)patientMainContainerFinishedLoadingInpatientMedications:(PatientMainContainer *)container {
	
	dataRefreshFinished = YES;
	
	if (![self.patientMainContainer hasInpatientMedications]) {
		
		[inpatientMedicationTV beginUpdates];
		
		inpatientMedicationTV.frame = CGRectMake(inpatientMedicationTV.frame.origin.x, inpatientMedicationTV.frame.origin.y, ClinicalDataTableWidth, inpatientMedicationTV.frame.size.height+HeightForClinicalDataRow);
		[self.clinicalDataV refreshTableViews];
		patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
											   patientDataSV.contentSize.height+HeightForClinicalDataRow);
		
		[inpatientMedicationTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:0 inSection:0]] withRowAnimation:UITableViewRowAnimationTop];
		
		[inpatientMedicationTV endUpdates];
	}
}


-(void)patientMainContainerWillAddOutpatientMedication:(PatientMainContainer *)container {
	[outpatientMedicationTV beginUpdates];
}


-(void)patientMainContainer:(PatientMainContainer *)container didAddOutpatientMedicationAtIndex:(NSInteger)index {
	
	outpatientMedicationTV.frame = CGRectMake(outpatientMedicationTV.frame.origin.x, outpatientMedicationTV.frame.origin.y, ClinicalDataTableWidth, outpatientMedicationTV.frame.size.height+HeightForClinicalDataRow);
	[self.clinicalDataV refreshTableViews];
	patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
										   patientDataSV.contentSize.height+HeightForClinicalDataRow);
	
	[outpatientMedicationTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:index inSection:0]] 
                        withRowAnimation:UITableViewRowAnimationTop];
	
	[outpatientMedicationTV endUpdates];
}

-(void)patientMainContainerFinishedLoadingOutpatientMedications:(PatientMainContainer *)container {
	
	dataRefreshFinished = YES;
	
	if (![self.patientMainContainer hasOutpatientMedications]) {
		
		[outpatientMedicationTV beginUpdates];
		
		outpatientMedicationTV.frame = CGRectMake(outpatientMedicationTV.frame.origin.x, outpatientMedicationTV.frame.origin.y, ClinicalDataTableWidth, outpatientMedicationTV.frame.size.height+HeightForClinicalDataRow);
		[self.clinicalDataV refreshTableViews];
		patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
											   patientDataSV.contentSize.height+HeightForClinicalDataRow);
		
		[outpatientMedicationTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:0 inSection:0]] withRowAnimation:UITableViewRowAnimationTop];
		
		[outpatientMedicationTV endUpdates];
	}
}

-(void)patientMainContainerWillAddNonVAMedication:(PatientMainContainer *)container {
	[nonVAMedicationTV beginUpdates];
}


-(void)patientMainContainer:(PatientMainContainer *)container didAddNonVAMedicationAtIndex:(NSInteger)index {
	
	nonVAMedicationTV.frame = CGRectMake(nonVAMedicationTV.frame.origin.x, nonVAMedicationTV.frame.origin.y, ClinicalDataTableWidth, nonVAMedicationTV.frame.size.height+HeightForClinicalDataRow);
	[self.clinicalDataV refreshTableViews];
	patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
										   patientDataSV.contentSize.height+HeightForClinicalDataRow);
	
	[nonVAMedicationTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:index inSection:0]] 
                             withRowAnimation:UITableViewRowAnimationTop];
	
	[nonVAMedicationTV endUpdates];
}

-(void)patientMainContainerFinishedLoadingNonVAMedications:(PatientMainContainer *)container {
	
	dataRefreshFinished = YES;
	
	if (![self.patientMainContainer hasNonVAMedications]) {
		
		[nonVAMedicationTV beginUpdates];
		
		nonVAMedicationTV.frame = CGRectMake(nonVAMedicationTV.frame.origin.x, nonVAMedicationTV.frame.origin.y, ClinicalDataTableWidth, nonVAMedicationTV.frame.size.height+HeightForClinicalDataRow);
		[self.clinicalDataV refreshTableViews];
		patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
											   patientDataSV.contentSize.height+HeightForClinicalDataRow);
		[nonVAMedicationTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:0 inSection:0]] withRowAnimation:UITableViewRowAnimationTop];
		
		[nonVAMedicationTV endUpdates];
	}
}


-(void)patientMainContainerWillAddProblem:(PatientMainContainer *)container {
	[problemTV beginUpdates];
}

-(void)patientMainContainer:(PatientMainContainer *)container didAddProblemAtIndex:(NSInteger)index {
	
	problemTV.frame = CGRectMake(problemTV.frame.origin.x, problemTV.frame.origin.y, ClinicalDataTableWidth, problemTV.frame.size.height+HeightForClinicalDataRow);
	[self.clinicalDataV refreshTableViews];
	patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
										   patientDataSV.contentSize.height+HeightForClinicalDataRow);
	
	[problemTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:index inSection:0]] 
					 withRowAnimation:UITableViewRowAnimationTop];

	[problemTV endUpdates];
}

-(void)patientMainContainerFinishedLoadingProblems:(PatientMainContainer *)container {

	dataRefreshFinished = YES;
	
	if (![self.patientMainContainer hasProblems]) {
		
		[problemTV beginUpdates];
		
		problemTV.frame = CGRectMake(problemTV.frame.origin.x, problemTV.frame.origin.y, ClinicalDataTableWidth, problemTV.frame.size.height+HeightForClinicalDataRow);
		[self.clinicalDataV refreshTableViews];
		patientDataSV.contentSize = CGSizeMake(ScrollViewContentWidth, 
											   patientDataSV.contentSize.height+HeightForClinicalDataRow);
		
		[problemTV insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:0 inSection:0]] withRowAnimation:UITableViewRowAnimationTop];
		
		[problemTV endUpdates];
	}
}


#pragma mark - UITableViewDataSource Methods
-(void)configureCell:(UITableViewCell *)cell forTableView:(UITableView *)tableView atIndexPath:(NSIndexPath *)indexPath {

	static NSString *NoDataFoundString = @"No Data Found";
	
	if (tableView == allergyTV) {
		
		Allergy *allergy = [self.patientMainContainer allergyAtIndex:indexPath.row];

		UILabel *allergySubstanceL = (UILabel *)[cell viewWithTag:1];
		UILabel *allergyReactionL = (UILabel *)[cell viewWithTag:2];
		
		if (allergy) {
			allergySubstanceL.text = [[allergy substance] capitalizedString];
			allergyReactionL.text = [[allergy reaction] capitalizedString];
		} else {
			allergySubstanceL.text = NoDataFoundString;
			allergyReactionL.text = @"";
		}
		
	} else if (tableView == inpatientMedicationTV) {
		
		Medication *medication = [self.patientMainContainer inpatientMedicationAtIndex:indexPath.row];
        
		UILabel *medicationNameL = (UILabel *)[cell viewWithTag:1];
        UILabel *sigL = (UILabel *)[cell viewWithTag:2];
        UILabel *statusL = (UILabel *)[cell viewWithTag:3];
         
		if (medication) {

			medicationNameL.text = [[medication drugName] capitalizedString];
			sigL.text = [[medication sig] capitalizedString];
            if ([[medication status] length] > 1) {
                statusL.text =[[medication status] substringToIndex:1];
                if ([statusL.text caseInsensitiveCompare:@"A"] == NSOrderedSame) {
                    statusL.textColor = [UIColor blackColor];
                    statusL.font = [UIFont systemFontOfSize:16];
                } else {
                    statusL.textColor = [UIColor blueColor];
                    statusL.font = [UIFont boldSystemFontOfSize:16];
                }
            }
        } else {

			medicationNameL.text = NoDataFoundString;
			sigL.text = @"";
            statusL.text = @"";
		}
		
	} else if (tableView == outpatientMedicationTV) {
		
		Medication *medication = [self.patientMainContainer outpatientMedicationAtIndex:indexPath.row];
		
		UILabel *medicationNameL = (UILabel *)[cell viewWithTag:1];
        UILabel *sigL = (UILabel *)[cell viewWithTag:2];
        UILabel *statusL = (UILabel *)[cell viewWithTag:3];
        
		if (medication) {
			medicationNameL.text = [[medication drugName] capitalizedString];
			sigL.text = [[medication sig] capitalizedString];
            if ([[medication status] length] > 1) {
                statusL.text =[[medication status] substringToIndex:1];
                if ([statusL.text caseInsensitiveCompare:@"A"] == NSOrderedSame) {
                    statusL.textColor = [UIColor blackColor];
                    statusL.font = [UIFont systemFontOfSize:16];
                } else {
                    statusL.textColor = [UIColor blueColor];
                    statusL.font = [UIFont boldSystemFontOfSize:16];
                }
            }
        } else {
			medicationNameL.text = NoDataFoundString;
			sigL.text = @"";
            statusL.text = @"";
		}
		
	} else if (tableView == nonVAMedicationTV) {
        
		Medication *medication = [self.patientMainContainer nonVAMedicationAtIndex:indexPath.row];

		UILabel *medicationNameL = (UILabel *)[cell viewWithTag:1];
        UILabel *sigL = (UILabel *)[cell viewWithTag:2];
        UILabel *statusL = (UILabel *)[cell viewWithTag:3];
        
		if (medication) {
			medicationNameL.text = [[medication drugName] capitalizedString];
			sigL.text = [[medication sig] capitalizedString];
            if ([[medication status] length] > 1) {
                statusL.text =[[medication status] substringToIndex:1];
                if ([statusL.text caseInsensitiveCompare:@"A"] == NSOrderedSame) {
                    statusL.textColor = [UIColor blackColor];
                    statusL.font = [UIFont systemFontOfSize:16];
                } else {
                    statusL.textColor = [UIColor blueColor];
                    statusL.font = [UIFont boldSystemFontOfSize:16];
                }
            }
        } else {
			medicationNameL.text = NoDataFoundString;
			sigL.text = @"";
            statusL.text = @"";
		}
		
	} else if (tableView == problemTV) {
		
		Problem *problem = [self.patientMainContainer problemAtIndex:indexPath.row];
		
		UILabel *problemDescriptionL = (UILabel *)[cell viewWithTag:1];		
		
		if (problem) {
			problemDescriptionL.text = [[problem description] capitalizedString];					
		} else {
			problemDescriptionL.text = NoDataFoundString;
		}
	}	
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	
	int numberOfRows = 0;
	
	if (tableView == allergyTV) {
		numberOfRows = [self.patientMainContainer numberOfAllergies];
	} else if (tableView == inpatientMedicationTV) {
		numberOfRows = [self.patientMainContainer numberOfInpatientMedications];
	} else if (tableView == outpatientMedicationTV) {
		numberOfRows = [self.patientMainContainer numberOfOutpatientMedications];
	} else if (tableView == nonVAMedicationTV) {
		numberOfRows = [self.patientMainContainer numberOfNonVAMedications];
	} else if (tableView == problemTV) {
		numberOfRows = [self.patientMainContainer numberOfProblems];
	}
	
	if (dataRefreshFinished && numberOfRows==0) {
		numberOfRows = 1;
	} 
	
    
    
	return numberOfRows;
}

-(NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
	
	if (tableView == allergyTV) {
		return @"Allergies";
	} else if (tableView == inpatientMedicationTV) {
		return @"Inpatient Medications (Active and Recently Expired)";
	} else if (tableView == outpatientMedicationTV) {
		return @"Outpatient Medications (Active and Recently Expired)";
	} else if (tableView == nonVAMedicationTV) {
		return @"Non-VA Medications (Active and Recently Expired)";
	} else if (tableView == problemTV) {
		return @"Problems (Active)";
	}
	
	return nil;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	static NSString *CellIdentifier = @"Cell";
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	
	if (cell == nil) {
		
		if (tableView == allergyTV) {
			[[NSBundle bundleForClass:[self class]] loadNibNamed:@"AllergyCell" owner:self options:nil];
		} else if (tableView == inpatientMedicationTV || tableView == outpatientMedicationTV || tableView == nonVAMedicationTV) {
			[[NSBundle bundleForClass:[self class]] loadNibNamed:@"MedicationCell" owner:self options:nil];
		} else if (tableView == problemTV) {
			[[NSBundle bundleForClass:[self class]] loadNibNamed:@"ProblemCell" owner:self options:nil];
		} 
		
		cell = self.customTVC;
		self.customTVC = nil;
	}
	
	[self configureCell:cell forTableView:tableView atIndexPath:indexPath];
	
	return cell;
}


#pragma mark - UITableViewDelegate Methods
-(CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {	
	return HeightForClinicalDataSectionTitle;
 }

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
	return HeightForClinicalDataRow;
}

-(UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
		
	NSString *sectionHeaderText = [self tableView:tableView titleForHeaderInSection:section];
	
	if (!sectionHeaderText) {
		return nil;
	}
	
	[[NSBundle bundleForClass:[self class]] loadNibNamed:@"Header" owner:self options:nil];
		
	UIView *sectionHeader = self.customHeaderV;
	UILabel *sectionHeaderL = (UILabel *)[sectionHeader viewWithTag:1];
	sectionHeaderL.text = sectionHeaderText;
	self.customHeaderV = nil;
	
	return sectionHeader;
}


#pragma mark - View Lifecycle
-(void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
	
   // [self refresh];
}

-(void)viewDidLoad {
	
	[super viewDidLoad];
	
//	/*************************************************************************/
//	//PROOF OF CONCEPT TO BE REMOVED	
//	UITapGestureRecognizer *tapGR = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(goToThirdPartyApp:)];
//	[self.patientIV addGestureRecognizer:tapGR];
//	[tapGR release];
//	/*************************************************************************/
	
	[self createTableViews];
	
	[self hideDataViewElements];
}

-(void)viewDidUnload {
	
	[clinicalDataV release], clinicalDataV = nil;
	[customHeaderV release], customHeaderV = nil;
	[customTVC release], customTVC = nil;
	[fullNameL release], fullNameL = nil;
	[patientDataSV release], patientDataSV = nil;
	[patientDataV release], patientDataV = nil;
	[patientIV release], patientIV = nil;
	
	[super viewDidUnload];
}

@end